@@ -6,6 +6,7 @@ use crate::outgoing_message::OutgoingError;
 use crate::outgoing_message::OutgoingMessage;
 use codex_app_server_protocol::JSONRPCErrorError;
 use codex_app_server_protocol::JSONRPCMessage;
+use codex_app_server_protocol::ServerRequest;
 use futures::SinkExt;
 use futures::StreamExt;
 use owo_colors::OwoColorize;
@@ -30,8 +31,10 @@ use tokio::net::TcpListener;
 use tokio::net::TcpStream;
 use tokio::sync::mpsc;
 use tokio::task::JoinHandle;
-use tokio_tungstenite::accept_async;
+use tokio_tungstenite::accept_async_with_config;
 use tokio_tungstenite::tungstenite::Message as WebSocketMessage;
+use tokio_tungstenite::tungstenite::protocol::WebSocketConfig;
+use tokio_util::sync::CancellationToken;
 use tracing::debug;
 use tracing::error;
 use tracing::info;
@@ -66,12 +69,6 @@ fn print_websocket_startup_banner(addr: SocketAddr) {
     }
 }
 
-#[allow(clippy::print_stderr)]
-fn print_websocket_connection(peer_addr: SocketAddr) {
-    let connected_label = colorize("websocket client connected from", Style::new().dimmed());
-    eprintln!("{connected_label} {peer_addr}");
-}
-
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub enum AppServerTransport {
     Stdio,
@@ -135,6 +132,7 @@ pub(crate) enum TransportEvent {
     ConnectionOpened {
         connection_id: ConnectionId,
         writer: mpsc::Sender<OutgoingMessage>,
+        disconnect_sender: Option<CancellationToken>,
     },
     ConnectionClosed {
         connection_id: ConnectionId,
@@ -147,6 +145,7 @@ pub(crate) enum TransportEvent {
 
 pub(crate) struct ConnectionState {
     pub(crate) outbound_initialized: Arc<AtomicBool>,
+    pub(crate) outbound_experimental_api_enabled: Arc<AtomicBool>,
     pub(crate) outbound_opted_out_notification_methods: Arc<RwLock<HashSet<String>>>,
     pub(crate) session: ConnectionSessionState,
 }
@@ -154,10 +153,12 @@ pub(crate) struct ConnectionState {
 impl ConnectionState {
     pub(crate) fn new(
         outbound_initialized: Arc<AtomicBool>,
+        outbound_experimental_api_enabled: Arc<AtomicBool>,
         outbound_opted_out_notification_methods: Arc<RwLock<HashSet<String>>>,
     ) -> Self {
         Self {
             outbound_initialized,
+            outbound_experimental_api_enabled,
             outbound_opted_out_notification_methods,
             session: ConnectionSessionState::default(),
         }
@@ -166,20 +167,36 @@ impl ConnectionState {
 
 pub(crate) struct OutboundConnectionState {
     pub(crate) initialized: Arc<AtomicBool>,
+    pub(crate) experimental_api_enabled: Arc<AtomicBool>,
     pub(crate) opted_out_notification_methods: Arc<RwLock<HashSet<String>>>,
     pub(crate) writer: mpsc::Sender<OutgoingMessage>,
+    disconnect_sender: Option<CancellationToken>,
 }
 
 impl OutboundConnectionState {
     pub(crate) fn new(
         writer: mpsc::Sender<OutgoingMessage>,
         initialized: Arc<AtomicBool>,
+        experimental_api_enabled: Arc<AtomicBool>,
         opted_out_notification_methods: Arc<RwLock<HashSet<String>>>,
+        disconnect_sender: Option<CancellationToken>,
     ) -> Self {
         Self {
             initialized,
+            experimental_api_enabled,
             opted_out_notification_methods,
             writer,
+            disconnect_sender,
+        }
+    }
+
+    fn can_disconnect(&self) -> bool {
+        self.disconnect_sender.is_some()
+    }
+
+    pub(crate) fn request_disconnect(&self) {
+        if let Some(disconnect_sender) = &self.disconnect_sender {
+            disconnect_sender.cancel();
         }
     }
 }
@@ -195,6 +212,7 @@ pub(crate) async fn start_stdio_connection(
         .send(TransportEvent::ConnectionOpened {
             connection_id,
             writer: writer_tx,
+            disconnect_sender: None,
         })
         .await
         .map_err(|_| std::io::Error::new(ErrorKind::BrokenPipe, "processor unavailable"))?;
@@ -254,6 +272,7 @@ pub(crate) async fn start_stdio_connection(
 pub(crate) async fn start_websocket_acceptor(
     bind_address: SocketAddr,
     transport_event_tx: mpsc::Sender<TransportEvent>,
+    shutdown_token: CancellationToken,
 ) -> IoResult<JoinHandle<()>> {
     let listener = TcpListener::bind(bind_address).await?;
     let local_addr = listener.local_addr()?;
@@ -263,23 +282,31 @@ pub(crate) async fn start_websocket_acceptor(
     let connection_counter = Arc::new(AtomicU64::new(1));
     Ok(tokio::spawn(async move {
         loop {
-            match listener.accept().await {
-                Ok((stream, peer_addr)) => {
-                    print_websocket_connection(peer_addr);
-                    let connection_id =
-                        ConnectionId(connection_counter.fetch_add(1, Ordering::Relaxed));
-                    let transport_event_tx_for_connection = transport_event_tx.clone();
-                    tokio::spawn(async move {
-                        run_websocket_connection(
-                            connection_id,
-                            stream,
-                            transport_event_tx_for_connection,
-                        )
-                        .await;
-                    });
+            tokio::select! {
+                _ = shutdown_token.cancelled() => {
+                    info!("websocket acceptor shutting down");
+                    break;
                 }
-                Err(err) => {
-                    error!("failed to accept websocket connection: {err}");
+                accept_result = listener.accept() => {
+                    match accept_result {
+                        Ok((stream, peer_addr)) => {
+                            info!(%peer_addr, "websocket client connected");
+                            let connection_id =
+                                ConnectionId(connection_counter.fetch_add(1, Ordering::Relaxed));
+                            let transport_event_tx_for_connection = transport_event_tx.clone();
+                            tokio::spawn(async move {
+                                run_websocket_connection(
+                                    connection_id,
+                                    stream,
+                                    transport_event_tx_for_connection,
+                                )
+                                .await;
+                            });
+                        }
+                        Err(err) => {
+                            error!("failed to accept websocket connection: {err}");
+                        }
+                    }
                 }
             }
         }
@@ -291,20 +318,23 @@ async fn run_websocket_connection(
     stream: TcpStream,
     transport_event_tx: mpsc::Sender<TransportEvent>,
 ) {
-    let websocket_stream = match accept_async(stream).await {
-        Ok(stream) => stream,
-        Err(err) => {
-            warn!("failed to complete websocket handshake: {err}");
-            return;
-        }
-    };
+    let websocket_stream =
+        match accept_async_with_config(stream, Some(WebSocketConfig::default())).await {
+            Ok(stream) => stream,
+            Err(err) => {
+                warn!("failed to complete websocket handshake: {err}");
+                return;
+            }
+        };
 
-    let (writer_tx, mut writer_rx) = mpsc::channel::<OutgoingMessage>(CHANNEL_CAPACITY);
+    let (writer_tx, writer_rx) = mpsc::channel::<OutgoingMessage>(CHANNEL_CAPACITY);
     let writer_tx_for_reader = writer_tx.clone();
+    let disconnect_token = CancellationToken::new();
     if transport_event_tx
         .send(TransportEvent::ConnectionOpened {
             connection_id,
             writer: writer_tx,
+            disconnect_sender: Some(disconnect_token.clone()),
         })
         .await
         .is_err()
@@ -312,9 +342,62 @@ async fn run_websocket_connection(
         return;
     }
 
-    let (mut websocket_writer, mut websocket_reader) = websocket_stream.split();
+    let (websocket_writer, websocket_reader) = websocket_stream.split();
+    let (writer_control_tx, writer_control_rx) =
+        mpsc::channel::<WebSocketMessage>(CHANNEL_CAPACITY);
+    let mut outbound_task = tokio::spawn(run_websocket_outbound_loop(
+        websocket_writer,
+        writer_rx,
+        writer_control_rx,
+        disconnect_token.clone(),
+    ));
+    let mut inbound_task = tokio::spawn(run_websocket_inbound_loop(
+        websocket_reader,
+        transport_event_tx.clone(),
+        writer_tx_for_reader,
+        writer_control_tx,
+        connection_id,
+        disconnect_token.clone(),
+    ));
+
+    tokio::select! {
+        _ = &mut outbound_task => {
+            disconnect_token.cancel();
+            inbound_task.abort();
+        }
+        _ = &mut inbound_task => {
+            disconnect_token.cancel();
+            outbound_task.abort();
+        }
+    }
+
+    let _ = transport_event_tx
+        .send(TransportEvent::ConnectionClosed { connection_id })
+        .await;
+}
+
+async fn run_websocket_outbound_loop(
+    mut websocket_writer: futures::stream::SplitSink<
+        tokio_tungstenite::WebSocketStream<TcpStream>,
+        WebSocketMessage,
+    >,
+    mut writer_rx: mpsc::Receiver<OutgoingMessage>,
+    mut writer_control_rx: mpsc::Receiver<WebSocketMessage>,
+    disconnect_token: CancellationToken,
+) {
     loop {
         tokio::select! {
+            _ = disconnect_token.cancelled() => {
+                break;
+            }
+            message = writer_control_rx.recv() => {
+                let Some(message) = message else {
+                    break;
+                };
+                if websocket_writer.send(message).await.is_err() {
+                    break;
+                }
+            }
             outgoing_message = writer_rx.recv() => {
                 let Some(outgoing_message) = outgoing_message else {
                     break;
@@ -326,6 +409,25 @@ async fn run_websocket_connection(
                     break;
                 }
             }
+        }
+    }
+}
+
+async fn run_websocket_inbound_loop(
+    mut websocket_reader: futures::stream::SplitStream<
+        tokio_tungstenite::WebSocketStream<TcpStream>,
+    >,
+    transport_event_tx: mpsc::Sender<TransportEvent>,
+    writer_tx_for_reader: mpsc::Sender<OutgoingMessage>,
+    writer_control_tx: mpsc::Sender<WebSocketMessage>,
+    connection_id: ConnectionId,
+    disconnect_token: CancellationToken,
+) {
+    loop {
+        tokio::select! {
+            _ = disconnect_token.cancelled() => {
+                break;
+            }
             incoming_message = websocket_reader.next() => {
                 match incoming_message {
                     Some(Ok(WebSocketMessage::Text(text))) => {
@@ -341,8 +443,13 @@ async fn run_websocket_connection(
                         }
                     }
                     Some(Ok(WebSocketMessage::Ping(payload))) => {
-                        if websocket_writer.send(WebSocketMessage::Pong(payload)).await.is_err() {
-                            break;
+                        match writer_control_tx.try_send(WebSocketMessage::Pong(payload)) {
+                            Ok(()) => {}
+                            Err(tokio::sync::mpsc::error::TrySendError::Closed(_)) => break,
+                            Err(tokio::sync::mpsc::error::TrySendError::Full(_)) => {
+                                warn!("websocket control queue full while replying to ping; closing connection");
+                                break;
+                            }
                         }
                     }
                     Some(Ok(WebSocketMessage::Pong(_))) => {}
@@ -359,10 +466,6 @@ async fn run_websocket_connection(
             }
         }
     }
-
-    let _ = transport_event_tx
-        .send(TransportEvent::ConnectionClosed { connection_id })
-        .await;
 }
 
 async fn forward_incoming_message(
@@ -461,30 +564,86 @@ fn should_skip_notification_for_connection(
     }
 }
 
+fn disconnect_connection(
+    connections: &mut HashMap<ConnectionId, OutboundConnectionState>,
+    connection_id: ConnectionId,
+) -> bool {
+    if let Some(connection_state) = connections.remove(&connection_id) {
+        connection_state.request_disconnect();
+        return true;
+    }
+    false
+}
+
+async fn send_message_to_connection(
+    connections: &mut HashMap<ConnectionId, OutboundConnectionState>,
+    connection_id: ConnectionId,
+    message: OutgoingMessage,
+) -> bool {
+    let Some(connection_state) = connections.get(&connection_id) else {
+        warn!("dropping message for disconnected connection: {connection_id:?}");
+        return false;
+    };
+    let message = filter_outgoing_message_for_connection(connection_state, message);
+    if should_skip_notification_for_connection(connection_state, &message) {
+        return false;
+    }
+
+    let writer = connection_state.writer.clone();
+    if connection_state.can_disconnect() {
+        match writer.try_send(message) {
+            Ok(()) => false,
+            Err(mpsc::error::TrySendError::Full(_)) => {
+                warn!(
+                    "disconnecting slow connection after outbound queue filled: {connection_id:?}"
+                );
+                disconnect_connection(connections, connection_id)
+            }
+            Err(mpsc::error::TrySendError::Closed(_)) => {
+                disconnect_connection(connections, connection_id)
+            }
+        }
+    } else if writer.send(message).await.is_err() {
+        disconnect_connection(connections, connection_id)
+    } else {
+        false
+    }
+}
+
+fn filter_outgoing_message_for_connection(
+    connection_state: &OutboundConnectionState,
+    message: OutgoingMessage,
+) -> OutgoingMessage {
+    let experimental_api_enabled = connection_state
+        .experimental_api_enabled
+        .load(Ordering::Acquire);
+    match message {
+        OutgoingMessage::Request(ServerRequest::CommandExecutionRequestApproval {
+            request_id,
+            mut params,
+        }) => {
+            if !experimental_api_enabled {
+                params.strip_experimental_fields();
+            }
+            OutgoingMessage::Request(ServerRequest::CommandExecutionRequestApproval {
+                request_id,
+                params,
+            })
+        }
+        _ => message,
+    }
+}
+
 pub(crate) async fn route_outgoing_envelope(
     connections: &mut HashMap<ConnectionId, OutboundConnectionState>,
     envelope: OutgoingEnvelope,
-) -> Vec<ConnectionId> {
-    let mut disconnected = Vec::new();
+) {
     match envelope {
         OutgoingEnvelope::ToConnection {
             connection_id,
             message,
         } => {
-            let Some(connection_state) = connections.get(&connection_id) else {
-                warn!(
-                    "dropping message for disconnected connection: {:?}",
-                    connection_id
-                );
-                return disconnected;
-            };
-            if should_skip_notification_for_connection(connection_state, &message) {
-                return disconnected;
-            }
-            if connection_state.writer.send(message).await.is_err() {
-                connections.remove(&connection_id);
-                disconnected.push(connection_id);
-            }
+            let _ = send_message_to_connection(connections, connection_id, message).await;
         }
         OutgoingEnvelope::Broadcast { message } => {
             let target_connections: Vec<ConnectionId> = connections
@@ -501,17 +660,11 @@ pub(crate) async fn route_outgoing_envelope(
                 .collect();
 
             for connection_id in target_connections {
-                let Some(connection_state) = connections.get(&connection_id) else {
-                    continue;
-                };
-                if connection_state.writer.send(message.clone()).await.is_err() {
-                    connections.remove(&connection_id);
-                    disconnected.push(connection_id);
-                }
+                let _ =
+                    send_message_to_connection(connections, connection_id, message.clone()).await;
             }
         }
     }
-    disconnected
 }
 
 #[cfg(test)]
@@ -520,6 +673,9 @@ mod tests {
     use crate::error_code::OVERLOADED_ERROR_CODE;
     use pretty_assertions::assert_eq;
     use serde_json::json;
+    use std::path::PathBuf;
+    use tokio::time::Duration;
+    use tokio::time::timeout;
 
     #[test]
     fn app_server_transport_parses_stdio_listen_url() {
@@ -754,10 +910,16 @@ mod tests {
         let mut connections = HashMap::new();
         connections.insert(
             connection_id,
-            OutboundConnectionState::new(writer_tx, initialized, opted_out_notification_methods),
+            OutboundConnectionState::new(
+                writer_tx,
+                initialized,
+                Arc::new(AtomicBool::new(true)),
+                opted_out_notification_methods,
+                None,
+            ),
         );
 
-        let disconnected = route_outgoing_envelope(
+        route_outgoing_envelope(
             &mut connections,
             OutgoingEnvelope::ToConnection {
                 connection_id,
@@ -771,10 +933,298 @@ mod tests {
         )
         .await;
 
-        assert_eq!(disconnected, Vec::<ConnectionId>::new());
         assert!(
             writer_rx.try_recv().is_err(),
             "opted-out notification should be dropped"
         );
     }
+
+    #[tokio::test]
+    async fn command_execution_request_approval_strips_experimental_fields_without_capability() {
+        let connection_id = ConnectionId(8);
+        let (writer_tx, mut writer_rx) = mpsc::channel(1);
+
+        let mut connections = HashMap::new();
+        connections.insert(
+            connection_id,
+            OutboundConnectionState::new(
+                writer_tx,
+                Arc::new(AtomicBool::new(true)),
+                Arc::new(AtomicBool::new(false)),
+                Arc::new(RwLock::new(HashSet::new())),
+                None,
+            ),
+        );
+
+        route_outgoing_envelope(
+            &mut connections,
+            OutgoingEnvelope::ToConnection {
+                connection_id,
+                message: OutgoingMessage::Request(ServerRequest::CommandExecutionRequestApproval {
+                    request_id: codex_app_server_protocol::RequestId::Integer(1),
+                    params: codex_app_server_protocol::CommandExecutionRequestApprovalParams {
+                        thread_id: "thr_123".to_string(),
+                        turn_id: "turn_123".to_string(),
+                        item_id: "call_123".to_string(),
+                        approval_id: None,
+                        reason: Some("Need extra read access".to_string()),
+                        network_approval_context: None,
+                        command: Some("cat file".to_string()),
+                        cwd: Some(PathBuf::from("/tmp")),
+                        command_actions: None,
+                        additional_permissions: Some(
+                            codex_app_server_protocol::AdditionalPermissionProfile {
+                                network: None,
+                                file_system: Some(
+                                    codex_app_server_protocol::AdditionalFileSystemPermissions {
+                                        read: Some(vec![PathBuf::from("/tmp/allowed")]),
+                                        write: None,
+                                    },
+                                ),
+                                macos: None,
+                            },
+                        ),
+                        proposed_execpolicy_amendment: None,
+                        proposed_network_policy_amendments: None,
+                        available_decisions: None,
+                    },
+                }),
+            },
+        )
+        .await;
+
+        let message = writer_rx
+            .recv()
+            .await
+            .expect("request should be delivered to the connection");
+        let json = serde_json::to_value(message).expect("request should serialize");
+        assert_eq!(json["params"].get("additionalPermissions"), None);
+    }
+
+    #[tokio::test]
+    async fn command_execution_request_approval_keeps_experimental_fields_with_capability() {
+        let connection_id = ConnectionId(9);
+        let (writer_tx, mut writer_rx) = mpsc::channel(1);
+
+        let mut connections = HashMap::new();
+        connections.insert(
+            connection_id,
+            OutboundConnectionState::new(
+                writer_tx,
+                Arc::new(AtomicBool::new(true)),
+                Arc::new(AtomicBool::new(true)),
+                Arc::new(RwLock::new(HashSet::new())),
+                None,
+            ),
+        );
+
+        route_outgoing_envelope(
+            &mut connections,
+            OutgoingEnvelope::ToConnection {
+                connection_id,
+                message: OutgoingMessage::Request(ServerRequest::CommandExecutionRequestApproval {
+                    request_id: codex_app_server_protocol::RequestId::Integer(1),
+                    params: codex_app_server_protocol::CommandExecutionRequestApprovalParams {
+                        thread_id: "thr_123".to_string(),
+                        turn_id: "turn_123".to_string(),
+                        item_id: "call_123".to_string(),
+                        approval_id: None,
+                        reason: Some("Need extra read access".to_string()),
+                        network_approval_context: None,
+                        command: Some("cat file".to_string()),
+                        cwd: Some(PathBuf::from("/tmp")),
+                        command_actions: None,
+                        additional_permissions: Some(
+                            codex_app_server_protocol::AdditionalPermissionProfile {
+                                network: None,
+                                file_system: Some(
+                                    codex_app_server_protocol::AdditionalFileSystemPermissions {
+                                        read: Some(vec![PathBuf::from("/tmp/allowed")]),
+                                        write: None,
+                                    },
+                                ),
+                                macos: None,
+                            },
+                        ),
+                        proposed_execpolicy_amendment: None,
+                        proposed_network_policy_amendments: None,
+                        available_decisions: None,
+                    },
+                }),
+            },
+        )
+        .await;
+
+        let message = writer_rx
+            .recv()
+            .await
+            .expect("request should be delivered to the connection");
+        let json = serde_json::to_value(message).expect("request should serialize");
+        assert_eq!(
+            json["params"]["additionalPermissions"],
+            json!({
+                "network": null,
+                "fileSystem": {
+                    "read": ["/tmp/allowed"],
+                    "write": null,
+                },
+                "macos": null,
+            })
+        );
+    }
+
+    #[tokio::test]
+    async fn broadcast_does_not_block_on_slow_connection() {
+        let fast_connection_id = ConnectionId(1);
+        let slow_connection_id = ConnectionId(2);
+
+        let (fast_writer_tx, mut fast_writer_rx) = mpsc::channel(1);
+        let (slow_writer_tx, mut slow_writer_rx) = mpsc::channel(1);
+        let fast_disconnect_token = CancellationToken::new();
+        let slow_disconnect_token = CancellationToken::new();
+
+        let mut connections = HashMap::new();
+        connections.insert(
+            fast_connection_id,
+            OutboundConnectionState::new(
+                fast_writer_tx,
+                Arc::new(AtomicBool::new(true)),
+                Arc::new(AtomicBool::new(true)),
+                Arc::new(RwLock::new(HashSet::new())),
+                Some(fast_disconnect_token.clone()),
+            ),
+        );
+        connections.insert(
+            slow_connection_id,
+            OutboundConnectionState::new(
+                slow_writer_tx.clone(),
+                Arc::new(AtomicBool::new(true)),
+                Arc::new(AtomicBool::new(true)),
+                Arc::new(RwLock::new(HashSet::new())),
+                Some(slow_disconnect_token.clone()),
+            ),
+        );
+
+        let queued_message =
+            OutgoingMessage::Notification(crate::outgoing_message::OutgoingNotification {
+                method: "codex/event/already-buffered".to_string(),
+                params: None,
+            });
+        slow_writer_tx
+            .try_send(queued_message)
+            .expect("channel should have room");
+
+        let broadcast_message =
+            OutgoingMessage::Notification(crate::outgoing_message::OutgoingNotification {
+                method: "codex/event/test".to_string(),
+                params: None,
+            });
+        timeout(
+            Duration::from_millis(100),
+            route_outgoing_envelope(
+                &mut connections,
+                OutgoingEnvelope::Broadcast {
+                    message: broadcast_message,
+                },
+            ),
+        )
+        .await
+        .expect("broadcast should not block on a full writer");
+        assert!(!connections.contains_key(&slow_connection_id));
+        assert!(slow_disconnect_token.is_cancelled());
+        assert!(!fast_disconnect_token.is_cancelled());
+        let fast_message = fast_writer_rx
+            .try_recv()
+            .expect("fast connection should receive broadcast");
+        assert!(matches!(
+            fast_message,
+            OutgoingMessage::Notification(crate::outgoing_message::OutgoingNotification {
+                method,
+                params: None,
+            }) if method == "codex/event/test"
+        ));
+
+        let slow_message = slow_writer_rx
+            .try_recv()
+            .expect("slow connection should retain its original buffered message");
+        assert!(matches!(
+            slow_message,
+            OutgoingMessage::Notification(crate::outgoing_message::OutgoingNotification {
+                method,
+                params: None,
+            }) if method == "codex/event/already-buffered"
+        ));
+    }
+
+    #[tokio::test]
+    async fn to_connection_stdio_waits_instead_of_disconnecting_when_writer_queue_is_full() {
+        let connection_id = ConnectionId(3);
+        let (writer_tx, mut writer_rx) = mpsc::channel(1);
+        writer_tx
+            .send(OutgoingMessage::Notification(
+                crate::outgoing_message::OutgoingNotification {
+                    method: "queued".to_string(),
+                    params: None,
+                },
+            ))
+            .await
+            .expect("channel should accept the first queued message");
+
+        let mut connections = HashMap::new();
+        connections.insert(
+            connection_id,
+            OutboundConnectionState::new(
+                writer_tx,
+                Arc::new(AtomicBool::new(true)),
+                Arc::new(AtomicBool::new(true)),
+                Arc::new(RwLock::new(HashSet::new())),
+                None,
+            ),
+        );
+
+        let route_task = tokio::spawn(async move {
+            route_outgoing_envelope(
+                &mut connections,
+                OutgoingEnvelope::ToConnection {
+                    connection_id,
+                    message: OutgoingMessage::Notification(
+                        crate::outgoing_message::OutgoingNotification {
+                            method: "second".to_string(),
+                            params: None,
+                        },
+                    ),
+                },
+            )
+            .await
+        });
+
+        let first = timeout(Duration::from_millis(100), writer_rx.recv())
+            .await
+            .expect("first queued message should be readable")
+            .expect("first queued message should exist");
+        let second = timeout(Duration::from_millis(100), writer_rx.recv())
+            .await
+            .expect("second message should eventually be delivered")
+            .expect("second message should exist");
+
+        timeout(Duration::from_millis(100), route_task)
+            .await
+            .expect("routing should finish after writer drains")
+            .expect("routing task should succeed");
+
+        assert!(matches!(
+            first,
+            OutgoingMessage::Notification(crate::outgoing_message::OutgoingNotification {
+                method,
+                params: None,
+            }) if method == "queued"
+        ));
+        assert!(matches!(
+            second,
+            OutgoingMessage::Notification(crate::outgoing_message::OutgoingNotification {
+                method,
+                params: None,
+            }) if method == "second"
+        ));
+    }
 }
