@@ -5,6 +5,8 @@ mod collaboration_mode_list;
 mod compaction;
 mod config_rpc;
 mod connection_handling_websocket;
+#[cfg(unix)]
+mod connection_handling_websocket_unix;
 mod dynamic_tools;
 mod experimental_api;
 mod experimental_feature_list;
@@ -13,8 +15,10 @@ mod model_list;
 mod output_schema;
 mod plan_item;
 mod rate_limits;
+mod realtime_conversation;
 mod request_user_input;
 mod review;
+mod safety_check_downgrade;
 mod skills_list;
 mod thread_archive;
 mod thread_fork;
@@ -24,7 +28,11 @@ mod thread_read;
 mod thread_resume;
 mod thread_rollback;
 mod thread_start;
+mod thread_status;
 mod thread_unarchive;
+mod thread_unsubscribe;
 mod turn_interrupt;
 mod turn_start;
+mod turn_start_zsh_fork;
 mod turn_steer;
+mod windows_sandbox_setup;
