@@ -20,8 +20,10 @@ use codex_core::connectors::AppMetadata;
 use codex_core::connectors::CONNECTORS_CACHE_TTL;
 pub use codex_core::connectors::connector_display_label;
 use codex_core::connectors::connector_install_url;
+use codex_core::connectors::filter_disallowed_connectors;
 pub use codex_core::connectors::list_accessible_connectors_from_mcp_tools;
 pub use codex_core::connectors::list_accessible_connectors_from_mcp_tools_with_options;
+pub use codex_core::connectors::list_accessible_connectors_from_mcp_tools_with_options_and_status;
 pub use codex_core::connectors::list_cached_accessible_connectors_from_mcp_tools;
 use codex_core::connectors::merge_connectors;
 pub use codex_core::connectors::with_app_enabled_state;
@@ -104,7 +106,7 @@ pub async fn list_cached_all_connectors(config: &Config) -> Option<Vec<AppInfo>>
     }
     let token_data = get_chatgpt_token_data()?;
     let cache_key = all_connectors_cache_key(config, &token_data);
-    read_cached_all_connectors(&cache_key)
+    read_cached_all_connectors(&cache_key).map(filter_disallowed_connectors)
 }
 
 pub async fn list_all_connectors_with_options(
@@ -121,7 +123,7 @@ pub async fn list_all_connectors_with_options(
         get_chatgpt_token_data().ok_or_else(|| anyhow::anyhow!("ChatGPT token not available"))?;
     let cache_key = all_connectors_cache_key(config, &token_data);
     if !force_refetch && let Some(cached_connectors) = read_cached_all_connectors(&cache_key) {
-        return Ok(cached_connectors);
+        return Ok(filter_disallowed_connectors(cached_connectors));
     }
 
     let mut apps = list_directory_connectors(config).await?;
@@ -147,6 +149,7 @@ pub async fn list_all_connectors_with_options(
             .cmp(&right.name)
             .then_with(|| left.id.cmp(&right.id))
     });
+    let connectors = filter_disallowed_connectors(connectors);
     write_cached_all_connectors(cache_key, &connectors);
     Ok(connectors)
 }
@@ -217,9 +220,11 @@ async fn list_directory_connectors(config: &Config) -> anyhow::Result<Vec<Direct
         let path = match next_token.as_deref() {
             Some(token) => {
                 let encoded_token = urlencoding::encode(token);
-                format!("/connectors/directory/list?tier=categorized&token={encoded_token}")
+                format!(
+                    "/connectors/directory/list?tier=categorized&token={encoded_token}&external_logos=true"
+                )
             }
-            None => "/connectors/directory/list?tier=categorized".to_string(),
+            None => "/connectors/directory/list?tier=categorized&external_logos=true".to_string(),
         };
         let response: DirectoryListResponse =
             chatgpt_get_request_with_timeout(config, path, Some(DIRECTORY_CONNECTORS_TIMEOUT))
@@ -244,7 +249,7 @@ async fn list_directory_connectors(config: &Config) -> anyhow::Result<Vec<Direct
 async fn list_workspace_connectors(config: &Config) -> anyhow::Result<Vec<DirectoryApp>> {
     let response: anyhow::Result<DirectoryListResponse> = chatgpt_get_request_with_timeout(
         config,
-        "/connectors/directory/list_workspace".to_string(),
+        "/connectors/directory/list_workspace?external_logos=true".to_string(),
         Some(DIRECTORY_CONNECTORS_TIMEOUT),
     )
     .await;
@@ -451,33 +456,6 @@ fn normalize_connector_value(value: Option<&str>) -> Option<String> {
         .filter(|value| !value.is_empty())
         .map(str::to_string)
 }
-
-const DISALLOWED_CONNECTOR_IDS: &[&str] = &[
-    "asdk_app_6938a94a61d881918ef32cb999ff937c",
-    "connector_2b0a9009c9c64bf9933a3dae3f2b1254",
-    "connector_68de829bf7648191acd70a907364c67c",
-    "connector_68e004f14af881919eb50893d3d9f523",
-    "connector_69272cb413a081919685ec3c88d1744e",
-];
-const DISALLOWED_CONNECTOR_PREFIX: &str = "connector_openai_";
-
-fn filter_disallowed_connectors(connectors: Vec<AppInfo>) -> Vec<AppInfo> {
-    connectors
-        .into_iter()
-        .filter(is_connector_allowed)
-        .collect()
-}
-
-fn is_connector_allowed(connector: &AppInfo) -> bool {
-    let connector_id = connector.id.as_str();
-    if connector_id.starts_with(DISALLOWED_CONNECTOR_PREFIX)
-        || DISALLOWED_CONNECTOR_IDS.contains(&connector_id)
-    {
-        return false;
-    }
-    true
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -522,7 +500,7 @@ mod tests {
     }
 
     #[test]
-    fn filters_openai_connectors() {
+    fn filters_openai_prefixed_connectors() {
         let filtered = filter_disallowed_connectors(vec![
             app("connector_openai_foo"),
             app("connector_openai_bar"),
