@@ -1,7 +1,7 @@
-pub mod aggregate;
 pub mod compact;
 pub mod memories;
 pub mod models;
+pub mod realtime_websocket;
 pub mod responses;
 pub mod responses_websocket;
 mod session;
