@@ -164,6 +164,8 @@ const X_CODEX_TURN_STATE_HEADER: &str = "x-codex-turn-state";
 const X_MODELS_ETAG_HEADER: &str = "x-models-etag";
 const X_REASONING_INCLUDED_HEADER: &str = "x-reasoning-included";
 const OPENAI_MODEL_HEADER: &str = "openai-model";
+const WEBSOCKET_CONNECTION_LIMIT_REACHED_CODE: &str = "websocket_connection_limit_reached";
+const WEBSOCKET_CONNECTION_LIMIT_REACHED_MESSAGE: &str = "Responses websocket connection limit reached (60 minutes). Create a new websocket connection to continue.";
 
 pub struct ResponsesWebsocketConnection {
     stream: Arc<Mutex<Option<WsStream>>>,
@@ -175,6 +177,19 @@ pub struct ResponsesWebsocketConnection {
     telemetry: Option<Arc<dyn WebsocketTelemetry>>,
 }
 
+impl std::fmt::Debug for ResponsesWebsocketConnection {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("ResponsesWebsocketConnection")
+            .field("stream", &"<ws-stream>")
+            .field("idle_timeout", &self.idle_timeout)
+            .field("server_reasoning_included", &self.server_reasoning_included)
+            .field("models_etag", &self.models_etag)
+            .field("server_model", &self.server_model)
+            .field("telemetry", &self.telemetry.as_ref().map(|_| "<telemetry>"))
+            .finish()
+    }
+}
+
 impl ResponsesWebsocketConnection {
     fn new(
         stream: WsStream,
@@ -404,6 +419,12 @@ fn map_ws_error(err: WsError, url: &Url) -> ApiError {
     }
 }
 
+#[derive(Debug, Deserialize)]
+struct WrappedWebsocketError {
+    code: Option<String>,
+    message: Option<String>,
+}
+
 #[derive(Debug, Deserialize)]
 struct WrappedWebsocketErrorEvent {
     #[serde(rename = "type")]
@@ -411,7 +432,7 @@ struct WrappedWebsocketErrorEvent {
     #[serde(alias = "status_code")]
     status: Option<u16>,
     #[serde(default)]
-    error: Option<Value>,
+    error: Option<WrappedWebsocketError>,
     #[serde(default)]
     headers: Option<JsonMap<String, Value>>,
 }
@@ -424,7 +445,10 @@ fn parse_wrapped_websocket_error_event(payload: &str) -> Option<WrappedWebsocket
     Some(event)
 }
 
-fn map_wrapped_websocket_error_event(event: WrappedWebsocketErrorEvent) -> Option<ApiError> {
+fn map_wrapped_websocket_error_event(
+    event: WrappedWebsocketErrorEvent,
+    original_payload: String,
+) -> Option<ApiError> {
     let WrappedWebsocketErrorEvent {
         status,
         error,
@@ -432,28 +456,29 @@ fn map_wrapped_websocket_error_event(event: WrappedWebsocketErrorEvent) -> Optio
         ..
     } = event;
 
+    if let Some(error) = error.as_ref()
+        && let Some(code) = error.code.as_deref()
+        && code == WEBSOCKET_CONNECTION_LIMIT_REACHED_CODE
+    {
+        return Some(ApiError::Retryable {
+            message: error
+                .message
+                .clone()
+                .unwrap_or_else(|| WEBSOCKET_CONNECTION_LIMIT_REACHED_MESSAGE.to_string()),
+            delay: None,
+        });
+    }
+
     let status = StatusCode::from_u16(status?).ok()?;
     if status.is_success() {
         return None;
     }
 
-    let body = error.map(|error| {
-        serde_json::to_string_pretty(&serde_json::json!({
-            "error": error
-        }))
-        .unwrap_or_else(|_| {
-            serde_json::json!({
-                "error": error
-            })
-            .to_string()
-        })
-    });
-
     Some(ApiError::Transport(TransportError::Http {
         status,
         url: None,
         headers: headers.map(json_headers_to_http_headers),
-        body,
+        body: Some(original_payload),
     }))
 }
 
@@ -538,7 +563,8 @@ async fn run_websocket_response_stream(
             Message::Text(text) => {
                 trace!("websocket event: {text}");
                 if let Some(wrapped_error) = parse_wrapped_websocket_error_event(&text)
-                    && let Some(error) = map_wrapped_websocket_error_event(wrapped_error)
+                    && let Some(error) =
+                        map_wrapped_websocket_error_event(wrapped_error, text.to_string())
                 {
                     return Err(error);
                 }
@@ -626,7 +652,7 @@ mod tests {
 
         let wrapped_error = parse_wrapped_websocket_error_event(&payload)
             .expect("expected websocket error payload to be parsed");
-        let api_error = map_wrapped_websocket_error_event(wrapped_error)
+        let api_error = map_wrapped_websocket_error_event(wrapped_error, payload)
             .expect("expected websocket error payload to map to ApiError");
 
         let ApiError::Transport(TransportError::Http {
@@ -686,7 +712,7 @@ mod tests {
 
         let wrapped_error = parse_wrapped_websocket_error_event(&payload)
             .expect("expected websocket error payload to be parsed");
-        let api_error = map_wrapped_websocket_error_event(wrapped_error)
+        let api_error = map_wrapped_websocket_error_event(wrapped_error, payload)
             .expect("expected websocket error payload to map to ApiError");
         let ApiError::Transport(TransportError::Http { status, body, .. }) = api_error else {
             panic!("expected ApiError::Transport(Http)");
@@ -697,6 +723,30 @@ mod tests {
         assert!(body.contains("Model does not support image inputs"));
     }
 
+    #[test]
+    fn parse_wrapped_websocket_error_event_with_connection_limit_maps_retryable() {
+        let payload = json!({
+            "type": "error",
+            "status": 400,
+            "error": {
+                "type": "invalid_request_error",
+                "code": "websocket_connection_limit_reached",
+                "message": "Responses websocket connection limit reached (60 minutes). Create a new websocket connection to continue."
+            }
+        })
+        .to_string();
+
+        let wrapped_error = parse_wrapped_websocket_error_event(&payload)
+            .expect("expected websocket error payload to be parsed");
+        let api_error = map_wrapped_websocket_error_event(wrapped_error, payload)
+            .expect("expected websocket error payload to map to ApiError");
+        let ApiError::Retryable { message, delay } = api_error else {
+            panic!("expected ApiError::Retryable");
+        };
+        assert_eq!(message, WEBSOCKET_CONNECTION_LIMIT_REACHED_MESSAGE);
+        assert_eq!(delay, None);
+    }
+
     #[test]
     fn parse_wrapped_websocket_error_event_without_status_is_not_mapped() {
         let payload = json!({
@@ -714,7 +764,7 @@ mod tests {
 
         let wrapped_error = parse_wrapped_websocket_error_event(&payload)
             .expect("expected websocket error payload to be parsed");
-        let api_error = map_wrapped_websocket_error_event(wrapped_error);
+        let api_error = map_wrapped_websocket_error_event(wrapped_error, payload);
         assert!(api_error.is_none());
     }
 
