@@ -1,17 +1,32 @@
 use crate::agent::AgentStatus;
 use crate::agent::guards::Guards;
+use crate::agent::status::is_final;
 use crate::error::CodexErr;
 use crate::error::Result as CodexResult;
+use crate::find_thread_path_by_id_str;
+use crate::session_prefix::format_subagent_notification_message;
+use crate::state_db;
 use crate::thread_manager::ThreadManagerState;
 use codex_protocol::ThreadId;
 use codex_protocol::protocol::Op;
 use codex_protocol::protocol::SessionSource;
+use codex_protocol::protocol::SubAgentSource;
+use codex_protocol::protocol::TokenUsage;
 use codex_protocol::user_input::UserInput;
-use std::path::PathBuf;
 use std::sync::Arc;
 use std::sync::Weak;
 use tokio::sync::watch;
 
+const AGENT_NAMES: &str = include_str!("agent_names.txt");
+
+fn agent_nickname_list() -> Vec<&'static str> {
+    AGENT_NAMES
+        .lines()
+        .map(str::trim)
+        .filter(|name| !name.is_empty())
+        .collect()
+}
+
 /// Control-plane handle for multi-agent operations.
 /// `AgentControl` is held by each session (via `SessionServices`). It provides capability to
 /// spawn new agents and the inter-agent communication layer.
@@ -44,13 +59,31 @@ impl AgentControl {
         session_source: Option<SessionSource>,
     ) -> CodexResult<ThreadId> {
         let state = self.upgrade()?;
-        let reservation = self.state.reserve_spawn_slot(config.agent_max_threads)?;
+        let mut reservation = self.state.reserve_spawn_slot(config.agent_max_threads)?;
+        let session_source = match session_source {
+            Some(SessionSource::SubAgent(SubAgentSource::ThreadSpawn {
+                parent_thread_id,
+                depth,
+                agent_role,
+                ..
+            })) => {
+                let agent_nickname = reservation.reserve_agent_nickname(&agent_nickname_list())?;
+                Some(SessionSource::SubAgent(SubAgentSource::ThreadSpawn {
+                    parent_thread_id,
+                    depth,
+                    agent_nickname: Some(agent_nickname),
+                    agent_role,
+                }))
+            }
+            other => other,
+        };
+        let notification_source = session_source.clone();
 
         // The same `AgentControl` is sent to spawn the thread.
         let new_thread = match session_source {
             Some(session_source) => {
                 state
-                    .spawn_new_thread_with_source(config, self.clone(), session_source, false)
+                    .spawn_new_thread_with_source(config, self.clone(), session_source, false, None)
                     .await?
             }
             None => state.spawn_new_thread(config, self.clone()).await?,
@@ -63,6 +96,7 @@ impl AgentControl {
         state.notify_thread_created(new_thread.thread_id);
 
         self.send_input(new_thread.thread_id, items).await?;
+        self.maybe_start_completion_watcher(new_thread.thread_id, notification_source);
 
         Ok(new_thread.thread_id)
     }
@@ -71,11 +105,51 @@ impl AgentControl {
     pub(crate) async fn resume_agent_from_rollout(
         &self,
         config: crate::config::Config,
-        rollout_path: PathBuf,
+        thread_id: ThreadId,
         session_source: SessionSource,
     ) -> CodexResult<ThreadId> {
         let state = self.upgrade()?;
-        let reservation = self.state.reserve_spawn_slot(config.agent_max_threads)?;
+        let mut reservation = self.state.reserve_spawn_slot(config.agent_max_threads)?;
+        let session_source = match session_source {
+            SessionSource::SubAgent(SubAgentSource::ThreadSpawn {
+                parent_thread_id,
+                depth,
+                ..
+            }) => {
+                // Collab resume callers rebuild a placeholder ThreadSpawn source. Rehydrate the
+                // stored nickname/role from sqlite when available; otherwise leave both unset.
+                let (resumed_agent_nickname, resumed_agent_role) =
+                    if let Some(state_db_ctx) = state_db::get_state_db(&config, None).await {
+                        match state_db_ctx.get_thread(thread_id).await {
+                            Ok(Some(metadata)) => (metadata.agent_nickname, metadata.agent_role),
+                            Ok(None) | Err(_) => (None, None),
+                        }
+                    } else {
+                        (None, None)
+                    };
+                let reserved_agent_nickname = resumed_agent_nickname
+                    .as_deref()
+                    .map(|agent_nickname| {
+                        reservation.reserve_agent_nickname_with_preference(
+                            &agent_nickname_list(),
+                            Some(agent_nickname),
+                        )
+                    })
+                    .transpose()?;
+                SessionSource::SubAgent(SubAgentSource::ThreadSpawn {
+                    parent_thread_id,
+                    depth,
+                    agent_nickname: reserved_agent_nickname,
+                    agent_role: resumed_agent_role,
+                })
+            }
+            other => other,
+        };
+        let notification_source = session_source.clone();
+        let rollout_path =
+            find_thread_path_by_id_str(config.codex_home.as_path(), &thread_id.to_string())
+                .await?
+                .ok_or_else(|| CodexErr::ThreadNotFound(thread_id))?;
 
         let resumed_thread = state
             .resume_thread_from_rollout_with_source(
@@ -89,6 +163,7 @@ impl AgentControl {
         // Resumed threads are re-registered in-memory and need the same listener
         // attachment path as freshly spawned threads.
         state.notify_thread_created(resumed_thread.thread_id);
+        self.maybe_start_completion_watcher(resumed_thread.thread_id, Some(notification_source));
 
         Ok(resumed_thread.thread_id)
     }
@@ -143,6 +218,23 @@ impl AgentControl {
         thread.agent_status().await
     }
 
+    pub(crate) async fn get_agent_nickname_and_role(
+        &self,
+        agent_id: ThreadId,
+    ) -> Option<(Option<String>, Option<String>)> {
+        let Ok(state) = self.upgrade() else {
+            return None;
+        };
+        let Ok(thread) = state.get_thread(agent_id).await else {
+            return None;
+        };
+        let session_source = thread.config_snapshot().await.session_source;
+        Some((
+            session_source.get_nickname(),
+            session_source.get_agent_role(),
+        ))
+    }
+
     /// Subscribe to status updates for `agent_id`, yielding the latest value and changes.
     pub(crate) async fn subscribe_status(
         &self,
@@ -153,13 +245,70 @@ impl AgentControl {
         Ok(thread.subscribe_status())
     }
 
+    pub(crate) async fn get_total_token_usage(&self, agent_id: ThreadId) -> Option<TokenUsage> {
+        let Ok(state) = self.upgrade() else {
+            return None;
+        };
+        let Ok(thread) = state.get_thread(agent_id).await else {
+            return None;
+        };
+        thread.total_token_usage().await
+    }
+
+    /// Starts a detached watcher for sub-agents spawned from another thread.
+    ///
+    /// This is only enabled for `SubAgentSource::ThreadSpawn`, where a parent thread exists and
+    /// can receive completion notifications.
+    fn maybe_start_completion_watcher(
+        &self,
+        child_thread_id: ThreadId,
+        session_source: Option<SessionSource>,
+    ) {
+        let Some(SessionSource::SubAgent(SubAgentSource::ThreadSpawn {
+            parent_thread_id, ..
+        })) = session_source
+        else {
+            return;
+        };
+        let control = self.clone();
+        tokio::spawn(async move {
+            let mut status_rx = match control.subscribe_status(child_thread_id).await {
+                Ok(rx) => rx,
+                Err(_) => return,
+            };
+            let mut status = status_rx.borrow().clone();
+            while !is_final(&status) {
+                if status_rx.changed().await.is_err() {
+                    status = control.get_status(child_thread_id).await;
+                    break;
+                }
+                status = status_rx.borrow().clone();
+            }
+            if !is_final(&status) {
+                return;
+            }
+
+            let Ok(state) = control.upgrade() else {
+                return;
+            };
+            let Ok(parent_thread) = state.get_thread(parent_thread_id).await else {
+                return;
+            };
+            parent_thread
+                .inject_user_message_without_turn(format_subagent_notification_message(
+                    &child_thread_id.to_string(),
+                    &status,
+                ))
+                .await;
+        });
+    }
+
     fn upgrade(&self) -> CodexResult<Arc<ThreadManagerState>> {
         self.manager
             .upgrade()
             .ok_or_else(|| CodexErr::UnsupportedOperation("thread manager dropped".to_string()))
     }
 }
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -169,16 +318,26 @@ mod tests {
     use crate::agent::agent_status_from_event;
     use crate::config::Config;
     use crate::config::ConfigBuilder;
+    use crate::config_loader::LoaderOverrides;
+    use crate::contextual_user_message::SUBAGENT_NOTIFICATION_OPEN_TAG;
+    use crate::features::Feature;
     use assert_matches::assert_matches;
     use codex_protocol::config_types::ModeKind;
+    use codex_protocol::models::ContentItem;
+    use codex_protocol::models::ResponseItem;
     use codex_protocol::protocol::ErrorEvent;
     use codex_protocol::protocol::EventMsg;
+    use codex_protocol::protocol::SessionSource;
+    use codex_protocol::protocol::SubAgentSource;
     use codex_protocol::protocol::TurnAbortReason;
     use codex_protocol::protocol::TurnAbortedEvent;
     use codex_protocol::protocol::TurnCompleteEvent;
     use codex_protocol::protocol::TurnStartedEvent;
     use pretty_assertions::assert_eq;
     use tempfile::TempDir;
+    use tokio::time::Duration;
+    use tokio::time::sleep;
+    use tokio::time::timeout;
     use toml::Value as TomlValue;
 
     async fn test_config_with_cli_overrides(
@@ -188,6 +347,12 @@ mod tests {
         let config = ConfigBuilder::default()
             .codex_home(home.path().to_path_buf())
             .cli_overrides(cli_overrides)
+            .loader_overrides(LoaderOverrides {
+                #[cfg(target_os = "macos")]
+                managed_preferences_base64: Some(String::new()),
+                macos_managed_config_requirements_base64: Some(String::new()),
+                ..LoaderOverrides::default()
+            })
             .build()
             .await
             .expect("load default test config");
@@ -239,6 +404,42 @@ mod tests {
         }
     }
 
+    fn has_subagent_notification(history_items: &[ResponseItem]) -> bool {
+        history_items.iter().any(|item| {
+            let ResponseItem::Message { role, content, .. } = item else {
+                return false;
+            };
+            if role != "user" {
+                return false;
+            }
+            content.iter().any(|content_item| match content_item {
+                ContentItem::InputText { text } | ContentItem::OutputText { text } => {
+                    text.contains(SUBAGENT_NOTIFICATION_OPEN_TAG)
+                }
+                ContentItem::InputImage { .. } => false,
+            })
+        })
+    }
+
+    async fn wait_for_subagent_notification(parent_thread: &Arc<CodexThread>) -> bool {
+        let wait = async {
+            loop {
+                let history_items = parent_thread
+                    .codex
+                    .session
+                    .clone_history()
+                    .await
+                    .raw_items()
+                    .to_vec();
+                if has_subagent_notification(&history_items) {
+                    return true;
+                }
+                sleep(Duration::from_millis(25)).await;
+            }
+        };
+        timeout(Duration::from_secs(2), wait).await.is_ok()
+    }
+
     #[tokio::test]
     async fn send_input_errors_when_manager_dropped() {
         let control = AgentControl::default();
@@ -332,11 +533,7 @@ mod tests {
         let control = AgentControl::default();
         let (_home, config) = test_config().await;
         let err = control
-            .resume_agent_from_rollout(
-                config,
-                PathBuf::from("/tmp/missing-rollout.jsonl"),
-                SessionSource::Exec,
-            )
+            .resume_agent_from_rollout(config, ThreadId::new(), SessionSource::Exec)
             .await
             .expect_err("resume_agent should fail without a manager");
         assert_eq!(
@@ -608,12 +805,6 @@ mod tests {
             .spawn_agent(config.clone(), text_input("hello"), None)
             .await
             .expect("spawn_agent should succeed");
-        let rollout_path = manager
-            .get_thread(resumable_id)
-            .await
-            .expect("thread should exist")
-            .rollout_path()
-            .expect("rollout path should exist");
         let _ = control
             .shutdown_agent(resumable_id)
             .await
@@ -625,7 +816,7 @@ mod tests {
             .expect("spawn_agent should succeed for active slot");
 
         let err = control
-            .resume_agent_from_rollout(config, rollout_path, SessionSource::Exec)
+            .resume_agent_from_rollout(config, resumable_id, SessionSource::Exec)
             .await
             .expect_err("resume should respect max threads");
         let CodexErr::AgentLimitReached {
@@ -657,9 +848,8 @@ mod tests {
         );
         let control = manager.agent_control();
 
-        let missing_rollout = config.codex_home.join("sessions/missing-rollout.jsonl");
         let _ = control
-            .resume_agent_from_rollout(config.clone(), missing_rollout, SessionSource::Exec)
+            .resume_agent_from_rollout(config.clone(), ThreadId::new(), SessionSource::Exec)
             .await
             .expect_err("resume should fail for missing rollout path");
 
@@ -672,4 +862,209 @@ mod tests {
             .await
             .expect("shutdown resumed thread");
     }
+
+    #[tokio::test]
+    async fn spawn_child_completion_notifies_parent_history() {
+        let harness = AgentControlHarness::new().await;
+        let (parent_thread_id, parent_thread) = harness.start_thread().await;
+
+        let child_thread_id = harness
+            .control
+            .spawn_agent(
+                harness.config.clone(),
+                text_input("hello child"),
+                Some(SessionSource::SubAgent(SubAgentSource::ThreadSpawn {
+                    parent_thread_id,
+                    depth: 1,
+                    agent_nickname: None,
+                    agent_role: Some("explorer".to_string()),
+                })),
+            )
+            .await
+            .expect("child spawn should succeed");
+
+        let child_thread = harness
+            .manager
+            .get_thread(child_thread_id)
+            .await
+            .expect("child thread should exist");
+        let _ = child_thread
+            .submit(Op::Shutdown {})
+            .await
+            .expect("child shutdown should submit");
+
+        assert_eq!(wait_for_subagent_notification(&parent_thread).await, true);
+    }
+
+    #[tokio::test]
+    async fn spawn_thread_subagent_gets_random_nickname_in_session_source() {
+        let harness = AgentControlHarness::new().await;
+        let (parent_thread_id, _parent_thread) = harness.start_thread().await;
+
+        let child_thread_id = harness
+            .control
+            .spawn_agent(
+                harness.config.clone(),
+                text_input("hello child"),
+                Some(SessionSource::SubAgent(SubAgentSource::ThreadSpawn {
+                    parent_thread_id,
+                    depth: 1,
+                    agent_nickname: None,
+                    agent_role: Some("explorer".to_string()),
+                })),
+            )
+            .await
+            .expect("child spawn should succeed");
+
+        let child_thread = harness
+            .manager
+            .get_thread(child_thread_id)
+            .await
+            .expect("child thread should be registered");
+        let snapshot = child_thread.config_snapshot().await;
+
+        let SessionSource::SubAgent(SubAgentSource::ThreadSpawn {
+            parent_thread_id: seen_parent_thread_id,
+            depth,
+            agent_nickname,
+            agent_role,
+        }) = snapshot.session_source
+        else {
+            panic!("expected thread-spawn sub-agent source");
+        };
+        assert_eq!(seen_parent_thread_id, parent_thread_id);
+        assert_eq!(depth, 1);
+        assert!(agent_nickname.is_some());
+        assert_eq!(agent_role, Some("explorer".to_string()));
+    }
+
+    #[tokio::test]
+    async fn resume_thread_subagent_restores_stored_nickname_and_role() {
+        let (home, mut config) = test_config().await;
+        config.features.enable(Feature::Sqlite);
+        let manager = ThreadManager::with_models_provider_and_home_for_tests(
+            CodexAuth::from_api_key("dummy"),
+            config.model_provider.clone(),
+            config.codex_home.clone(),
+        );
+        let control = manager.agent_control();
+        let harness = AgentControlHarness {
+            _home: home,
+            config,
+            manager,
+            control,
+        };
+        let (parent_thread_id, _parent_thread) = harness.start_thread().await;
+
+        let child_thread_id = harness
+            .control
+            .spawn_agent(
+                harness.config.clone(),
+                text_input("hello child"),
+                Some(SessionSource::SubAgent(SubAgentSource::ThreadSpawn {
+                    parent_thread_id,
+                    depth: 1,
+                    agent_nickname: None,
+                    agent_role: Some("explorer".to_string()),
+                })),
+            )
+            .await
+            .expect("child spawn should succeed");
+
+        let child_thread = harness
+            .manager
+            .get_thread(child_thread_id)
+            .await
+            .expect("child thread should exist");
+        let mut status_rx = harness
+            .control
+            .subscribe_status(child_thread_id)
+            .await
+            .expect("status subscription should succeed");
+        if matches!(status_rx.borrow().clone(), AgentStatus::PendingInit) {
+            timeout(Duration::from_secs(5), async {
+                loop {
+                    status_rx
+                        .changed()
+                        .await
+                        .expect("child status should advance past pending init");
+                    if !matches!(status_rx.borrow().clone(), AgentStatus::PendingInit) {
+                        break;
+                    }
+                }
+            })
+            .await
+            .expect("child should initialize before shutdown");
+        }
+        let original_snapshot = child_thread.config_snapshot().await;
+        let original_nickname = original_snapshot
+            .session_source
+            .get_nickname()
+            .expect("spawned sub-agent should have a nickname");
+        let state_db = child_thread
+            .state_db()
+            .expect("sqlite state db should be available for nickname resume test");
+        timeout(Duration::from_secs(5), async {
+            loop {
+                if let Ok(Some(metadata)) = state_db.get_thread(child_thread_id).await
+                    && metadata.agent_nickname.is_some()
+                    && metadata.agent_role.as_deref() == Some("explorer")
+                {
+                    break;
+                }
+                sleep(Duration::from_millis(10)).await;
+            }
+        })
+        .await
+        .expect("child thread metadata should be persisted to sqlite before shutdown");
+
+        let _ = harness
+            .control
+            .shutdown_agent(child_thread_id)
+            .await
+            .expect("child shutdown should submit");
+
+        let resumed_thread_id = harness
+            .control
+            .resume_agent_from_rollout(
+                harness.config.clone(),
+                child_thread_id,
+                SessionSource::SubAgent(SubAgentSource::ThreadSpawn {
+                    parent_thread_id,
+                    depth: 1,
+                    agent_nickname: None,
+                    agent_role: None,
+                }),
+            )
+            .await
+            .expect("resume should succeed");
+        assert_eq!(resumed_thread_id, child_thread_id);
+
+        let resumed_snapshot = harness
+            .manager
+            .get_thread(resumed_thread_id)
+            .await
+            .expect("resumed child thread should exist")
+            .config_snapshot()
+            .await;
+        let SessionSource::SubAgent(SubAgentSource::ThreadSpawn {
+            parent_thread_id: resumed_parent_thread_id,
+            depth: resumed_depth,
+            agent_nickname: resumed_nickname,
+            agent_role: resumed_role,
+        }) = resumed_snapshot.session_source
+        else {
+            panic!("expected thread-spawn sub-agent source");
+        };
+        assert_eq!(resumed_parent_thread_id, parent_thread_id);
+        assert_eq!(resumed_depth, 1);
+        assert_eq!(resumed_nickname, Some(original_nickname));
+        assert_eq!(resumed_role, Some("explorer".to_string()));
+
+        let _ = harness
+            .control
+            .shutdown_agent(resumed_thread_id)
+            .await
+            .expect("resumed child shutdown should submit");
+    }
 }
