@@ -34,16 +34,25 @@ pub(crate) fn build_track_events_context(
     }
 }
 
+#[derive(Clone, Debug)]
 pub(crate) struct SkillInvocation {
     pub(crate) skill_name: String,
     pub(crate) skill_scope: SkillScope,
     pub(crate) skill_path: PathBuf,
+    pub(crate) invocation_type: InvocationType,
+}
+
+#[derive(Clone, Copy, Debug, Serialize)]
+#[serde(rename_all = "lowercase")]
+pub(crate) enum InvocationType {
+    Explicit,
+    Implicit,
 }
 
 pub(crate) struct AppInvocation {
     pub(crate) connector_id: Option<String>,
     pub(crate) app_name: Option<String>,
-    pub(crate) invoke_type: Option<String>,
+    pub(crate) invocation_type: Option<InvocationType>,
 }
 
 #[derive(Clone)]
@@ -197,7 +206,7 @@ struct SkillInvocationEventParams {
     skill_scope: Option<String>,
     repo_url: Option<String>,
     thread_id: Option<String>,
-    invoke_type: Option<String>,
+    invoke_type: Option<InvocationType>,
     model_slug: Option<String>,
 }
 
@@ -208,7 +217,7 @@ struct CodexAppMetadata {
     turn_id: Option<String>,
     app_name: Option<String>,
     product_client_id: Option<String>,
-    invoke_type: Option<String>,
+    invoke_type: Option<InvocationType>,
     model_slug: Option<String>,
 }
 
@@ -328,7 +337,7 @@ async fn send_track_skill_invocations(auth_manager: &AuthManager, job: TrackSkil
                 skill_name: invocation.skill_name.clone(),
                 event_params: SkillInvocationEventParams {
                     thread_id: Some(tracking.thread_id.clone()),
-                    invoke_type: Some("explicit".to_string()),
+                    invoke_type: Some(invocation.invocation_type),
                     model_slug: Some(tracking.model_slug.clone()),
                     product_client_id: Some(crate::default_client::originator().value),
                     repo_url,
@@ -383,7 +392,7 @@ fn codex_app_metadata(tracking: &TrackEventsContext, app: AppInvocation) -> Code
         turn_id: Some(tracking.turn_id.clone()),
         app_name: app.app_name,
         product_client_id: Some(crate::default_client::originator().value),
-        invoke_type: app.invoke_type,
+        invoke_type: app.invocation_type,
         model_slug: Some(tracking.model_slug.clone()),
     }
 }
@@ -437,7 +446,7 @@ async fn send_track_events(
     }
 }
 
-fn skill_id_for_local_skill(
+pub(crate) fn skill_id_for_local_skill(
     repo_url: Option<&str>,
     repo_root: Option<&Path>,
     skill_path: &Path,
@@ -485,6 +494,7 @@ mod tests {
     use super::AppInvocation;
     use super::CodexAppMentionedEventRequest;
     use super::CodexAppUsedEventRequest;
+    use super::InvocationType;
     use super::TrackEventRequest;
     use super::TrackEventsContext;
     use super::codex_app_metadata;
@@ -567,7 +577,7 @@ mod tests {
                 AppInvocation {
                     connector_id: Some("calendar".to_string()),
                     app_name: Some("Calendar".to_string()),
-                    invoke_type: Some("explicit".to_string()),
+                    invocation_type: Some(InvocationType::Explicit),
                 },
             ),
         });
@@ -605,7 +615,7 @@ mod tests {
                 AppInvocation {
                     connector_id: Some("drive".to_string()),
                     app_name: Some("Google Drive".to_string()),
-                    invoke_type: Some("implicit".to_string()),
+                    invocation_type: Some(InvocationType::Implicit),
                 },
             ),
         });
@@ -639,7 +649,7 @@ mod tests {
         let app = AppInvocation {
             connector_id: Some("calendar".to_string()),
             app_name: Some("Calendar".to_string()),
-            invoke_type: Some("implicit".to_string()),
+            invocation_type: Some(InvocationType::Implicit),
         };
 
         let turn_1 = TrackEventsContext {
