@@ -1,10 +1,10 @@
 use super::LoaderOverrides;
-use super::diagnostics::config_error_from_toml;
-use super::diagnostics::io_error_from_config_error;
 #[cfg(target_os = "macos")]
 use super::macos::ManagedAdminConfigLayer;
 #[cfg(target_os = "macos")]
 use super::macos::load_managed_admin_config_layer;
+use codex_config::config_error_from_toml;
+use codex_config::io_error_from_config_error;
 use codex_utils_absolute_path::AbsolutePathBuf;
 use std::io;
 use std::path::Path;
