@@ -12,6 +12,8 @@ use codex_protocol::models::LocalShellExecAction;
 use codex_protocol::models::LocalShellStatus;
 use codex_protocol::models::ReasoningItemContent;
 use codex_protocol::models::ReasoningItemReasoningSummary;
+use codex_protocol::openai_models::InputModality;
+use codex_protocol::openai_models::default_input_modalities;
 use pretty_assertions::assert_eq;
 use regex_lite::Regex;
 
@@ -62,13 +64,6 @@ fn user_input_text_msg(text: &str) -> ResponseItem {
     }
 }
 
-fn function_call_output(call_id: &str, content: &str) -> ResponseItem {
-    ResponseItem::FunctionCallOutput {
-        call_id: call_id.to_string(),
-        output: FunctionCallOutputPayload::from_text(content.to_string()),
-    }
-}
-
 fn custom_tool_call_output(call_id: &str, output: &str) -> ResponseItem {
     ResponseItem::CustomToolCallOutput {
         call_id: call_id.to_string(),
@@ -189,48 +184,43 @@ fn non_last_reasoning_tokens_ignore_entries_after_last_user() {
 }
 
 #[test]
-fn trailing_codex_generated_tokens_stop_at_first_non_generated_item() {
-    let earlier_output = function_call_output("call-earlier", "earlier output");
-    let trailing_function_output = function_call_output("call-tail-1", "tail function output");
-    let trailing_custom_output = custom_tool_call_output("call-tail-2", "tail custom output");
+fn items_after_last_model_generated_tokens_include_user_and_tool_output() {
     let history = create_history_with_items(vec![
-        earlier_output,
-        user_msg("boundary item"),
-        trailing_function_output.clone(),
-        trailing_custom_output.clone(),
+        assistant_msg("already counted by API"),
+        user_msg("new user message"),
+        custom_tool_call_output("call-tail", "new tool output"),
     ]);
-    let expected_tokens = estimate_item_token_count(&trailing_function_output)
-        .saturating_add(estimate_item_token_count(&trailing_custom_output));
+    let expected_tokens = estimate_item_token_count(&user_msg("new user message")).saturating_add(
+        estimate_item_token_count(&custom_tool_call_output("call-tail", "new tool output")),
+    );
 
     assert_eq!(
-        history.get_trailing_codex_generated_items_tokens(),
+        history
+            .items_after_last_model_generated_item()
+            .iter()
+            .map(estimate_item_token_count)
+            .fold(0i64, i64::saturating_add),
         expected_tokens
     );
 }
 
 #[test]
-fn trailing_codex_generated_tokens_exclude_function_call_tail() {
-    let history = create_history_with_items(vec![ResponseItem::FunctionCall {
-        id: None,
-        name: "not-generated".to_string(),
-        arguments: "{}".to_string(),
-        call_id: "call-tail".to_string(),
-    }]);
+fn items_after_last_model_generated_tokens_are_zero_without_model_generated_items() {
+    let history = create_history_with_items(vec![user_msg("no model output yet")]);
 
-    assert_eq!(history.get_trailing_codex_generated_items_tokens(), 0);
+    assert_eq!(
+        history
+            .items_after_last_model_generated_item()
+            .iter()
+            .map(estimate_item_token_count)
+            .fold(0i64, i64::saturating_add),
+        0
+    );
 }
 
 #[test]
-fn total_token_usage_includes_only_trailing_codex_generated_items() {
-    let non_trailing_output = function_call_output("call-before-message", "not trailing");
-    let trailing_assistant = assistant_msg("assistant boundary");
-    let trailing_output = custom_tool_call_output("tool-tail", "trailing output");
-    let mut history = create_history_with_items(vec![
-        non_trailing_output,
-        user_msg("boundary"),
-        trailing_assistant,
-        trailing_output.clone(),
-    ]);
+fn total_token_usage_includes_all_items_after_last_model_generated_item() {
+    let mut history = create_history_with_items(vec![assistant_msg("already counted by API")]);
     history.update_token_info(
         &TokenUsage {
             total_tokens: 100,
@@ -238,20 +228,136 @@ fn total_token_usage_includes_only_trailing_codex_generated_items() {
         },
         None,
     );
+    let added_user = user_msg("new user message");
+    let added_tool_output = custom_tool_call_output("tool-tail", "new tool output");
+    history.record_items(
+        [&added_user, &added_tool_output],
+        TruncationPolicy::Tokens(10_000),
+    );
 
     assert_eq!(
         history.get_total_token_usage(true),
-        100 + estimate_item_token_count(&trailing_output)
+        100 + estimate_item_token_count(&added_user)
+            + estimate_item_token_count(&added_tool_output)
     );
 }
 
+#[test]
+fn for_prompt_strips_images_when_model_does_not_support_images() {
+    let items = vec![
+        ResponseItem::Message {
+            id: None,
+            role: "user".to_string(),
+            content: vec![
+                ContentItem::InputText {
+                    text: "look at this".to_string(),
+                },
+                ContentItem::InputImage {
+                    image_url: "https://example.com/img.png".to_string(),
+                },
+                ContentItem::InputText {
+                    text: "caption".to_string(),
+                },
+            ],
+            end_turn: None,
+            phase: None,
+        },
+        ResponseItem::FunctionCall {
+            id: None,
+            name: "view_image".to_string(),
+            arguments: "{}".to_string(),
+            call_id: "call-1".to_string(),
+        },
+        ResponseItem::FunctionCallOutput {
+            call_id: "call-1".to_string(),
+            output: FunctionCallOutputPayload::from_content_items(vec![
+                FunctionCallOutputContentItem::InputText {
+                    text: "image result".to_string(),
+                },
+                FunctionCallOutputContentItem::InputImage {
+                    image_url: "https://example.com/result.png".to_string(),
+                },
+            ]),
+        },
+    ];
+    let history = create_history_with_items(items);
+    let text_only_modalities = vec![InputModality::Text];
+    let stripped = history.for_prompt(&text_only_modalities);
+
+    let expected = vec![
+        ResponseItem::Message {
+            id: None,
+            role: "user".to_string(),
+            content: vec![
+                ContentItem::InputText {
+                    text: "look at this".to_string(),
+                },
+                ContentItem::InputText {
+                    text: "image content omitted because you do not support image input"
+                        .to_string(),
+                },
+                ContentItem::InputText {
+                    text: "caption".to_string(),
+                },
+            ],
+            end_turn: None,
+            phase: None,
+        },
+        ResponseItem::FunctionCall {
+            id: None,
+            name: "view_image".to_string(),
+            arguments: "{}".to_string(),
+            call_id: "call-1".to_string(),
+        },
+        ResponseItem::FunctionCallOutput {
+            call_id: "call-1".to_string(),
+            output: FunctionCallOutputPayload::from_content_items(vec![
+                FunctionCallOutputContentItem::InputText {
+                    text: "image result".to_string(),
+                },
+                FunctionCallOutputContentItem::InputText {
+                    text: "image content omitted because you do not support image input"
+                        .to_string(),
+                },
+            ]),
+        },
+    ];
+    assert_eq!(stripped, expected);
+
+    // With image support, images are preserved
+    let modalities = default_input_modalities();
+    let with_images = create_history_with_items(vec![ResponseItem::Message {
+        id: None,
+        role: "user".to_string(),
+        content: vec![
+            ContentItem::InputText {
+                text: "look".to_string(),
+            },
+            ContentItem::InputImage {
+                image_url: "https://example.com/img.png".to_string(),
+            },
+        ],
+        end_turn: None,
+        phase: None,
+    }]);
+    let preserved = with_images.for_prompt(&modalities);
+    assert_eq!(preserved.len(), 1);
+    if let ResponseItem::Message { content, .. } = &preserved[0] {
+        assert_eq!(content.len(), 2);
+        assert!(matches!(content[1], ContentItem::InputImage { .. }));
+    } else {
+        panic!("expected Message");
+    }
+}
+
 #[test]
 fn get_history_for_prompt_drops_ghost_commits() {
     let items = vec![ResponseItem::GhostSnapshot {
         ghost_commit: GhostCommit::new("ghost-1".to_string(), None, Vec::new(), Vec::new()),
     }];
     let history = create_history_with_items(items);
-    let filtered = history.for_prompt();
+    let modalities = default_input_modalities();
+    let filtered = history.for_prompt(&modalities);
     assert_eq!(filtered, vec![]);
 }
 
@@ -427,10 +533,11 @@ fn drop_last_n_user_turns_preserves_prefix() {
         assistant_msg("a2"),
     ];
 
+    let modalities = default_input_modalities();
     let mut history = create_history_with_items(items);
     history.drop_last_n_user_turns(1);
     assert_eq!(
-        history.for_prompt(),
+        history.for_prompt(&modalities),
         vec![
             assistant_msg("session prefix item"),
             user_msg("u1"),
@@ -447,7 +554,7 @@ fn drop_last_n_user_turns_preserves_prefix() {
     ]);
     history.drop_last_n_user_turns(99);
     assert_eq!(
-        history.for_prompt(),
+        history.for_prompt(&modalities),
         vec![assistant_msg("session prefix item")]
     );
 }
@@ -456,7 +563,6 @@ fn drop_last_n_user_turns_preserves_prefix() {
 fn drop_last_n_user_turns_ignores_session_prefix_user_messages() {
     let items = vec![
         user_input_text_msg("<environment_context>ctx</environment_context>"),
-        user_input_text_msg("<user_instructions>do the thing</user_instructions>"),
         user_input_text_msg(
             "# AGENTS.md instructions for test_directory\n\n<INSTRUCTIONS>\ntest_text\n</INSTRUCTIONS>",
         ),
@@ -464,18 +570,21 @@ fn drop_last_n_user_turns_ignores_session_prefix_user_messages() {
             "<skill>\n<name>demo</name>\n<path>skills/demo/SKILL.md</path>\nbody\n</skill>",
         ),
         user_input_text_msg("<user_shell_command>echo 42</user_shell_command>"),
+        user_input_text_msg(
+            "<subagent_notification>{\"agent_id\":\"a\",\"status\":\"completed\"}</subagent_notification>",
+        ),
         user_input_text_msg("turn 1 user"),
         assistant_msg("turn 1 assistant"),
         user_input_text_msg("turn 2 user"),
         assistant_msg("turn 2 assistant"),
     ];
 
+    let modalities = default_input_modalities();
     let mut history = create_history_with_items(items);
     history.drop_last_n_user_turns(1);
 
     let expected_prefix_and_first_turn = vec![
         user_input_text_msg("<environment_context>ctx</environment_context>"),
-        user_input_text_msg("<user_instructions>do the thing</user_instructions>"),
         user_input_text_msg(
             "# AGENTS.md instructions for test_directory\n\n<INSTRUCTIONS>\ntest_text\n</INSTRUCTIONS>",
         ),
@@ -483,15 +592,20 @@ fn drop_last_n_user_turns_ignores_session_prefix_user_messages() {
             "<skill>\n<name>demo</name>\n<path>skills/demo/SKILL.md</path>\nbody\n</skill>",
         ),
         user_input_text_msg("<user_shell_command>echo 42</user_shell_command>"),
+        user_input_text_msg(
+            "<subagent_notification>{\"agent_id\":\"a\",\"status\":\"completed\"}</subagent_notification>",
+        ),
         user_input_text_msg("turn 1 user"),
         assistant_msg("turn 1 assistant"),
     ];
 
-    assert_eq!(history.for_prompt(), expected_prefix_and_first_turn);
+    assert_eq!(
+        history.for_prompt(&modalities),
+        expected_prefix_and_first_turn
+    );
 
     let expected_prefix_only = vec![
         user_input_text_msg("<environment_context>ctx</environment_context>"),
-        user_input_text_msg("<user_instructions>do the thing</user_instructions>"),
         user_input_text_msg(
             "# AGENTS.md instructions for test_directory\n\n<INSTRUCTIONS>\ntest_text\n</INSTRUCTIONS>",
         ),
@@ -499,11 +613,13 @@ fn drop_last_n_user_turns_ignores_session_prefix_user_messages() {
             "<skill>\n<name>demo</name>\n<path>skills/demo/SKILL.md</path>\nbody\n</skill>",
         ),
         user_input_text_msg("<user_shell_command>echo 42</user_shell_command>"),
+        user_input_text_msg(
+            "<subagent_notification>{\"agent_id\":\"a\",\"status\":\"completed\"}</subagent_notification>",
+        ),
     ];
 
     let mut history = create_history_with_items(vec![
         user_input_text_msg("<environment_context>ctx</environment_context>"),
-        user_input_text_msg("<user_instructions>do the thing</user_instructions>"),
         user_input_text_msg(
             "# AGENTS.md instructions for test_directory\n\n<INSTRUCTIONS>\ntest_text\n</INSTRUCTIONS>",
         ),
@@ -511,17 +627,19 @@ fn drop_last_n_user_turns_ignores_session_prefix_user_messages() {
             "<skill>\n<name>demo</name>\n<path>skills/demo/SKILL.md</path>\nbody\n</skill>",
         ),
         user_input_text_msg("<user_shell_command>echo 42</user_shell_command>"),
+        user_input_text_msg(
+            "<subagent_notification>{\"agent_id\":\"a\",\"status\":\"completed\"}</subagent_notification>",
+        ),
         user_input_text_msg("turn 1 user"),
         assistant_msg("turn 1 assistant"),
         user_input_text_msg("turn 2 user"),
         assistant_msg("turn 2 assistant"),
     ]);
     history.drop_last_n_user_turns(2);
-    assert_eq!(history.for_prompt(), expected_prefix_only);
+    assert_eq!(history.for_prompt(&modalities), expected_prefix_only);
 
     let mut history = create_history_with_items(vec![
         user_input_text_msg("<environment_context>ctx</environment_context>"),
-        user_input_text_msg("<user_instructions>do the thing</user_instructions>"),
         user_input_text_msg(
             "# AGENTS.md instructions for test_directory\n\n<INSTRUCTIONS>\ntest_text\n</INSTRUCTIONS>",
         ),
@@ -529,13 +647,16 @@ fn drop_last_n_user_turns_ignores_session_prefix_user_messages() {
             "<skill>\n<name>demo</name>\n<path>skills/demo/SKILL.md</path>\nbody\n</skill>",
         ),
         user_input_text_msg("<user_shell_command>echo 42</user_shell_command>"),
+        user_input_text_msg(
+            "<subagent_notification>{\"agent_id\":\"a\",\"status\":\"completed\"}</subagent_notification>",
+        ),
         user_input_text_msg("turn 1 user"),
         assistant_msg("turn 1 assistant"),
         user_input_text_msg("turn 2 user"),
         assistant_msg("turn 2 assistant"),
     ]);
     history.drop_last_n_user_turns(3);
-    assert_eq!(history.for_prompt(), expected_prefix_only);
+    assert_eq!(history.for_prompt(&modalities), expected_prefix_only);
 }
 
 #[test]
@@ -579,8 +700,9 @@ fn normalization_retains_local_shell_outputs() {
         },
     ];
 
+    let modalities = default_input_modalities();
     let history = create_history_with_items(items.clone());
-    let normalized = history.for_prompt();
+    let normalized = history.for_prompt(&modalities);
     assert_eq!(normalized, items);
 }
 
@@ -782,7 +904,7 @@ fn normalize_adds_missing_output_for_function_call() {
     }];
     let mut h = create_history_with_items(items);
 
-    h.normalize_history();
+    h.normalize_history(&default_input_modalities());
 
     assert_eq!(
         h.raw_items(),
@@ -813,7 +935,7 @@ fn normalize_adds_missing_output_for_custom_tool_call() {
     }];
     let mut h = create_history_with_items(items);
 
-    h.normalize_history();
+    h.normalize_history(&default_input_modalities());
 
     assert_eq!(
         h.raw_items(),
@@ -850,7 +972,7 @@ fn normalize_adds_missing_output_for_local_shell_call_with_id() {
     }];
     let mut h = create_history_with_items(items);
 
-    h.normalize_history();
+    h.normalize_history(&default_input_modalities());
 
     assert_eq!(
         h.raw_items(),
@@ -884,7 +1006,7 @@ fn normalize_removes_orphan_function_call_output() {
     }];
     let mut h = create_history_with_items(items);
 
-    h.normalize_history();
+    h.normalize_history(&default_input_modalities());
 
     assert_eq!(h.raw_items(), vec![]);
 }
@@ -898,7 +1020,7 @@ fn normalize_removes_orphan_custom_tool_call_output() {
     }];
     let mut h = create_history_with_items(items);
 
-    h.normalize_history();
+    h.normalize_history(&default_input_modalities());
 
     assert_eq!(h.raw_items(), vec![]);
 }
@@ -943,7 +1065,7 @@ fn normalize_mixed_inserts_and_removals() {
     ];
     let mut h = create_history_with_items(items);
 
-    h.normalize_history();
+    h.normalize_history(&default_input_modalities());
 
     assert_eq!(
         h.raw_items(),
@@ -998,7 +1120,7 @@ fn normalize_adds_missing_output_for_function_call_inserts_output() {
         call_id: "call-x".to_string(),
     }];
     let mut h = create_history_with_items(items);
-    h.normalize_history();
+    h.normalize_history(&default_input_modalities());
     assert_eq!(
         h.raw_items(),
         vec![
@@ -1028,7 +1150,7 @@ fn normalize_adds_missing_output_for_custom_tool_call_panics_in_debug() {
         input: "{}".to_string(),
     }];
     let mut h = create_history_with_items(items);
-    h.normalize_history();
+    h.normalize_history(&default_input_modalities());
 }
 
 #[cfg(debug_assertions)]
@@ -1048,7 +1170,7 @@ fn normalize_adds_missing_output_for_local_shell_call_with_id_panics_in_debug()
         }),
     }];
     let mut h = create_history_with_items(items);
-    h.normalize_history();
+    h.normalize_history(&default_input_modalities());
 }
 
 #[cfg(debug_assertions)]
@@ -1060,7 +1182,7 @@ fn normalize_removes_orphan_function_call_output_panics_in_debug() {
         output: FunctionCallOutputPayload::from_text("ok".to_string()),
     }];
     let mut h = create_history_with_items(items);
-    h.normalize_history();
+    h.normalize_history(&default_input_modalities());
 }
 
 #[cfg(debug_assertions)]
@@ -1072,7 +1194,7 @@ fn normalize_removes_orphan_custom_tool_call_output_panics_in_debug() {
         output: "ok".to_string(),
     }];
     let mut h = create_history_with_items(items);
-    h.normalize_history();
+    h.normalize_history(&default_input_modalities());
 }
 
 #[cfg(debug_assertions)]
@@ -1111,5 +1233,197 @@ fn normalize_mixed_inserts_and_removals_panics_in_debug() {
         },
     ];
     let mut h = create_history_with_items(items);
-    h.normalize_history();
+    h.normalize_history(&default_input_modalities());
+}
+
+#[test]
+fn image_data_url_payload_does_not_dominate_message_estimate() {
+    let payload = "A".repeat(100_000);
+    let image_url = format!("data:image/png;base64,{payload}");
+    let image_item = ResponseItem::Message {
+        id: None,
+        role: "user".to_string(),
+        content: vec![
+            ContentItem::InputText {
+                text: "Here is the screenshot".to_string(),
+            },
+            ContentItem::InputImage { image_url },
+        ],
+        end_turn: None,
+        phase: None,
+    };
+    let text_only_item = ResponseItem::Message {
+        id: None,
+        role: "user".to_string(),
+        content: vec![ContentItem::InputText {
+            text: "Here is the screenshot".to_string(),
+        }],
+        end_turn: None,
+        phase: None,
+    };
+
+    let raw_len = serde_json::to_string(&image_item).unwrap().len() as i64;
+    let estimated = estimate_response_item_model_visible_bytes(&image_item);
+    let expected = raw_len - payload.len() as i64 + IMAGE_BYTES_ESTIMATE;
+    let text_only_estimated = estimate_response_item_model_visible_bytes(&text_only_item);
+
+    assert_eq!(estimated, expected);
+    assert!(estimated < raw_len);
+    assert!(estimated > text_only_estimated);
+}
+
+#[test]
+fn image_data_url_payload_does_not_dominate_function_call_output_estimate() {
+    let payload = "B".repeat(50_000);
+    let image_url = format!("data:image/png;base64,{payload}");
+    let item = ResponseItem::FunctionCallOutput {
+        call_id: "call-abc".to_string(),
+        output: FunctionCallOutputPayload::from_content_items(vec![
+            FunctionCallOutputContentItem::InputText {
+                text: "Screenshot captured".to_string(),
+            },
+            FunctionCallOutputContentItem::InputImage { image_url },
+        ]),
+    };
+
+    let raw_len = serde_json::to_string(&item).unwrap().len() as i64;
+    let estimated = estimate_response_item_model_visible_bytes(&item);
+    let expected = raw_len - payload.len() as i64 + IMAGE_BYTES_ESTIMATE;
+
+    assert_eq!(estimated, expected);
+    assert!(estimated < raw_len);
+}
+
+#[test]
+fn non_base64_image_urls_are_unchanged() {
+    let message_item = ResponseItem::Message {
+        id: None,
+        role: "user".to_string(),
+        content: vec![ContentItem::InputImage {
+            image_url: "https://example.com/foo.png".to_string(),
+        }],
+        end_turn: None,
+        phase: None,
+    };
+    let function_output_item = ResponseItem::FunctionCallOutput {
+        call_id: "call-1".to_string(),
+        output: FunctionCallOutputPayload::from_content_items(vec![
+            FunctionCallOutputContentItem::InputImage {
+                image_url: "file:///tmp/foo.png".to_string(),
+            },
+        ]),
+    };
+
+    assert_eq!(
+        estimate_response_item_model_visible_bytes(&message_item),
+        serde_json::to_string(&message_item).unwrap().len() as i64
+    );
+    assert_eq!(
+        estimate_response_item_model_visible_bytes(&function_output_item),
+        serde_json::to_string(&function_output_item).unwrap().len() as i64
+    );
+}
+
+#[test]
+fn data_url_without_base64_marker_is_unchanged() {
+    let item = ResponseItem::Message {
+        id: None,
+        role: "user".to_string(),
+        content: vec![ContentItem::InputImage {
+            image_url: "data:image/svg+xml,<svg xmlns='http://www.w3.org/2000/svg'/>".to_string(),
+        }],
+        end_turn: None,
+        phase: None,
+    };
+
+    assert_eq!(
+        estimate_response_item_model_visible_bytes(&item),
+        serde_json::to_string(&item).unwrap().len() as i64
+    );
+}
+
+#[test]
+fn non_image_base64_data_url_is_unchanged() {
+    let payload = "C".repeat(4_096);
+    let image_url = format!("data:application/octet-stream;base64,{payload}");
+    let item = ResponseItem::FunctionCallOutput {
+        call_id: "call-octet".to_string(),
+        output: FunctionCallOutputPayload::from_content_items(vec![
+            FunctionCallOutputContentItem::InputImage { image_url },
+        ]),
+    };
+
+    let raw_len = serde_json::to_string(&item).unwrap().len() as i64;
+    let estimated = estimate_response_item_model_visible_bytes(&item);
+
+    assert_eq!(estimated, raw_len);
+}
+
+#[test]
+fn mixed_case_data_url_markers_are_adjusted() {
+    let payload = "F".repeat(1_024);
+    let image_url = format!("DATA:image/png;BASE64,{payload}");
+    let item = ResponseItem::Message {
+        id: None,
+        role: "user".to_string(),
+        content: vec![ContentItem::InputImage { image_url }],
+        end_turn: None,
+        phase: None,
+    };
+
+    let raw_len = serde_json::to_string(&item).unwrap().len() as i64;
+    let estimated = estimate_response_item_model_visible_bytes(&item);
+    let expected = raw_len - payload.len() as i64 + IMAGE_BYTES_ESTIMATE;
+
+    assert_eq!(estimated, expected);
+}
+
+#[test]
+fn multiple_inline_images_apply_multiple_fixed_costs() {
+    let payload_one = "D".repeat(100);
+    let payload_two = "E".repeat(200);
+    let image_url_one = format!("data:image/png;base64,{payload_one}");
+    let image_url_two = format!("data:image/jpeg;base64,{payload_two}");
+    let item = ResponseItem::Message {
+        id: None,
+        role: "user".to_string(),
+        content: vec![
+            ContentItem::InputText {
+                text: "images".to_string(),
+            },
+            ContentItem::InputImage {
+                image_url: image_url_one,
+            },
+            ContentItem::InputImage {
+                image_url: image_url_two,
+            },
+        ],
+        end_turn: None,
+        phase: None,
+    };
+
+    let raw_len = serde_json::to_string(&item).unwrap().len() as i64;
+    let payload_sum = (payload_one.len() + payload_two.len()) as i64;
+    let estimated = estimate_response_item_model_visible_bytes(&item);
+    let expected = raw_len - payload_sum + (2 * IMAGE_BYTES_ESTIMATE);
+
+    assert_eq!(estimated, expected);
+}
+
+#[test]
+fn text_only_items_unchanged() {
+    let item = ResponseItem::Message {
+        id: None,
+        role: "assistant".to_string(),
+        content: vec![ContentItem::OutputText {
+            text: "Hello world, this is a response.".to_string(),
+        }],
+        end_turn: None,
+        phase: None,
+    };
+
+    let estimated = estimate_response_item_model_visible_bytes(&item);
+    let raw_len = serde_json::to_string(&item).unwrap().len() as i64;
+
+    assert_eq!(estimated, raw_len);
 }
