@@ -15,7 +15,6 @@ use tokio::io::AsyncReadExt;
 use tokio::io::BufReader;
 use tokio::process::Child;
 use tokio_util::sync::CancellationToken;
-use uuid::Uuid;
 
 use crate::error::CodexErr;
 use crate::error::Result;
@@ -67,7 +66,6 @@ pub struct ExecParams {
     pub expiration: ExecExpiration,
     pub env: HashMap<String, String>,
     pub network: Option<NetworkProxy>,
-    pub network_attempt_id: Option<Uuid>,
     pub sandbox_permissions: SandboxPermissions,
     pub windows_sandbox_level: codex_protocol::config_types::WindowsSandboxLevel,
     pub justification: Option<String>,
@@ -75,7 +73,7 @@ pub struct ExecParams {
 }
 
 /// Mechanism to terminate an exec invocation before it finishes naturally.
-#[derive(Debug)]
+#[derive(Clone, Debug)]
 pub enum ExecExpiration {
     Timeout(Duration),
     DefaultTimeout,
@@ -97,7 +95,7 @@ impl From<u64> for ExecExpiration {
 }
 
 impl ExecExpiration {
-    async fn wait(self) {
+    pub(crate) async fn wait(self) {
         match self {
             ExecExpiration::Timeout(duration) => tokio::time::sleep(duration).await,
             ExecExpiration::DefaultTimeout => {
@@ -186,15 +184,13 @@ pub async fn process_exec_tool_call(
         mut env,
         expiration,
         network,
-        network_attempt_id,
         sandbox_permissions,
         windows_sandbox_level,
         justification,
         arg0: _,
     } = params;
-    let network_attempt_id = network_attempt_id.map(|attempt_id| attempt_id.to_string());
     if let Some(network) = network.as_ref() {
-        network.apply_to_env_for_attempt(&mut env, network_attempt_id.as_deref());
+        network.apply_to_env(&mut env);
     }
     let (program, args) = command.split_first().ok_or_else(|| {
         CodexErr::Io(io::Error::new(
@@ -210,6 +206,7 @@ pub async fn process_exec_tool_call(
         env,
         expiration,
         sandbox_permissions,
+        additional_permissions: None,
         justification,
     };
 
@@ -229,7 +226,7 @@ pub async fn process_exec_tool_call(
         .map_err(CodexErr::from)?;
 
     // Route through the sandboxing module for a single, unified execution path.
-    crate::sandboxing::execute_env(exec_req, sandbox_policy, stdout_stream).await
+    crate::sandboxing::execute_env(exec_req, stdout_stream).await
 }
 
 pub(crate) async fn execute_exec_env(
@@ -242,27 +239,21 @@ pub(crate) async fn execute_exec_env(
         cwd,
         env,
         network,
-        network_attempt_id,
         expiration,
         sandbox,
         windows_sandbox_level,
         sandbox_permissions,
+        sandbox_policy: _sandbox_policy_from_env,
         justification,
         arg0,
     } = env;
 
-    let network_attempt_id = match network_attempt_id.as_deref() {
-        Some(attempt_id) => Uuid::parse_str(attempt_id).ok(),
-        None => network.as_ref().map(|_| Uuid::new_v4()),
-    };
-
     let params = ExecParams {
         command,
         cwd,
         expiration,
         env,
         network: network.clone(),
-        network_attempt_id,
         sandbox_permissions,
         windows_sandbox_level,
         justification,
@@ -356,14 +347,12 @@ async fn exec_windows_sandbox(
         cwd,
         mut env,
         network,
-        network_attempt_id,
         expiration,
         windows_sandbox_level,
         ..
     } = params;
-    let network_attempt_id = network_attempt_id.map(|attempt_id| attempt_id.to_string());
     if let Some(network) = network.as_ref() {
-        network.apply_to_env_for_attempt(&mut env, network_attempt_id.as_deref());
+        network.apply_to_env(&mut env);
     }
 
     // TODO(iceweasel-oai): run_windows_sandbox_capture should support all
@@ -530,6 +519,9 @@ pub(crate) mod errors {
                 SandboxTransformError::SeatbeltUnavailable => CodexErr::UnsupportedOperation(
                     "seatbelt sandbox is only available on macOS".to_string(),
                 ),
+                SandboxTransformError::InvalidAdditionalPermissionsPath(path) => {
+                    CodexErr::InvalidRequest(format!("invalid additional_permissions path: {path}"))
+                }
             }
         }
     }
@@ -717,16 +709,13 @@ async fn exec(
         cwd,
         mut env,
         network,
-        network_attempt_id,
         arg0,
         expiration,
         windows_sandbox_level: _,
         ..
     } = params;
-    let network_attempt_id = network_attempt_id.map(|attempt_id| attempt_id.to_string());
-
     if let Some(network) = network.as_ref() {
-        network.apply_to_env_for_attempt(&mut env, network_attempt_id.as_deref());
+        network.apply_to_env(&mut env);
     }
 
     let (program, args) = command.split_first().ok_or_else(|| {
@@ -1137,7 +1126,6 @@ mod tests {
             expiration: 500.into(),
             env,
             network: None,
-            network_attempt_id: None,
             sandbox_permissions: SandboxPermissions::UseDefault,
             windows_sandbox_level: codex_protocol::config_types::WindowsSandboxLevel::Disabled,
             justification: None,
@@ -1191,7 +1179,6 @@ mod tests {
             expiration: ExecExpiration::Cancellation(cancel_token),
             env,
             network: None,
-            network_attempt_id: None,
             sandbox_permissions: SandboxPermissions::UseDefault,
             windows_sandbox_level: codex_protocol::config_types::WindowsSandboxLevel::Disabled,
             justification: None,
