@@ -13,10 +13,12 @@ use codex_execpolicy::AmendError;
 use codex_execpolicy::Decision;
 use codex_execpolicy::Error as ExecPolicyRuleError;
 use codex_execpolicy::Evaluation;
+use codex_execpolicy::NetworkRuleProtocol;
 use codex_execpolicy::Policy;
 use codex_execpolicy::PolicyParser;
 use codex_execpolicy::RuleMatch;
 use codex_execpolicy::blocking_append_allow_prefix_rule;
+use codex_execpolicy::blocking_append_network_rule;
 use codex_protocol::approvals::ExecPolicyAmendment;
 use codex_protocol::protocol::AskForApproval;
 use codex_protocol::protocol::SandboxPolicy;
@@ -32,6 +34,10 @@ use shlex::try_join as shlex_try_join;
 
 const PROMPT_CONFLICT_REASON: &str =
     "approval required by policy, but AskForApproval is set to Never";
+const REJECT_SANDBOX_APPROVAL_REASON: &str =
+    "approval required by policy, but AskForApproval::Reject.sandbox_approval is set";
+const REJECT_RULES_APPROVAL_REASON: &str =
+    "approval required by policy rule, but AskForApproval::Reject.rules is set";
 const RULES_DIR_NAME: &str = "rules";
 const RULE_EXTENSION: &str = "rules";
 const DEFAULT_POLICY_FILE: &str = "default.rules";
@@ -91,6 +97,37 @@ fn is_policy_match(rule_match: &RuleMatch) -> bool {
     }
 }
 
+/// Returns a rejection reason when `approval_policy` disallows surfacing the
+/// current prompt to the user.
+///
+/// `prompt_is_rule` distinguishes policy-rule prompts from sandbox/escalation
+/// prompts so `Reject.rules` and `Reject.sandbox_approval` are honored
+/// independently. When both are present, policy-rule prompts take precedence.
+fn prompt_is_rejected_by_policy(
+    approval_policy: AskForApproval,
+    prompt_is_rule: bool,
+) -> Option<&'static str> {
+    match approval_policy {
+        AskForApproval::Never => Some(PROMPT_CONFLICT_REASON),
+        AskForApproval::OnFailure => None,
+        AskForApproval::OnRequest => None,
+        AskForApproval::UnlessTrusted => None,
+        AskForApproval::Reject(reject_config) => {
+            if prompt_is_rule {
+                if reject_config.rejects_rules_approval() {
+                    Some(REJECT_RULES_APPROVAL_REASON)
+                } else {
+                    None
+                }
+            } else if reject_config.rejects_sandbox_approval() {
+                Some(REJECT_SANDBOX_APPROVAL_REASON)
+            } else {
+                None
+            }
+        }
+    }
+}
+
 #[derive(Debug, Error)]
 pub enum ExecPolicyError {
     #[error("failed to read rules files from {dir}: {source}")]
@@ -189,6 +226,9 @@ impl ExecPolicyManager {
         let requested_amendment = derive_requested_execpolicy_amendment_from_prefix_rule(
             prefix_rule.as_ref(),
             &evaluation.matched_rules,
+            exec_policy.as_ref(),
+            &commands,
+            &exec_policy_fallback,
         );
 
         match evaluation.decision {
@@ -196,12 +236,14 @@ impl ExecPolicyManager {
                 reason: derive_forbidden_reason(command, &evaluation),
             },
             Decision::Prompt => {
-                if matches!(approval_policy, AskForApproval::Never) {
-                    ExecApprovalRequirement::Forbidden {
-                        reason: PROMPT_CONFLICT_REASON.to_string(),
-                    }
-                } else {
-                    ExecApprovalRequirement::NeedsApproval {
+                let prompt_is_rule = evaluation.matched_rules.iter().any(|rule_match| {
+                    is_policy_match(rule_match) && rule_match.decision() == Decision::Prompt
+                });
+                match prompt_is_rejected_by_policy(approval_policy, prompt_is_rule) {
+                    Some(reason) => ExecApprovalRequirement::Forbidden {
+                        reason: reason.to_string(),
+                    },
+                    None => ExecApprovalRequirement::NeedsApproval {
                         reason: derive_prompt_reason(command, &evaluation),
                         proposed_execpolicy_amendment: requested_amendment.or_else(|| {
                             if auto_amendment_allowed {
@@ -212,7 +254,7 @@ impl ExecPolicyManager {
                                 None
                             }
                         }),
-                    }
+                    },
                 }
             }
             Decision::Allow => ExecApprovalRequirement::Skip {
@@ -253,6 +295,43 @@ impl ExecPolicyManager {
         self.policy.store(Arc::new(updated_policy));
         Ok(())
     }
+
+    pub(crate) async fn append_network_rule_and_update(
+        &self,
+        codex_home: &Path,
+        host: &str,
+        protocol: NetworkRuleProtocol,
+        decision: Decision,
+        justification: Option<String>,
+    ) -> Result<(), ExecPolicyUpdateError> {
+        let policy_path = default_policy_path(codex_home);
+        let host = host.to_string();
+        spawn_blocking({
+            let policy_path = policy_path.clone();
+            let host = host.clone();
+            let justification = justification.clone();
+            move || {
+                blocking_append_network_rule(
+                    &policy_path,
+                    &host,
+                    protocol,
+                    decision,
+                    justification.as_deref(),
+                )
+            }
+        })
+        .await
+        .map_err(|source| ExecPolicyUpdateError::JoinBlockingTask { source })?
+        .map_err(|source| ExecPolicyUpdateError::AppendRule {
+            path: policy_path,
+            source,
+        })?;
+
+        let mut updated_policy = self.current().as_ref().clone();
+        updated_policy.add_network_rule(&host, protocol, decision, justification)?;
+        self.policy.store(Arc::new(updated_policy));
+        Ok(())
+    }
 }
 
 impl Default for ExecPolicyManager {
@@ -400,7 +479,10 @@ pub async fn load_exec_policy(config_stack: &ConfigLayerStack) -> Result<Policy,
         }
     }
 
-    Ok(Policy::new(combined_rules))
+    let mut combined_network_rules = policy.network_rules().to_vec();
+    combined_network_rules.extend(requirements_policy.as_ref().network_rules().iter().cloned());
+
+    Ok(Policy::from_parts(combined_rules, combined_network_rules))
 }
 
 /// If a command is not matched by any execpolicy rule, derive a [`Decision`].
@@ -457,7 +539,7 @@ pub fn render_decision_for_unmatched_command(
                     // In restricted sandboxes (ReadOnly/WorkspaceWrite), do not prompt for
                     // non‑escalated, non‑dangerous commands — let the sandbox enforce
                     // restrictions (e.g., block network/write) without a user prompt.
-                    if sandbox_permissions.requires_escalated_permissions() {
+                    if sandbox_permissions.requires_additional_permissions() {
                         Decision::Prompt
                     } else {
                         Decision::Allow
@@ -465,6 +547,20 @@ pub fn render_decision_for_unmatched_command(
                 }
             }
         }
+        AskForApproval::Reject(_) => match sandbox_policy {
+            SandboxPolicy::DangerFullAccess | SandboxPolicy::ExternalSandbox { .. } => {
+                // Mirror on-request behavior for unmatched commands; prompt-vs-reject is handled
+                // by `prompt_is_rejected_by_policy`.
+                Decision::Allow
+            }
+            SandboxPolicy::ReadOnly { .. } | SandboxPolicy::WorkspaceWrite { .. } => {
+                if sandbox_permissions.requires_additional_permissions() {
+                    Decision::Prompt
+                } else {
+                    Decision::Allow
+                }
+            }
+        },
     }
 }
 
@@ -541,6 +637,9 @@ fn try_derive_execpolicy_amendment_for_allow_rules(
 fn derive_requested_execpolicy_amendment_from_prefix_rule(
     prefix_rule: Option<&Vec<String>>,
     matched_rules: &[RuleMatch],
+    exec_policy: &Policy,
+    commands: &[Vec<String>],
+    exec_policy_fallback: &impl Fn(&[String]) -> Decision,
 ) -> Option<ExecPolicyAmendment> {
     let prefix_rule = prefix_rule?;
     if prefix_rule.is_empty() {
@@ -561,7 +660,39 @@ fn derive_requested_execpolicy_amendment_from_prefix_rule(
         return None;
     }
 
-    Some(ExecPolicyAmendment::new(prefix_rule.clone()))
+    let amendment = ExecPolicyAmendment::new(prefix_rule.clone());
+    if prefix_rule_would_approve_all_commands(
+        exec_policy,
+        &amendment.command,
+        commands,
+        exec_policy_fallback,
+    ) {
+        Some(amendment)
+    } else {
+        None
+    }
+}
+
+fn prefix_rule_would_approve_all_commands(
+    exec_policy: &Policy,
+    prefix_rule: &[String],
+    commands: &[Vec<String>],
+    exec_policy_fallback: &impl Fn(&[String]) -> Decision,
+) -> bool {
+    let mut policy_with_prefix_rule = exec_policy.clone();
+    if policy_with_prefix_rule
+        .add_prefix_rule(prefix_rule, Decision::Allow)
+        .is_err()
+    {
+        return false;
+    }
+
+    commands.iter().all(|command| {
+        policy_with_prefix_rule
+            .check(command, exec_policy_fallback)
+            .decision
+            == Decision::Allow
+    })
 }
 
 /// Only return a reason when a policy rule drove the prompt decision.
@@ -690,6 +821,7 @@ mod tests {
     use crate::config_loader::ConfigRequirementsToml;
     use codex_app_server_protocol::ConfigLayerSource;
     use codex_protocol::protocol::AskForApproval;
+    use codex_protocol::protocol::RejectConfig;
     use codex_protocol::protocol::SandboxPolicy;
     use codex_utils_absolute_path::AbsolutePathBuf;
     use pretty_assertions::assert_eq;
@@ -824,6 +956,41 @@ mod tests {
         );
     }
 
+    #[tokio::test]
+    async fn merges_requirements_exec_policy_network_rules() -> anyhow::Result<()> {
+        let temp_dir = tempdir()?;
+
+        let mut requirements_exec_policy = Policy::empty();
+        requirements_exec_policy.add_network_rule(
+            "blocked.example.com",
+            codex_execpolicy::NetworkRuleProtocol::Https,
+            Decision::Forbidden,
+            None,
+        )?;
+
+        let requirements = ConfigRequirements {
+            exec_policy: Some(codex_config::Sourced::new(
+                codex_config::RequirementsExecPolicy::new(requirements_exec_policy),
+                codex_config::RequirementSource::Unknown,
+            )),
+            ..ConfigRequirements::default()
+        };
+        let dot_codex_folder = AbsolutePathBuf::from_absolute_path(temp_dir.path())?;
+        let layer = ConfigLayerEntry::new(
+            ConfigLayerSource::Project { dot_codex_folder },
+            TomlValue::Table(Default::default()),
+        );
+        let config_stack =
+            ConfigLayerStack::new(vec![layer], requirements, ConfigRequirementsToml::default())?;
+
+        let policy = load_exec_policy(&config_stack).await?;
+        let (allowed, denied) = policy.compiled_network_domains();
+
+        assert!(allowed.is_empty());
+        assert_eq!(denied, vec!["blocked.example.com".to_string()]);
+        Ok(())
+    }
+
     #[tokio::test]
     async fn ignores_policies_outside_policy_dir() {
         let temp_dir = tempdir().expect("create temp dir");
@@ -1073,7 +1240,7 @@ prefix_rule(pattern=["rm"], decision="forbidden")
     }
 
     #[tokio::test]
-    async fn keeps_requested_amendment_for_heredoc_fallback_prompts() {
+    async fn drops_requested_amendment_for_heredoc_fallback_prompts_when_it_wont_match() {
         let command = vec![
             "bash".to_string(),
             "-lc".to_string(),
@@ -1095,7 +1262,7 @@ prefix_rule(pattern=["rm"], decision="forbidden")
             requirement,
             ExecApprovalRequirement::NeedsApproval {
                 reason: None,
-                proposed_execpolicy_amendment: Some(ExecPolicyAmendment::new(requested_prefix)),
+                proposed_execpolicy_amendment: None,
             }
         );
     }
@@ -1197,6 +1364,122 @@ prefix_rule(
         );
     }
 
+    #[test]
+    fn unmatched_reject_policy_still_prompts_for_restricted_sandbox_escalation() {
+        let command = vec!["madeup-cmd".to_string()];
+
+        assert_eq!(
+            Decision::Prompt,
+            render_decision_for_unmatched_command(
+                AskForApproval::Reject(RejectConfig {
+                    sandbox_approval: false,
+                    rules: false,
+                    mcp_elicitations: false,
+                }),
+                &SandboxPolicy::new_read_only_policy(),
+                &command,
+                SandboxPermissions::RequireEscalated,
+                false,
+            )
+        );
+    }
+
+    #[tokio::test]
+    async fn exec_approval_requirement_rejects_unmatched_prompt_when_sandbox_rejection_enabled() {
+        let command = vec!["madeup-cmd".to_string()];
+
+        let requirement = ExecPolicyManager::default()
+            .create_exec_approval_requirement_for_command(ExecApprovalRequest {
+                command: &command,
+                approval_policy: AskForApproval::Reject(RejectConfig {
+                    sandbox_approval: true,
+                    rules: false,
+                    mcp_elicitations: false,
+                }),
+                sandbox_policy: &SandboxPolicy::new_read_only_policy(),
+                sandbox_permissions: SandboxPermissions::RequireEscalated,
+                prefix_rule: None,
+            })
+            .await;
+
+        assert_eq!(
+            requirement,
+            ExecApprovalRequirement::Forbidden {
+                reason: REJECT_SANDBOX_APPROVAL_REASON.to_string(),
+            }
+        );
+    }
+
+    #[tokio::test]
+    async fn mixed_rule_and_sandbox_prompt_prioritizes_rule_for_rejection_decision() {
+        let policy_src = r#"prefix_rule(pattern=["git"], decision="prompt")"#;
+        let mut parser = PolicyParser::new();
+        parser
+            .parse("test.rules", policy_src)
+            .expect("parse policy");
+        let manager = ExecPolicyManager::new(Arc::new(parser.build()));
+        let command = vec![
+            "bash".to_string(),
+            "-lc".to_string(),
+            "git status && madeup-cmd".to_string(),
+        ];
+
+        let requirement = manager
+            .create_exec_approval_requirement_for_command(ExecApprovalRequest {
+                command: &command,
+                approval_policy: AskForApproval::Reject(RejectConfig {
+                    sandbox_approval: true,
+                    rules: false,
+                    mcp_elicitations: false,
+                }),
+                sandbox_policy: &SandboxPolicy::new_read_only_policy(),
+                sandbox_permissions: SandboxPermissions::RequireEscalated,
+                prefix_rule: None,
+            })
+            .await;
+
+        assert!(matches!(
+            requirement,
+            ExecApprovalRequirement::NeedsApproval { .. }
+        ));
+    }
+
+    #[tokio::test]
+    async fn mixed_rule_and_sandbox_prompt_rejects_when_rules_rejection_enabled() {
+        let policy_src = r#"prefix_rule(pattern=["git"], decision="prompt")"#;
+        let mut parser = PolicyParser::new();
+        parser
+            .parse("test.rules", policy_src)
+            .expect("parse policy");
+        let manager = ExecPolicyManager::new(Arc::new(parser.build()));
+        let command = vec![
+            "bash".to_string(),
+            "-lc".to_string(),
+            "git status && madeup-cmd".to_string(),
+        ];
+
+        let requirement = manager
+            .create_exec_approval_requirement_for_command(ExecApprovalRequest {
+                command: &command,
+                approval_policy: AskForApproval::Reject(RejectConfig {
+                    sandbox_approval: false,
+                    rules: true,
+                    mcp_elicitations: false,
+                }),
+                sandbox_policy: &SandboxPolicy::new_read_only_policy(),
+                sandbox_permissions: SandboxPermissions::RequireEscalated,
+                prefix_rule: None,
+            })
+            .await;
+
+        assert_eq!(
+            requirement,
+            ExecApprovalRequirement::Forbidden {
+                reason: REJECT_RULES_APPROVAL_REASON.to_string(),
+            }
+        );
+    }
+
     #[tokio::test]
     async fn exec_approval_requirement_falls_back_to_heuristics() {
         let command = vec!["cargo".to_string(), "build".to_string()];
@@ -1304,6 +1587,38 @@ prefix_rule(
         );
     }
 
+    #[tokio::test]
+    async fn request_rule_falls_back_when_prefix_rule_does_not_approve_all_commands() {
+        let command = vec![
+            "bash".to_string(),
+            "-lc".to_string(),
+            "cargo install cargo-insta && rm -rf /tmp/codex".to_string(),
+        ];
+        let manager = ExecPolicyManager::default();
+
+        let requirement = manager
+            .create_exec_approval_requirement_for_command(ExecApprovalRequest {
+                command: &command,
+                approval_policy: AskForApproval::OnRequest,
+                sandbox_policy: &SandboxPolicy::DangerFullAccess,
+                sandbox_permissions: SandboxPermissions::RequireEscalated,
+                prefix_rule: Some(vec!["cargo".to_string(), "install".to_string()]),
+            })
+            .await;
+
+        assert_eq!(
+            requirement,
+            ExecApprovalRequirement::NeedsApproval {
+                reason: None,
+                proposed_execpolicy_amendment: Some(ExecPolicyAmendment::new(vec![
+                    "rm".to_string(),
+                    "-rf".to_string(),
+                    "/tmp/codex".to_string(),
+                ])),
+            }
+        );
+    }
+
     #[tokio::test]
     async fn heuristics_apply_when_other_commands_match_policy() {
         let policy_src = r#"prefix_rule(pattern=["apple"], decision="allow")"#;
@@ -1560,11 +1875,28 @@ prefix_rule(
         );
     }
 
+    fn derive_requested_execpolicy_amendment_for_test(
+        prefix_rule: Option<&Vec<String>>,
+        matched_rules: &[RuleMatch],
+    ) -> Option<ExecPolicyAmendment> {
+        let commands = prefix_rule
+            .cloned()
+            .map(|prefix_rule| vec![prefix_rule])
+            .unwrap_or_else(|| vec![vec!["echo".to_string()]]);
+        derive_requested_execpolicy_amendment_from_prefix_rule(
+            prefix_rule,
+            matched_rules,
+            &Policy::empty(),
+            &commands,
+            &|_: &[String]| Decision::Allow,
+        )
+    }
+
     #[test]
     fn derive_requested_execpolicy_amendment_returns_none_for_missing_prefix_rule() {
         assert_eq!(
             None,
-            derive_requested_execpolicy_amendment_from_prefix_rule(None, &[])
+            derive_requested_execpolicy_amendment_for_test(None, &[])
         );
     }
 
@@ -1572,7 +1904,7 @@ prefix_rule(
     fn derive_requested_execpolicy_amendment_returns_none_for_empty_prefix_rule() {
         assert_eq!(
             None,
-            derive_requested_execpolicy_amendment_from_prefix_rule(Some(&Vec::new()), &[])
+            derive_requested_execpolicy_amendment_for_test(Some(&Vec::new()), &[])
         );
     }
 
@@ -1580,7 +1912,7 @@ prefix_rule(
     fn derive_requested_execpolicy_amendment_returns_none_for_exact_banned_prefix_rule() {
         assert_eq!(
             None,
-            derive_requested_execpolicy_amendment_from_prefix_rule(
+            derive_requested_execpolicy_amendment_for_test(
                 Some(&vec!["python".to_string(), "-c".to_string()]),
                 &[],
             )
@@ -1599,7 +1931,7 @@ prefix_rule(
         ] {
             assert_eq!(
                 None,
-                derive_requested_execpolicy_amendment_from_prefix_rule(Some(&prefix_rule), &[])
+                derive_requested_execpolicy_amendment_for_test(Some(&prefix_rule), &[])
             );
         }
     }
@@ -1625,7 +1957,7 @@ prefix_rule(
         ] {
             assert_eq!(
                 None,
-                derive_requested_execpolicy_amendment_from_prefix_rule(Some(&prefix_rule), &[])
+                derive_requested_execpolicy_amendment_for_test(Some(&prefix_rule), &[])
             );
         }
     }
@@ -1640,7 +1972,7 @@ prefix_rule(
 
         assert_eq!(
             Some(ExecPolicyAmendment::new(prefix_rule.clone())),
-            derive_requested_execpolicy_amendment_from_prefix_rule(Some(&prefix_rule), &[])
+            derive_requested_execpolicy_amendment_for_test(Some(&prefix_rule), &[])
         );
     }
 
@@ -1655,7 +1987,7 @@ prefix_rule(
         }];
         assert_eq!(
             None,
-            derive_requested_execpolicy_amendment_from_prefix_rule(
+            derive_requested_execpolicy_amendment_for_test(
                 Some(&prefix_rule),
                 &matched_rules_prompt
             ),
@@ -1668,7 +2000,7 @@ prefix_rule(
         }];
         assert_eq!(
             None,
-            derive_requested_execpolicy_amendment_from_prefix_rule(
+            derive_requested_execpolicy_amendment_for_test(
                 Some(&prefix_rule),
                 &matched_rules_allow
             ),
@@ -1681,9 +2013,9 @@ prefix_rule(
         }];
         assert_eq!(
             None,
-            derive_requested_execpolicy_amendment_from_prefix_rule(
+            derive_requested_execpolicy_amendment_for_test(
                 Some(&prefix_rule),
-                &matched_rules_forbidden
+                &matched_rules_forbidden,
             ),
             "should return none when prompt policy matches"
         );
