@@ -37,6 +37,10 @@ const ALIASES: &[Alias] = &[
         legacy_key: "collab",
         feature: Feature::Collab,
     },
+    Alias {
+        legacy_key: "memory_tool",
+        feature: Feature::MemoryTool,
+    },
 ];
 
 pub(crate) fn legacy_feature_keys() -> impl Iterator<Item = &'static str> {
