@@ -1,12 +1,12 @@
 use serde::Deserialize;
 use serde::Serialize;
 
-use codex_protocol::models::ContentItem;
 use codex_protocol::models::ResponseItem;
 
-pub const USER_INSTRUCTIONS_OPEN_TAG_LEGACY: &str = "<user_instructions>";
+use crate::contextual_user_message::AGENTS_MD_FRAGMENT;
+use crate::contextual_user_message::SKILL_FRAGMENT;
+
 pub const USER_INSTRUCTIONS_PREFIX: &str = "# AGENTS.md instructions for ";
-pub const SKILL_INSTRUCTIONS_PREFIX: &str = "<skill";
 
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
 #[serde(rename = "user_instructions", rename_all = "snake_case")]
@@ -16,31 +16,20 @@ pub(crate) struct UserInstructions {
 }
 
 impl UserInstructions {
-    pub fn is_user_instructions(message: &[ContentItem]) -> bool {
-        if let [ContentItem::InputText { text }] = message {
-            text.starts_with(USER_INSTRUCTIONS_PREFIX)
-                || text.starts_with(USER_INSTRUCTIONS_OPEN_TAG_LEGACY)
-        } else {
-            false
-        }
+    pub(crate) fn serialize_to_text(&self) -> String {
+        format!(
+            "{prefix}{directory}\n\n<INSTRUCTIONS>\n{contents}\n{suffix}",
+            prefix = AGENTS_MD_FRAGMENT.start_marker(),
+            directory = self.directory,
+            contents = self.text,
+            suffix = AGENTS_MD_FRAGMENT.end_marker(),
+        )
     }
 }
 
 impl From<UserInstructions> for ResponseItem {
     fn from(ui: UserInstructions) -> Self {
-        ResponseItem::Message {
-            id: None,
-            role: "user".to_string(),
-            content: vec![ContentItem::InputText {
-                text: format!(
-                    "{USER_INSTRUCTIONS_PREFIX}{directory}\n\n<INSTRUCTIONS>\n{contents}\n</INSTRUCTIONS>",
-                    directory = ui.directory,
-                    contents = ui.text
-                ),
-            }],
-            end_turn: None,
-            phase: None,
-        }
+        AGENTS_MD_FRAGMENT.into_message(ui.serialize_to_text())
     }
 }
 
@@ -52,36 +41,21 @@ pub(crate) struct SkillInstructions {
     pub contents: String,
 }
 
-impl SkillInstructions {
-    pub fn is_skill_instructions(message: &[ContentItem]) -> bool {
-        if let [ContentItem::InputText { text }] = message {
-            text.starts_with(SKILL_INSTRUCTIONS_PREFIX)
-        } else {
-            false
-        }
-    }
-}
+impl SkillInstructions {}
 
 impl From<SkillInstructions> for ResponseItem {
     fn from(si: SkillInstructions) -> Self {
-        ResponseItem::Message {
-            id: None,
-            role: "user".to_string(),
-            content: vec![ContentItem::InputText {
-                text: format!(
-                    "<skill>\n<name>{}</name>\n<path>{}</path>\n{}\n</skill>",
-                    si.name, si.path, si.contents
-                ),
-            }],
-            end_turn: None,
-            phase: None,
-        }
+        SKILL_FRAGMENT.into_message(SKILL_FRAGMENT.wrap(format!(
+            "<name>{}</name>\n<path>{}</path>\n{}",
+            si.name, si.path, si.contents
+        )))
     }
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
+    use codex_protocol::models::ContentItem;
     use pretty_assertions::assert_eq;
 
     #[test]
@@ -110,21 +84,10 @@ mod tests {
 
     #[test]
     fn test_is_user_instructions() {
-        assert!(UserInstructions::is_user_instructions(
-            &[ContentItem::InputText {
-                text: "# AGENTS.md instructions for test_directory\n\n<INSTRUCTIONS>\ntest_text\n</INSTRUCTIONS>".to_string(),
-            }]
+        assert!(AGENTS_MD_FRAGMENT.matches_text(
+            "# AGENTS.md instructions for test_directory\n\n<INSTRUCTIONS>\ntest_text\n</INSTRUCTIONS>"
         ));
-        assert!(UserInstructions::is_user_instructions(&[
-            ContentItem::InputText {
-                text: "<user_instructions>test_text</user_instructions>".to_string(),
-            }
-        ]));
-        assert!(!UserInstructions::is_user_instructions(&[
-            ContentItem::InputText {
-                text: "test_text".to_string(),
-            }
-        ]));
+        assert!(!AGENTS_MD_FRAGMENT.matches_text("test_text"));
     }
 
     #[test]
@@ -154,16 +117,9 @@ mod tests {
 
     #[test]
     fn test_is_skill_instructions() {
-        assert!(SkillInstructions::is_skill_instructions(&[
-            ContentItem::InputText {
-                text: "<skill>\n<name>demo-skill</name>\n<path>skills/demo/SKILL.md</path>\nbody\n</skill>"
-                    .to_string(),
-            }
-        ]));
-        assert!(!SkillInstructions::is_skill_instructions(&[
-            ContentItem::InputText {
-                text: "regular text".to_string(),
-            }
-        ]));
+        assert!(SKILL_FRAGMENT.matches_text(
+            "<skill>\n<name>demo-skill</name>\n<path>skills/demo/SKILL.md</path>\nbody\n</skill>"
+        ));
+        assert!(!SKILL_FRAGMENT.matches_text("regular text"));
     }
 }
