@@ -13,6 +13,7 @@ pub mod auth;
 mod client;
 mod client_common;
 pub mod codex;
+mod realtime_conversation;
 pub use codex::SteerInputError;
 mod codex_thread;
 mod compact_remote;
@@ -26,6 +27,7 @@ pub mod config;
 pub mod config_loader;
 pub mod connectors;
 mod context_manager;
+mod contextual_user_message;
 pub mod custom_prompts;
 pub mod env;
 mod environment_context;
@@ -33,6 +35,7 @@ pub mod error;
 pub mod exec;
 pub mod exec_env;
 mod exec_policy;
+pub mod external_agent_config;
 pub mod features;
 mod file_watcher;
 mod flags;
@@ -54,13 +57,11 @@ mod message_history;
 mod model_provider_info;
 pub mod path_utils;
 pub mod personality_migration;
-mod proposed_plan_parser;
 mod sandbox_tags;
 pub mod sandboxing;
 mod session_prefix;
 mod shell_detect;
 mod stream_events_utils;
-mod tagged_block_parser;
 pub mod test_support;
 mod text_encoding;
 pub mod token_data;
@@ -83,7 +84,6 @@ mod seatbelt_permissions;
 mod thread_manager;
 pub mod web_search;
 pub mod windows_sandbox_read_grants;
-pub use codex_protocol::protocol::InitialHistory;
 pub use thread_manager::NewThread;
 pub use thread_manager::ThreadManager;
 #[deprecated(note = "use ThreadManager")]
@@ -136,14 +136,24 @@ mod state;
 mod tasks;
 mod user_shell_command;
 pub mod util;
-pub use codex_shell_command::bash;
-pub use codex_shell_command::is_dangerous_command;
-pub use codex_shell_command::is_safe_command;
-pub use codex_shell_command::parse_command;
-pub use codex_shell_command::powershell;
+pub(crate) use codex_protocol::protocol;
+pub(crate) use codex_shell_command::bash;
+pub(crate) use codex_shell_command::is_dangerous_command;
+pub(crate) use codex_shell_command::is_safe_command;
+pub(crate) use codex_shell_command::parse_command;
+pub(crate) use codex_shell_command::powershell;
 
-pub use apply_patch::CODEX_APPLY_PATCH_ARG1;
+pub use client::ModelClient;
+pub use client::ModelClientSession;
+pub use client::ResponsesWebsocketVersion;
 pub use client::X_CODEX_TURN_METADATA_HEADER;
+pub use client::ws_version_from_features;
+pub use client_common::Prompt;
+pub use client_common::REVIEW_PROMPT;
+pub use client_common::ResponseEvent;
+pub use client_common::ResponseStream;
+pub use compact::content_items_to_text;
+pub use event_mapping::parse_turn_item;
 pub use exec_policy::ExecPolicyError;
 pub use exec_policy::check_execpolicy_for_warnings;
 pub use exec_policy::format_exec_policy_error_with_source;
@@ -152,26 +162,6 @@ pub use file_watcher::FileWatcherEvent;
 pub use safety::get_platform_sandbox;
 pub use tools::spec::parse_tool_input_schema;
 pub use turn_metadata::build_turn_metadata_header;
-// Re-export the protocol types from the standalone `codex-protocol` crate so existing
-// `codex_core::protocol::...` references continue to work across the workspace.
-pub use codex_protocol::protocol;
-// Re-export protocol config enums to ensure call sites can use the same types
-// as those in the protocol crate when constructing protocol messages.
-pub use codex_protocol::config_types as protocol_config_types;
-
-pub use client::ModelClient;
-pub use client::ModelClientSession;
-pub use client_common::Prompt;
-pub use client_common::REVIEW_PROMPT;
-pub use client_common::ResponseEvent;
-pub use client_common::ResponseStream;
-pub use codex_protocol::models::ContentItem;
-pub use codex_protocol::models::LocalShellAction;
-pub use codex_protocol::models::LocalShellExecAction;
-pub use codex_protocol::models::LocalShellStatus;
-pub use codex_protocol::models::ResponseItem;
-pub use compact::content_items_to_text;
-pub use event_mapping::parse_turn_item;
 pub mod compact;
 pub mod memory_trace;
 pub mod otel_init;
