@@ -3,8 +3,13 @@ use std::time::Instant;
 
 use tracing::error;
 
+use crate::analytics_client::AppInvocation;
+use crate::analytics_client::InvocationType;
+use crate::analytics_client::build_track_events_context;
 use crate::codex::Session;
 use crate::codex::TurnContext;
+use crate::config::types::AppToolApproval;
+use crate::connectors;
 use crate::mcp::CODEX_APPS_MCP_SERVER_NAME;
 use crate::protocol::EventMsg;
 use crate::protocol::McpInvocation;
@@ -62,9 +67,51 @@ pub(crate) async fn handle_mcp_tool_call(
         arguments: arguments_value.clone(),
     };
 
-    if let Some(decision) =
-        maybe_request_mcp_tool_approval(sess.as_ref(), turn_context, &call_id, &server, &tool_name)
-            .await
+    let metadata = lookup_mcp_tool_metadata(sess.as_ref(), &server, &tool_name).await;
+    let app_tool_policy = if server == CODEX_APPS_MCP_SERVER_NAME {
+        connectors::app_tool_policy(
+            &turn_context.config,
+            metadata
+                .as_ref()
+                .and_then(|metadata| metadata.connector_id.as_deref()),
+            &tool_name,
+            metadata
+                .as_ref()
+                .and_then(|metadata| metadata.tool_title.as_deref()),
+            metadata
+                .as_ref()
+                .and_then(|metadata| metadata.annotations.as_ref()),
+        )
+    } else {
+        connectors::AppToolPolicy::default()
+    };
+
+    if server == CODEX_APPS_MCP_SERVER_NAME && !app_tool_policy.enabled {
+        let result = notify_mcp_tool_call_skip(
+            sess.as_ref(),
+            turn_context,
+            &call_id,
+            invocation,
+            "MCP tool call blocked by app configuration".to_string(),
+        )
+        .await;
+        let status = if result.is_ok() { "ok" } else { "error" };
+        turn_context
+            .otel_manager
+            .counter("codex.mcp.call", 1, &[("status", status)]);
+        return ResponseInputItem::McpToolCallOutput { call_id, result };
+    }
+
+    if let Some(decision) = maybe_request_mcp_tool_approval(
+        sess.as_ref(),
+        turn_context,
+        &call_id,
+        &server,
+        &tool_name,
+        metadata.as_ref(),
+        app_tool_policy.approval,
+    )
+    .await
     {
         let result = match decision {
             McpToolApprovalDecision::Accept | McpToolApprovalDecision::AcceptAndRemember => {
@@ -102,6 +149,7 @@ pub(crate) async fn handle_mcp_tool_call(
                     tool_call_end_event.clone(),
                 )
                 .await;
+                maybe_track_codex_app_used(sess.as_ref(), turn_context, &server, &tool_name).await;
                 result
             }
             McpToolApprovalDecision::Decline => {
@@ -166,6 +214,7 @@ pub(crate) async fn handle_mcp_tool_call(
     });
 
     notify_mcp_tool_call_event(sess.as_ref(), turn_context, tool_call_end_event.clone()).await;
+    maybe_track_codex_app_used(sess.as_ref(), turn_context, &server, &tool_name).await;
 
     let status = if result.is_ok() { "ok" } else { "error" };
     turn_context
@@ -210,6 +259,50 @@ async fn notify_mcp_tool_call_event(sess: &Session, turn_context: &TurnContext,
     sess.send_event(turn_context, event).await;
 }
 
+struct McpAppUsageMetadata {
+    connector_id: Option<String>,
+    app_name: Option<String>,
+}
+
+async fn maybe_track_codex_app_used(
+    sess: &Session,
+    turn_context: &TurnContext,
+    server: &str,
+    tool_name: &str,
+) {
+    if server != CODEX_APPS_MCP_SERVER_NAME {
+        return;
+    }
+    let metadata = lookup_mcp_app_usage_metadata(sess, server, tool_name).await;
+    let (connector_id, app_name) = metadata
+        .map(|metadata| (metadata.connector_id, metadata.app_name))
+        .unwrap_or((None, None));
+    let invocation_type = if let Some(connector_id) = connector_id.as_deref() {
+        let mentioned_connector_ids = sess.get_connector_selection().await;
+        if mentioned_connector_ids.contains(connector_id) {
+            InvocationType::Explicit
+        } else {
+            InvocationType::Implicit
+        }
+    } else {
+        InvocationType::Implicit
+    };
+
+    let tracking = build_track_events_context(
+        turn_context.model_info.slug.clone(),
+        sess.conversation_id.to_string(),
+        turn_context.sub_id.clone(),
+    );
+    sess.services.analytics_events_client.track_app_used(
+        tracking,
+        AppInvocation {
+            connector_id,
+            app_name,
+            invocation_type: Some(invocation_type),
+        },
+    );
+}
+
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 enum McpToolApprovalDecision {
     Accept,
@@ -219,7 +312,7 @@ enum McpToolApprovalDecision {
 }
 
 struct McpToolApprovalMetadata {
-    annotations: ToolAnnotations,
+    annotations: Option<ToolAnnotations>,
     connector_id: Option<String>,
     connector_name: Option<String>,
     tool_title: Option<String>,
@@ -234,7 +327,7 @@ const MCP_TOOL_APPROVAL_CANCEL: &str = "Cancel";
 #[derive(Debug, Serialize)]
 struct McpToolApprovalKey {
     server: String,
-    connector_id: String,
+    connector_id: Option<String>,
     tool_name: String,
 }
 
@@ -244,26 +337,36 @@ async fn maybe_request_mcp_tool_approval(
     call_id: &str,
     server: &str,
     tool_name: &str,
+    metadata: Option<&McpToolApprovalMetadata>,
+    approval_mode: AppToolApproval,
 ) -> Option<McpToolApprovalDecision> {
-    if is_full_access_mode(turn_context) {
+    if approval_mode == AppToolApproval::Approve {
         return None;
     }
-    if server != CODEX_APPS_MCP_SERVER_NAME {
-        return None;
+    let annotations = metadata.and_then(|metadata| metadata.annotations.as_ref());
+    if approval_mode == AppToolApproval::Auto {
+        if is_full_access_mode(turn_context) {
+            return None;
+        }
+        if !annotations.is_some_and(requires_mcp_tool_approval) {
+            return None;
+        }
     }
 
-    let metadata = lookup_mcp_tool_metadata(sess, server, tool_name).await?;
-    if !requires_mcp_tool_approval(&metadata.annotations) {
-        return None;
-    }
-    let approval_key = metadata
-        .connector_id
-        .as_deref()
-        .map(|connector_id| McpToolApprovalKey {
-            server: server.to_string(),
-            connector_id: connector_id.to_string(),
-            tool_name: tool_name.to_string(),
-        });
+    let approval_key = if approval_mode == AppToolApproval::Auto {
+        let connector_id = metadata.and_then(|metadata| metadata.connector_id.clone());
+        if server == CODEX_APPS_MCP_SERVER_NAME && connector_id.is_none() {
+            None
+        } else {
+            Some(McpToolApprovalKey {
+                server: server.to_string(),
+                connector_id,
+                tool_name: tool_name.to_string(),
+            })
+        }
+    } else {
+        None
+    };
     if let Some(key) = approval_key.as_ref()
         && mcp_tool_approval_is_remembered(sess, key).await
     {
@@ -273,10 +376,11 @@ async fn maybe_request_mcp_tool_approval(
     let question_id = format!("{MCP_TOOL_APPROVAL_QUESTION_ID_PREFIX}_{call_id}");
     let question = build_mcp_tool_approval_question(
         question_id.clone(),
+        server,
         tool_name,
-        metadata.tool_title.as_deref(),
-        metadata.connector_name.as_deref(),
-        &metadata.annotations,
+        metadata.and_then(|metadata| metadata.tool_title.as_deref()),
+        metadata.and_then(|metadata| metadata.connector_name.as_deref()),
+        annotations,
         approval_key.is_some(),
     );
     let args = RequestUserInputArgs {
@@ -285,7 +389,10 @@ async fn maybe_request_mcp_tool_approval(
     let response = sess
         .request_user_input(turn_context, call_id.to_string(), args)
         .await;
-    let decision = parse_mcp_tool_approval_response(response, &question_id);
+    let decision = normalize_approval_decision_for_mode(
+        parse_mcp_tool_approval_response(response, &question_id),
+        approval_mode,
+    );
     if matches!(decision, McpToolApprovalDecision::AcceptAndRemember)
         && let Some(key) = approval_key
     {
@@ -295,9 +402,9 @@ async fn maybe_request_mcp_tool_approval(
 }
 
 fn is_full_access_mode(turn_context: &TurnContext) -> bool {
-    matches!(turn_context.approval_policy, AskForApproval::Never)
+    matches!(turn_context.approval_policy.value(), AskForApproval::Never)
         && matches!(
-            turn_context.sandbox_policy,
+            turn_context.sandbox_policy.get(),
             SandboxPolicy::DangerFullAccess | SandboxPolicy::ExternalSandbox { .. }
         )
 }
@@ -317,15 +424,37 @@ async fn lookup_mcp_tool_metadata(
 
     tools.into_values().find_map(|tool_info| {
         if tool_info.server_name == server && tool_info.tool_name == tool_name {
-            tool_info
-                .tool
-                .annotations
-                .map(|annotations| McpToolApprovalMetadata {
-                    annotations,
-                    connector_id: tool_info.connector_id,
-                    connector_name: tool_info.connector_name,
-                    tool_title: tool_info.tool.title,
-                })
+            Some(McpToolApprovalMetadata {
+                annotations: tool_info.tool.annotations,
+                connector_id: tool_info.connector_id,
+                connector_name: tool_info.connector_name,
+                tool_title: tool_info.tool.title,
+            })
+        } else {
+            None
+        }
+    })
+}
+
+async fn lookup_mcp_app_usage_metadata(
+    sess: &Session,
+    server: &str,
+    tool_name: &str,
+) -> Option<McpAppUsageMetadata> {
+    let tools = sess
+        .services
+        .mcp_connection_manager
+        .read()
+        .await
+        .list_all_tools()
+        .await;
+
+    tools.into_values().find_map(|tool_info| {
+        if tool_info.server_name == server && tool_info.tool_name == tool_name {
+            Some(McpAppUsageMetadata {
+                connector_id: tool_info.connector_id,
+                app_name: tool_info.connector_name,
+            })
         } else {
             None
         }
@@ -334,14 +463,16 @@ async fn lookup_mcp_tool_metadata(
 
 fn build_mcp_tool_approval_question(
     question_id: String,
+    server: &str,
     tool_name: &str,
     tool_title: Option<&str>,
     connector_name: Option<&str>,
-    annotations: &ToolAnnotations,
+    annotations: Option<&ToolAnnotations>,
     allow_remember_option: bool,
 ) -> RequestUserInputQuestion {
-    let destructive = annotations.destructive_hint == Some(true);
-    let open_world = annotations.open_world_hint == Some(true);
+    let destructive =
+        annotations.and_then(|annotations| annotations.destructive_hint) == Some(true);
+    let open_world = annotations.and_then(|annotations| annotations.open_world_hint) == Some(true);
     let reason = match (destructive, open_world) {
         (true, true) => "may modify data and access external systems",
         (true, false) => "may modify or delete data",
@@ -352,7 +483,13 @@ fn build_mcp_tool_approval_question(
     let tool_label = tool_title.unwrap_or(tool_name);
     let app_label = connector_name
         .map(|name| format!("The {name} app"))
-        .unwrap_or_else(|| "This app".to_string());
+        .unwrap_or_else(|| {
+            if server == CODEX_APPS_MCP_SERVER_NAME {
+                "This app".to_string()
+            } else {
+                format!("The {server} MCP server")
+            }
+        });
     let question = format!(
         "{app_label} wants to run the tool \"{tool_label}\", which {reason}. Allow this action?"
     );
@@ -422,6 +559,19 @@ fn parse_mcp_tool_approval_response(
     }
 }
 
+fn normalize_approval_decision_for_mode(
+    decision: McpToolApprovalDecision,
+    approval_mode: AppToolApproval,
+) -> McpToolApprovalDecision {
+    if approval_mode == AppToolApproval::Prompt
+        && decision == McpToolApprovalDecision::AcceptAndRemember
+    {
+        McpToolApprovalDecision::Accept
+    } else {
+        decision
+    }
+}
+
 async fn mcp_tool_approval_is_remembered(sess: &Session, key: &McpToolApprovalKey) -> bool {
     let store = sess.services.tool_approvals.lock().await;
     matches!(store.get(key), Some(ReviewDecision::ApprovedForSession))
@@ -433,8 +583,11 @@ async fn remember_mcp_tool_approval(sess: &Session, key: McpToolApprovalKey) {
 }
 
 fn requires_mcp_tool_approval(annotations: &ToolAnnotations) -> bool {
-    annotations.read_only_hint == Some(false)
-        && (annotations.destructive_hint == Some(true) || annotations.open_world_hint == Some(true))
+    if annotations.destructive_hint == Some(true) {
+        return true;
+    }
+
+    annotations.read_only_hint == Some(false) && annotations.open_world_hint == Some(true)
 }
 
 async fn notify_mcp_tool_call_skip(
@@ -492,9 +645,66 @@ mod tests {
     }
 
     #[test]
-    fn approval_not_required_when_read_only_true() {
+    fn approval_required_when_destructive_even_if_read_only_true() {
         let annotations = annotations(Some(true), Some(true), Some(true));
-        assert_eq!(requires_mcp_tool_approval(&annotations), false);
+        assert_eq!(requires_mcp_tool_approval(&annotations), true);
+    }
+
+    #[test]
+    fn prompt_mode_does_not_allow_session_remember() {
+        assert_eq!(
+            normalize_approval_decision_for_mode(
+                McpToolApprovalDecision::AcceptAndRemember,
+                AppToolApproval::Prompt,
+            ),
+            McpToolApprovalDecision::Accept
+        );
+    }
+
+    #[test]
+    fn custom_mcp_tool_question_mentions_server_name() {
+        let question = build_mcp_tool_approval_question(
+            "q".to_string(),
+            "custom_server",
+            "run_action",
+            Some("Run Action"),
+            None,
+            Some(&annotations(Some(false), Some(true), None)),
+            true,
+        );
+
+        assert_eq!(question.header, "Approve app tool call?");
+        assert_eq!(
+            question.question,
+            "The custom_server MCP server wants to run the tool \"Run Action\", which may modify or delete data. Allow this action?"
+        );
+        assert!(
+            question
+                .options
+                .expect("options")
+                .into_iter()
+                .map(|option| option.label)
+                .any(|label| label == MCP_TOOL_APPROVAL_ACCEPT_AND_REMEMBER)
+        );
+    }
+
+    #[test]
+    fn codex_apps_tool_question_keeps_legacy_app_label() {
+        let question = build_mcp_tool_approval_question(
+            "q".to_string(),
+            CODEX_APPS_MCP_SERVER_NAME,
+            "run_action",
+            Some("Run Action"),
+            None,
+            Some(&annotations(Some(false), Some(true), None)),
+            true,
+        );
+
+        assert!(
+            question
+                .question
+                .starts_with("This app wants to run the tool \"Run Action\"")
+        );
     }
 
     #[test]
