@@ -3,6 +3,7 @@ use std::collections::HashSet;
 use std::fmt::Write as _;
 use std::path::Path;
 use tracing::warn;
+use uuid::Uuid;
 
 use crate::memories::ensure_layout;
 use crate::memories::raw_memories_file;
@@ -81,6 +82,11 @@ async fn rebuild_raw_memories_file(
         )
         .map_err(raw_memories_format_error)?;
         writeln!(body, "cwd: {}", memory.cwd.display()).map_err(raw_memories_format_error)?;
+        writeln!(body, "rollout_path: {}", memory.rollout_path.display())
+            .map_err(raw_memories_format_error)?;
+        let rollout_summary_file = format!("{}.md", rollout_summary_file_stem(memory));
+        writeln!(body, "rollout_summary_file: {rollout_summary_file}")
+            .map_err(raw_memories_format_error)?;
         writeln!(body).map_err(raw_memories_format_error)?;
         body.push_str(memory.raw_memory.trim());
         body.push_str("\n\n");
@@ -134,6 +140,8 @@ async fn write_rollout_summary_for_thread(
         memory.source_updated_at.to_rfc3339()
     )
     .map_err(rollout_summary_format_error)?;
+    writeln!(body, "rollout_path: {}", memory.rollout_path.display())
+        .map_err(rollout_summary_format_error)?;
     writeln!(body, "cwd: {}", memory.cwd.display()).map_err(rollout_summary_format_error)?;
     writeln!(body).map_err(rollout_summary_format_error)?;
     body.push_str(&memory.rollout_summary);
@@ -157,12 +165,67 @@ fn rollout_summary_format_error(err: std::fmt::Error) -> std::io::Error {
     std::io::Error::other(format!("format rollout summary: {err}"))
 }
 
-fn rollout_summary_file_stem(memory: &Stage1Output) -> String {
-    const ROLLOUT_SLUG_MAX_LEN: usize = 20;
+pub(crate) fn rollout_summary_file_stem(memory: &Stage1Output) -> String {
+    rollout_summary_file_stem_from_parts(
+        memory.thread_id,
+        memory.source_updated_at,
+        memory.rollout_slug.as_deref(),
+    )
+}
 
-    let thread_id = memory.thread_id.to_string();
-    let Some(raw_slug) = memory.rollout_slug.as_deref() else {
-        return thread_id;
+pub(super) fn rollout_summary_file_stem_from_parts(
+    thread_id: codex_protocol::ThreadId,
+    source_updated_at: chrono::DateTime<chrono::Utc>,
+    rollout_slug: Option<&str>,
+) -> String {
+    const ROLLOUT_SLUG_MAX_LEN: usize = 60;
+    const SHORT_HASH_ALPHABET: &[u8; 62] =
+        b"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+    const SHORT_HASH_SPACE: u32 = 14_776_336;
+
+    let thread_id = thread_id.to_string();
+    let (timestamp_fragment, short_hash_seed) = match Uuid::parse_str(&thread_id) {
+        Ok(thread_uuid) => {
+            let timestamp = thread_uuid
+                .get_timestamp()
+                .and_then(|uuid_timestamp| {
+                    let (seconds, nanos) = uuid_timestamp.to_unix();
+                    i64::try_from(seconds).ok().and_then(|secs| {
+                        chrono::DateTime::<chrono::Utc>::from_timestamp(secs, nanos)
+                    })
+                })
+                .unwrap_or(source_updated_at);
+            let short_hash_seed = (thread_uuid.as_u128() & 0xFFFF_FFFF) as u32;
+            (
+                timestamp.format("%Y-%m-%dT%H-%M-%S").to_string(),
+                short_hash_seed,
+            )
+        }
+        Err(_) => {
+            let mut short_hash_seed = 0u32;
+            for byte in thread_id.bytes() {
+                short_hash_seed = short_hash_seed
+                    .wrapping_mul(31)
+                    .wrapping_add(u32::from(byte));
+            }
+            (
+                source_updated_at.format("%Y-%m-%dT%H-%M-%S").to_string(),
+                short_hash_seed,
+            )
+        }
+    };
+    let mut short_hash_value = short_hash_seed % SHORT_HASH_SPACE;
+    let mut short_hash_chars = ['0'; 4];
+    for idx in (0..short_hash_chars.len()).rev() {
+        let alphabet_idx = (short_hash_value % SHORT_HASH_ALPHABET.len() as u32) as usize;
+        short_hash_chars[idx] = SHORT_HASH_ALPHABET[alphabet_idx] as char;
+        short_hash_value /= SHORT_HASH_ALPHABET.len() as u32;
+    }
+    let short_hash: String = short_hash_chars.iter().collect();
+    let file_prefix = format!("{timestamp_fragment}-{short_hash}");
+
+    let Some(raw_slug) = rollout_slug else {
+        return file_prefix;
     };
 
     let mut slug = String::with_capacity(ROLLOUT_SLUG_MAX_LEN);
@@ -183,59 +246,81 @@ fn rollout_summary_file_stem(memory: &Stage1Output) -> String {
     }
 
     if slug.is_empty() {
-        thread_id
+        file_prefix
     } else {
-        format!("{thread_id}-{slug}")
+        format!("{file_prefix}-{slug}")
     }
 }
 
 #[cfg(test)]
 mod tests {
     use super::rollout_summary_file_stem;
+    use super::rollout_summary_file_stem_from_parts;
     use chrono::TimeZone;
     use chrono::Utc;
     use codex_protocol::ThreadId;
     use codex_state::Stage1Output;
     use pretty_assertions::assert_eq;
     use std::path::PathBuf;
+    const FIXED_PREFIX: &str = "2025-02-11T15-35-19-jqmb";
 
-    fn stage1_output_with_slug(rollout_slug: Option<&str>) -> Stage1Output {
+    fn stage1_output_with_slug(thread_id: ThreadId, rollout_slug: Option<&str>) -> Stage1Output {
         Stage1Output {
-            thread_id: ThreadId::new(),
+            thread_id,
             source_updated_at: Utc.timestamp_opt(123, 0).single().expect("timestamp"),
             raw_memory: "raw memory".to_string(),
             rollout_summary: "summary".to_string(),
             rollout_slug: rollout_slug.map(ToString::to_string),
+            rollout_path: PathBuf::from("/tmp/rollout.jsonl"),
             cwd: PathBuf::from("/tmp/workspace"),
             generated_at: Utc.timestamp_opt(124, 0).single().expect("timestamp"),
         }
     }
 
-    #[test]
-    fn rollout_summary_file_stem_uses_thread_id_when_slug_missing() {
-        let memory = stage1_output_with_slug(None);
-        let thread_id = memory.thread_id.to_string();
-
-        assert_eq!(rollout_summary_file_stem(&memory), thread_id);
+    fn fixed_thread_id() -> ThreadId {
+        ThreadId::try_from("0194f5a6-89ab-7cde-8123-456789abcdef").expect("valid thread id")
     }
 
     #[test]
-    fn rollout_summary_file_stem_sanitizes_and_truncates_slug() {
-        let memory =
-            stage1_output_with_slug(Some("Unsafe Slug/With Spaces & Symbols + EXTRA_LONG_12345"));
-        let thread_id = memory.thread_id.to_string();
+    fn rollout_summary_file_stem_uses_uuid_timestamp_and_hash_when_slug_missing() {
+        let thread_id = fixed_thread_id();
+        let memory = stage1_output_with_slug(thread_id, None);
 
+        assert_eq!(rollout_summary_file_stem(&memory), FIXED_PREFIX);
         assert_eq!(
-            rollout_summary_file_stem(&memory),
-            format!("{thread_id}-unsafe_slug_with_spa")
+            rollout_summary_file_stem_from_parts(
+                memory.thread_id,
+                memory.source_updated_at,
+                memory.rollout_slug.as_deref(),
+            ),
+            FIXED_PREFIX
         );
     }
 
     #[test]
-    fn rollout_summary_file_stem_uses_thread_id_when_slug_is_empty() {
-        let memory = stage1_output_with_slug(Some(""));
-        let thread_id = memory.thread_id.to_string();
+    fn rollout_summary_file_stem_sanitizes_and_truncates_slug() {
+        let thread_id = fixed_thread_id();
+        let memory = stage1_output_with_slug(
+            thread_id,
+            Some("Unsafe Slug/With Spaces & Symbols + EXTRA_LONG_12345_67890_ABCDE_fghij_klmno"),
+        );
 
-        assert_eq!(rollout_summary_file_stem(&memory), thread_id);
+        let stem = rollout_summary_file_stem(&memory);
+        let slug = stem
+            .strip_prefix(&format!("{FIXED_PREFIX}-"))
+            .expect("slug suffix should be present");
+        assert_eq!(slug.len(), 60);
+        assert_eq!(
+            slug,
+            "unsafe_slug_with_spaces___symbols___extra_long_12345_67890_a"
+        );
+    }
+
+    #[test]
+    fn rollout_summary_file_stem_uses_uuid_timestamp_and_hash_when_slug_is_empty() {
+        let thread_id = fixed_thread_id();
+        let memory = stage1_output_with_slug(thread_id, Some(""));
+
+        assert_eq!(rollout_summary_file_stem(&memory), FIXED_PREFIX);
     }
 }
