@@ -47,9 +47,11 @@ pub(crate) fn builder_from_session_meta(
         session_meta.meta.source.clone(),
     );
     builder.model_provider = session_meta.meta.model_provider.clone();
+    builder.agent_nickname = session_meta.meta.agent_nickname.clone();
+    builder.agent_role = session_meta.meta.agent_role.clone();
     builder.cwd = session_meta.meta.cwd.clone();
     builder.cli_version = Some(session_meta.meta.cli_version.clone());
-    builder.sandbox_policy = SandboxPolicy::ReadOnly;
+    builder.sandbox_policy = SandboxPolicy::new_read_only_policy();
     builder.approval_mode = AskForApproval::OnRequest;
     if let Some(git) = session_meta.git.as_ref() {
         builder.git_sha = git.commit_hash.clone();
@@ -512,6 +514,8 @@ mod tests {
             originator: "cli".to_string(),
             cli_version: "0.0.0".to_string(),
             source: SessionSource::default(),
+            agent_nickname: None,
+            agent_role: None,
             model_provider: Some("openai".to_string()),
             base_instructions: None,
             dynamic_tools: None,
@@ -660,6 +664,8 @@ mod tests {
             originator: "cli".to_string(),
             cli_version: "0.0.0".to_string(),
             source: SessionSource::default(),
+            agent_nickname: None,
+            agent_role: None,
             model_provider: Some("test-provider".to_string()),
             base_instructions: None,
             dynamic_tools: None,
