@@ -560,6 +560,8 @@ async fn test_list_conversations_latest_first() {
                 git_sha: None,
                 git_origin_url: None,
                 source: Some(SessionSource::VSCode),
+                agent_nickname: None,
+                agent_role: None,
                 model_provider: Some(TEST_PROVIDER.to_string()),
                 cli_version: Some("test_version".to_string()),
                 created_at: Some("2025-01-03T12-00-00".into()),
@@ -574,6 +576,8 @@ async fn test_list_conversations_latest_first() {
                 git_sha: None,
                 git_origin_url: None,
                 source: Some(SessionSource::VSCode),
+                agent_nickname: None,
+                agent_role: None,
                 model_provider: Some(TEST_PROVIDER.to_string()),
                 cli_version: Some("test_version".to_string()),
                 created_at: Some("2025-01-02T12-00-00".into()),
@@ -588,6 +592,8 @@ async fn test_list_conversations_latest_first() {
                 git_sha: None,
                 git_origin_url: None,
                 source: Some(SessionSource::VSCode),
+                agent_nickname: None,
+                agent_role: None,
                 model_provider: Some(TEST_PROVIDER.to_string()),
                 cli_version: Some("test_version".to_string()),
                 created_at: Some("2025-01-01T12-00-00".into()),
@@ -695,6 +701,8 @@ async fn test_pagination_cursor() {
                 git_sha: None,
                 git_origin_url: None,
                 source: Some(SessionSource::VSCode),
+                agent_nickname: None,
+                agent_role: None,
                 model_provider: Some(TEST_PROVIDER.to_string()),
                 cli_version: Some("test_version".to_string()),
                 created_at: Some("2025-03-05T09-00-00".into()),
@@ -709,6 +717,8 @@ async fn test_pagination_cursor() {
                 git_sha: None,
                 git_origin_url: None,
                 source: Some(SessionSource::VSCode),
+                agent_nickname: None,
+                agent_role: None,
                 model_provider: Some(TEST_PROVIDER.to_string()),
                 cli_version: Some("test_version".to_string()),
                 created_at: Some("2025-03-04T09-00-00".into()),
@@ -759,6 +769,8 @@ async fn test_pagination_cursor() {
                 git_sha: None,
                 git_origin_url: None,
                 source: Some(SessionSource::VSCode),
+                agent_nickname: None,
+                agent_role: None,
                 model_provider: Some(TEST_PROVIDER.to_string()),
                 cli_version: Some("test_version".to_string()),
                 created_at: Some("2025-03-03T09-00-00".into()),
@@ -773,6 +785,8 @@ async fn test_pagination_cursor() {
                 git_sha: None,
                 git_origin_url: None,
                 source: Some(SessionSource::VSCode),
+                agent_nickname: None,
+                agent_role: None,
                 model_provider: Some(TEST_PROVIDER.to_string()),
                 cli_version: Some("test_version".to_string()),
                 created_at: Some("2025-03-02T09-00-00".into()),
@@ -814,6 +828,8 @@ async fn test_pagination_cursor() {
             git_sha: None,
             git_origin_url: None,
             source: Some(SessionSource::VSCode),
+            agent_nickname: None,
+            agent_role: None,
             model_provider: Some(TEST_PROVIDER.to_string()),
             cli_version: Some("test_version".to_string()),
             created_at: Some("2025-03-01T09-00-00".into()),
@@ -894,6 +910,8 @@ async fn test_get_thread_contents() {
             git_sha: None,
             git_origin_url: None,
             source: Some(SessionSource::VSCode),
+            agent_nickname: None,
+            agent_role: None,
             model_provider: Some(TEST_PROVIDER.to_string()),
             cli_version: Some("test_version".to_string()),
             created_at: Some(ts.into()),
@@ -1086,6 +1104,8 @@ async fn test_updated_at_uses_file_mtime() -> Result<()> {
                 originator: "test_originator".into(),
                 cli_version: "test_version".into(),
                 source: SessionSource::VSCode,
+                agent_nickname: None,
+                agent_role: None,
                 model_provider: Some("test-provider".into()),
                 base_instructions: None,
                 dynamic_tools: None,
@@ -1203,6 +1223,8 @@ async fn test_stable_ordering_same_second_pagination() {
                 git_sha: None,
                 git_origin_url: None,
                 source: Some(SessionSource::VSCode),
+                agent_nickname: None,
+                agent_role: None,
                 model_provider: Some(TEST_PROVIDER.to_string()),
                 cli_version: Some("test_version".to_string()),
                 created_at: Some(ts.to_string()),
@@ -1217,6 +1239,8 @@ async fn test_stable_ordering_same_second_pagination() {
                 git_sha: None,
                 git_origin_url: None,
                 source: Some(SessionSource::VSCode),
+                agent_nickname: None,
+                agent_role: None,
                 model_provider: Some(TEST_PROVIDER.to_string()),
                 cli_version: Some("test_version".to_string()),
                 created_at: Some(ts.to_string()),
@@ -1258,6 +1282,8 @@ async fn test_stable_ordering_same_second_pagination() {
             git_sha: None,
             git_origin_url: None,
             source: Some(SessionSource::VSCode),
+            agent_nickname: None,
+            agent_role: None,
             model_provider: Some(TEST_PROVIDER.to_string()),
             cli_version: Some("test_version".to_string()),
             created_at: Some(ts.to_string()),
