@@ -12,10 +12,10 @@ use crate::skills::model::SkillLoadOutcome;
 use crate::skills::model::SkillMetadata;
 use crate::skills::model::SkillPolicy;
 use crate::skills::model::SkillToolDependency;
-use crate::skills::permissions::SkillManifestPermissions;
 use crate::skills::permissions::compile_permission_profile;
 use crate::skills::system::system_cache_root_dir;
 use codex_app_server_protocol::ConfigLayerSource;
+use codex_protocol::models::PermissionProfile;
 use codex_protocol::protocol::SkillScope;
 use dirs::home_dir;
 use dunce::canonicalize as canonicalize_path;
@@ -54,7 +54,16 @@ struct SkillMetadataFile {
     #[serde(default)]
     policy: Option<Policy>,
     #[serde(default)]
-    permissions: Option<SkillManifestPermissions>,
+    permissions: Option<PermissionProfile>,
+}
+
+#[derive(Default)]
+struct LoadedSkillMetadata {
+    interface: Option<SkillInterface>,
+    dependencies: Option<SkillDependencies>,
+    policy: Option<SkillPolicy>,
+    permission_profile: Option<PermissionProfile>,
+    permissions: Option<Permissions>,
 }
 
 #[derive(Debug, Default, Deserialize)]
@@ -137,7 +146,17 @@ impl fmt::Display for SkillParseError {
 impl Error for SkillParseError {}
 
 pub fn load_skills(config: &Config) -> SkillLoadOutcome {
-    load_skills_from_roots(skill_roots(config))
+    load_skills_with_home_dir(config, home_dir().as_deref())
+}
+
+fn load_skills_with_home_dir(config: &Config, home_dir: Option<&Path>) -> SkillLoadOutcome {
+    let mut roots = skill_roots_from_layer_stack_inner(&config.config_layer_stack, home_dir);
+    roots.extend(repo_agents_skill_roots(
+        &config.config_layer_stack,
+        &config.cwd,
+    ));
+    dedupe_skill_roots_by_path(&mut roots);
+    load_skills_from_roots(roots)
 }
 
 pub(crate) struct SkillRoot {
@@ -157,7 +176,7 @@ where
     let mut seen: HashSet<PathBuf> = HashSet::new();
     outcome
         .skills
-        .retain(|skill| seen.insert(skill.path.clone()));
+        .retain(|skill| seen.insert(skill.path_to_skills_md.clone()));
 
     fn scope_rank(scope: SkillScope) -> u8 {
         // Higher-priority scopes first (matches root scan order for dedupe).
@@ -173,7 +192,7 @@ where
         scope_rank(a.scope)
             .cmp(&scope_rank(b.scope))
             .then_with(|| a.name.cmp(&b.name))
-            .then_with(|| a.path.cmp(&b.path))
+            .then_with(|| a.path_to_skills_md.cmp(&b.path_to_skills_md))
     });
 
     outcome
@@ -240,6 +259,7 @@ fn skill_roots_from_layer_stack_inner(
     roots
 }
 
+#[cfg(test)]
 fn skill_roots(config: &Config) -> Vec<SkillRoot> {
     skill_roots_from_layer_stack_with_agents(&config.config_layer_stack, &config.cwd)
 }
@@ -495,7 +515,13 @@ fn parse_skill_file(path: &Path, scope: SkillScope) -> Result<SkillMetadata, Ski
         .as_deref()
         .map(sanitize_single_line)
         .filter(|value| !value.is_empty());
-    let (interface, dependencies, policy, permissions) = load_skill_metadata(path);
+    let LoadedSkillMetadata {
+        interface,
+        dependencies,
+        policy,
+        permission_profile,
+        permissions,
+    } = load_skill_metadata(path);
 
     validate_len(&name, MAX_NAME_LEN, "name")?;
     validate_len(&description, MAX_DESCRIPTION_LEN, "description")?;
@@ -516,29 +542,23 @@ fn parse_skill_file(path: &Path, scope: SkillScope) -> Result<SkillMetadata, Ski
         interface,
         dependencies,
         policy,
+        permission_profile,
         permissions,
-        path: resolved_path,
+        path_to_skills_md: resolved_path,
         scope,
     })
 }
 
-fn load_skill_metadata(
-    skill_path: &Path,
-) -> (
-    Option<SkillInterface>,
-    Option<SkillDependencies>,
-    Option<SkillPolicy>,
-    Option<Permissions>,
-) {
+fn load_skill_metadata(skill_path: &Path) -> LoadedSkillMetadata {
     // Fail open: optional metadata should not block loading SKILL.md.
     let Some(skill_dir) = skill_path.parent() else {
-        return (None, None, None, None);
+        return LoadedSkillMetadata::default();
     };
     let metadata_path = skill_dir
         .join(SKILLS_METADATA_DIR)
         .join(SKILLS_METADATA_FILENAME);
     if !metadata_path.exists() {
-        return (None, None, None, None);
+        return LoadedSkillMetadata::default();
     }
 
     let contents = match fs::read_to_string(&metadata_path) {
@@ -549,7 +569,7 @@ fn load_skill_metadata(
                 path = metadata_path.display(),
                 label = SKILLS_METADATA_FILENAME
             );
-            return (None, None, None, None);
+            return LoadedSkillMetadata::default();
         }
     };
 
@@ -561,7 +581,7 @@ fn load_skill_metadata(
                 path = metadata_path.display(),
                 label = SKILLS_METADATA_FILENAME
             );
-            return (None, None, None, None);
+            return LoadedSkillMetadata::default();
         }
     };
 
@@ -571,13 +591,15 @@ fn load_skill_metadata(
         policy,
         permissions,
     } = parsed;
+    let permission_profile = permissions.clone().filter(|profile| !profile.is_empty());
 
-    (
-        resolve_interface(interface, skill_dir),
-        resolve_dependencies(dependencies),
-        resolve_policy(policy),
-        compile_permission_profile(skill_dir, permissions),
-    )
+    LoadedSkillMetadata {
+        interface: resolve_interface(interface, skill_dir),
+        dependencies: resolve_dependencies(dependencies),
+        policy: resolve_policy(policy),
+        permission_profile,
+        permissions: compile_permission_profile(skill_dir, permissions),
+    }
 }
 
 fn resolve_interface(interface: Option<Interface>, skill_dir: &Path) -> Option<SkillInterface> {
@@ -806,7 +828,6 @@ fn extract_frontmatter(contents: &str) -> Option<String> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::config::CONFIG_TOML_FILE;
     use crate::config::ConfigBuilder;
     use crate::config::ConfigOverrides;
     use crate::config::ConfigToml;
@@ -817,7 +838,10 @@ mod tests {
     use crate::config_loader::ConfigLayerStack;
     use crate::config_loader::ConfigRequirements;
     use crate::config_loader::ConfigRequirementsToml;
+    use codex_config::CONFIG_TOML_FILE;
     use codex_protocol::config_types::TrustLevel;
+    use codex_protocol::models::FileSystemPermissions;
+    use codex_protocol::models::PermissionProfile;
     use codex_protocol::protocol::SkillScope;
     use codex_utils_absolute_path::AbsolutePathBuf;
     use pretty_assertions::assert_eq;
@@ -867,6 +891,11 @@ mod tests {
             .expect("defaults for test should always succeed")
     }
 
+    fn load_skills_for_test(config: &Config) -> SkillLoadOutcome {
+        // Keep unit tests hermetic by never scanning the real `$HOME/.agents/skills`.
+        super::load_skills_with_home_dir(config, None)
+    }
+
     fn mark_as_git_repo(dir: &Path) {
         // Config/project-root discovery only checks for the presence of `.git` (file or dir),
         // so we can avoid shelling out to `git init` in tests.
@@ -1032,8 +1061,9 @@ mod tests {
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::User,
             }]
         );
@@ -1129,7 +1159,7 @@ mod tests {
         );
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1180,8 +1210,9 @@ mod tests {
                     ],
                 }),
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::User,
             }]
         );
@@ -1208,7 +1239,7 @@ interface:
         );
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1236,8 +1267,9 @@ interface:
                 }),
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(skill_path.as_path()),
+                path_to_skills_md: normalized(skill_path.as_path()),
                 scope: SkillScope::User,
             }]
         );
@@ -1258,7 +1290,7 @@ policy:
         );
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1289,7 +1321,7 @@ policy: {}
         );
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1325,14 +1357,13 @@ permissions:
   file_system:
     read:
       - "./data"
-      - "./data"
     write:
       - "./output"
 "#,
         );
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1340,6 +1371,17 @@ permissions:
             outcome.errors
         );
         assert_eq!(outcome.skills.len(), 1);
+        assert_eq!(
+            outcome.skills[0].permission_profile,
+            Some(PermissionProfile {
+                network: Some(true),
+                file_system: Some(FileSystemPermissions {
+                    read: Some(vec![PathBuf::from("./data")]),
+                    write: Some(vec![PathBuf::from("./output")]),
+                }),
+                macos: None,
+            })
+        );
         #[cfg(target_os = "macos")]
         let macos_seatbelt_profile_extensions =
             Some(crate::seatbelt_permissions::MacOsSeatbeltProfileExtensions::default());
@@ -1372,6 +1414,7 @@ permissions:
                     }
                 ),
                 network: None,
+                allow_login_shell: true,
                 shell_environment_policy: ShellEnvironmentPolicy::default(),
                 windows_sandbox_mode: None,
                 macos_seatbelt_profile_extensions,
@@ -1393,7 +1436,7 @@ permissions: {}
         );
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1408,6 +1451,7 @@ permissions: {}
                 crate::protocol::SandboxPolicy::new_read_only_policy(),
             ),
             network: None,
+            allow_login_shell: true,
             shell_environment_policy: ShellEnvironmentPolicy::default(),
             windows_sandbox_mode: None,
             macos_seatbelt_profile_extensions: Some(
@@ -1421,10 +1465,12 @@ permissions: {}
                 crate::protocol::SandboxPolicy::new_read_only_policy(),
             ),
             network: None,
+            allow_login_shell: true,
             shell_environment_policy: ShellEnvironmentPolicy::default(),
             windows_sandbox_mode: None,
             macos_seatbelt_profile_extensions: None,
         });
+        assert_eq!(outcome.skills[0].permission_profile, None);
         assert_eq!(outcome.skills[0].permissions, expected);
     }
 
@@ -1449,7 +1495,7 @@ permissions:
         );
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1499,7 +1545,7 @@ permissions:
         );
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1515,6 +1561,7 @@ permissions:
                     crate::protocol::SandboxPolicy::new_read_only_policy(),
                 ),
                 network: None,
+                allow_login_shell: true,
                 shell_environment_policy: ShellEnvironmentPolicy::default(),
                 windows_sandbox_mode: None,
                 macos_seatbelt_profile_extensions: None,
@@ -1543,7 +1590,7 @@ permissions:
         );
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1566,8 +1613,9 @@ permissions:
                 }),
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::User,
             }]
         );
@@ -1591,7 +1639,7 @@ permissions:
         );
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1607,8 +1655,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::User,
             }]
         );
@@ -1638,7 +1687,7 @@ permissions:
         );
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1661,8 +1710,9 @@ permissions:
                 }),
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::User,
             }]
         );
@@ -1687,7 +1737,7 @@ permissions:
         );
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1703,8 +1753,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::User,
             }]
         );
@@ -1732,7 +1783,7 @@ permissions:
         symlink_dir(shared.path(), &codex_home.path().join("skills/shared"));
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1748,8 +1799,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&shared_skill_path),
+                path_to_skills_md: normalized(&shared_skill_path),
                 scope: SkillScope::User,
             }]
         );
@@ -1769,7 +1821,7 @@ permissions:
         symlink_file(&shared_skill_path, &skill_dir.join(SKILLS_FILENAME));
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1793,7 +1845,7 @@ permissions:
         let skill_path = write_skill_at(&cycle_dir, "demo", "cycle-skill", "still loads");
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1809,8 +1861,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::User,
             }]
         );
@@ -1846,8 +1899,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&shared_skill_path),
+                path_to_skills_md: normalized(&shared_skill_path),
                 scope: SkillScope::Admin,
             }]
         );
@@ -1871,7 +1925,7 @@ permissions:
         symlink_dir(shared.path(), &repo_skills_root.join("shared"));
 
         let cfg = make_config_for_cwd(&codex_home, repo_dir.path().to_path_buf()).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -1887,8 +1941,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&linked_skill_path),
+                path_to_skills_md: normalized(&linked_skill_path),
                 scope: SkillScope::Repo,
             }]
         );
@@ -1955,8 +2010,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&within_depth_path),
+                path_to_skills_md: normalized(&within_depth_path),
                 scope: SkillScope::User,
             }]
         );
@@ -1968,7 +2024,7 @@ permissions:
         let skill_path = write_skill(&codex_home, "demo", "demo-skill", "does things\ncarefully");
         let cfg = make_config(&codex_home).await;
 
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert!(
             outcome.errors.is_empty(),
             "unexpected errors: {:?}",
@@ -1983,8 +2039,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::User,
             }]
         );
@@ -2000,7 +2057,7 @@ permissions:
         fs::write(&skill_path, contents).unwrap();
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert!(
             outcome.errors.is_empty(),
             "unexpected errors: {:?}",
@@ -2015,8 +2072,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::User,
             }]
         );
@@ -2034,7 +2092,7 @@ permissions:
         fs::write(skill_dir.join(SKILLS_FILENAME), contents).unwrap();
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert_eq!(outcome.skills.len(), 0);
         assert_eq!(outcome.errors.len(), 1);
         assert!(
@@ -2063,7 +2121,7 @@ permissions:
         fs::write(invalid_dir.join(SKILLS_FILENAME), "---\nname: bad").unwrap();
 
         let cfg = make_config(&codex_home).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert_eq!(outcome.skills.len(), 0);
         assert_eq!(outcome.errors.len(), 1);
         assert!(
@@ -2081,7 +2139,7 @@ permissions:
         write_skill(&codex_home, "max-len", "max-len", &max_desc);
         let cfg = make_config(&codex_home).await;
 
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert!(
             outcome.errors.is_empty(),
             "unexpected errors: {:?}",
@@ -2091,7 +2149,7 @@ permissions:
 
         let too_long_desc = "\u{1F4A1}".repeat(MAX_DESCRIPTION_LEN + 1);
         write_skill(&codex_home, "too-long", "too-long", &too_long_desc);
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert_eq!(outcome.skills.len(), 1);
         assert_eq!(outcome.errors.len(), 1);
         assert!(
@@ -2113,7 +2171,7 @@ permissions:
         let skill_path = write_skill_at(&skills_root, "repo", "repo-skill", "from repo");
         let cfg = make_config_for_cwd(&codex_home, repo_dir.path().to_path_buf()).await;
 
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert!(
             outcome.errors.is_empty(),
             "unexpected errors: {:?}",
@@ -2128,8 +2186,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::Repo,
             }]
         );
@@ -2149,7 +2208,7 @@ permissions:
         );
         let cfg = make_config_for_cwd(&codex_home, repo_dir.path().to_path_buf()).await;
 
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert!(
             outcome.errors.is_empty(),
             "unexpected errors: {:?}",
@@ -2164,8 +2223,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::Repo,
             }]
         );
@@ -2202,7 +2262,7 @@ permissions:
 
         let cfg = make_config_for_cwd(&codex_home, nested_dir).await;
 
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert!(
             outcome.errors.is_empty(),
             "unexpected errors: {:?}",
@@ -2218,8 +2278,9 @@ permissions:
                     interface: None,
                     dependencies: None,
                     policy: None,
+                    permission_profile: None,
                     permissions: None,
-                    path: normalized(&nested_skill_path),
+                    path_to_skills_md: normalized(&nested_skill_path),
                     scope: SkillScope::Repo,
                 },
                 SkillMetadata {
@@ -2229,8 +2290,9 @@ permissions:
                     interface: None,
                     dependencies: None,
                     policy: None,
+                    permission_profile: None,
                     permissions: None,
-                    path: normalized(&root_skill_path),
+                    path_to_skills_md: normalized(&root_skill_path),
                     scope: SkillScope::Repo,
                 },
             ]
@@ -2254,7 +2316,7 @@ permissions:
 
         let cfg = make_config_for_cwd(&codex_home, work_dir.path().to_path_buf()).await;
 
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert!(
             outcome.errors.is_empty(),
             "unexpected errors: {:?}",
@@ -2269,8 +2331,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::Repo,
             }]
         );
@@ -2307,8 +2370,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::Repo,
             }]
         );
@@ -2333,7 +2397,7 @@ permissions:
 
         let cfg = make_config_for_cwd(&codex_home, repo_dir.path().to_path_buf()).await;
 
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert!(
             outcome.errors.is_empty(),
             "unexpected errors: {:?}",
@@ -2349,8 +2413,9 @@ permissions:
                     interface: None,
                     dependencies: None,
                     policy: None,
+                    permission_profile: None,
                     permissions: None,
-                    path: normalized(&repo_skill_path),
+                    path_to_skills_md: normalized(&repo_skill_path),
                     scope: SkillScope::Repo,
                 },
                 SkillMetadata {
@@ -2360,8 +2425,9 @@ permissions:
                     interface: None,
                     dependencies: None,
                     policy: None,
+                    permission_profile: None,
                     permissions: None,
-                    path: normalized(&user_skill_path),
+                    path_to_skills_md: normalized(&user_skill_path),
                     scope: SkillScope::User,
                 },
             ]
@@ -2398,7 +2464,7 @@ permissions:
         );
 
         let cfg = make_config_for_cwd(&codex_home, nested_dir).await;
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
 
         assert!(
             outcome.errors.is_empty(),
@@ -2425,8 +2491,9 @@ permissions:
                     interface: None,
                     dependencies: None,
                     policy: None,
+                    permission_profile: None,
                     permissions: None,
-                    path: first_path,
+                    path_to_skills_md: first_path,
                     scope: SkillScope::Repo,
                 },
                 SkillMetadata {
@@ -2436,8 +2503,9 @@ permissions:
                     interface: None,
                     dependencies: None,
                     policy: None,
+                    permission_profile: None,
                     permissions: None,
-                    path: second_path,
+                    path_to_skills_md: second_path,
                     scope: SkillScope::Repo,
                 },
             ]
@@ -2464,7 +2532,7 @@ permissions:
 
         let cfg = make_config_for_cwd(&codex_home, repo_dir).await;
 
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert!(
             outcome.errors.is_empty(),
             "unexpected errors: {:?}",
@@ -2493,7 +2561,7 @@ permissions:
 
         let cfg = make_config_for_cwd(&codex_home, file_path).await;
 
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert!(
             outcome.errors.is_empty(),
             "unexpected errors: {:?}",
@@ -2508,8 +2576,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::Repo,
             }]
         );
@@ -2534,7 +2603,7 @@ permissions:
 
         let cfg = make_config_for_cwd(&codex_home, nested_dir).await;
 
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert!(
             outcome.errors.is_empty(),
             "unexpected errors: {:?}",
@@ -2552,7 +2621,7 @@ permissions:
 
         let cfg = make_config_for_cwd(&codex_home, work_dir.path().to_path_buf()).await;
 
-        let outcome = load_skills(&cfg);
+        let outcome = load_skills_for_test(&cfg);
         assert!(
             outcome.errors.is_empty(),
             "unexpected errors: {:?}",
@@ -2567,8 +2636,9 @@ permissions:
                 interface: None,
                 dependencies: None,
                 policy: None,
+                permission_profile: None,
                 permissions: None,
-                path: normalized(&skill_path),
+                path_to_skills_md: normalized(&skill_path),
                 scope: SkillScope::System,
             }]
         );
