@@ -1,8 +1,10 @@
 mod env_var_dependencies;
 pub mod injection;
+pub(crate) mod invocation_utils;
 pub mod loader;
 pub mod manager;
 pub mod model;
+pub mod permissions;
 pub mod remote;
 pub mod render;
 pub mod system;
@@ -12,6 +14,8 @@ pub(crate) use env_var_dependencies::resolve_skill_dependencies_for_turn;
 pub(crate) use injection::SkillInjections;
 pub(crate) use injection::build_skill_injections;
 pub(crate) use injection::collect_explicit_skill_mentions;
+pub(crate) use invocation_utils::build_implicit_skill_path_indexes;
+pub(crate) use invocation_utils::maybe_emit_implicit_skill_invocation;
 pub use loader::load_skills;
 pub use manager::SkillsManager;
 pub use model::SkillError;
