@@ -3,10 +3,15 @@ use std::path::Component;
 use std::path::Path;
 use std::path::PathBuf;
 
+#[cfg(target_os = "macos")]
+use codex_protocol::models::MacOsAutomationValue;
+use codex_protocol::models::MacOsPermissions;
+#[cfg(target_os = "macos")]
+use codex_protocol::models::MacOsPreferencesValue;
+use codex_protocol::models::PermissionProfile;
 use codex_utils_absolute_path::AbsolutePathBuf;
 use dirs::home_dir;
 use dunce::canonicalize as canonicalize_path;
-use serde::Deserialize;
 use tracing::warn;
 
 use crate::config::Constrained;
@@ -20,63 +25,24 @@ use crate::seatbelt_permissions::MacOsSeatbeltProfileExtensions;
 #[cfg(not(target_os = "macos"))]
 type MacOsSeatbeltProfileExtensions = ();
 
-#[derive(Debug, Clone, PartialEq, Eq, Default, Deserialize)]
-pub(crate) struct SkillManifestPermissions {
-    #[serde(default)]
-    pub(crate) network: bool,
-    #[serde(default)]
-    pub(crate) file_system: SkillManifestFileSystemPermissions,
-    #[serde(default)]
-    pub(crate) macos: SkillManifestMacOsPermissions,
-}
-
-#[derive(Debug, Clone, PartialEq, Eq, Default, Deserialize)]
-pub(crate) struct SkillManifestFileSystemPermissions {
-    #[serde(default)]
-    pub(crate) read: Vec<String>,
-    #[serde(default)]
-    pub(crate) write: Vec<String>,
-}
-
-#[derive(Debug, Clone, PartialEq, Eq, Default, Deserialize)]
-pub(crate) struct SkillManifestMacOsPermissions {
-    #[serde(default)]
-    pub(crate) preferences: Option<MacOsPreferencesValue>,
-    #[serde(default)]
-    pub(crate) automations: Option<MacOsAutomationValue>,
-    #[serde(default)]
-    pub(crate) accessibility: bool,
-    #[serde(default)]
-    pub(crate) calendar: bool,
-}
-
-#[derive(Debug, Clone, PartialEq, Eq, Deserialize)]
-#[serde(untagged)]
-pub(crate) enum MacOsPreferencesValue {
-    Bool(bool),
-    Mode(String),
-}
-
-#[derive(Debug, Clone, PartialEq, Eq, Deserialize)]
-#[serde(untagged)]
-pub(crate) enum MacOsAutomationValue {
-    Bool(bool),
-    BundleIds(Vec<String>),
-}
-
 pub(crate) fn compile_permission_profile(
     skill_dir: &Path,
-    permissions: Option<SkillManifestPermissions>,
+    permissions: Option<PermissionProfile>,
 ) -> Option<Permissions> {
-    let permissions = permissions?;
+    let PermissionProfile {
+        network,
+        file_system,
+        macos,
+    } = permissions?;
+    let file_system = file_system.unwrap_or_default();
     let fs_read = normalize_permission_paths(
         skill_dir,
-        &permissions.file_system.read,
+        file_system.read.as_deref().unwrap_or_default(),
         "permissions.file_system.read",
     );
     let fs_write = normalize_permission_paths(
         skill_dir,
-        &permissions.file_system.write,
+        file_system.write.as_deref().unwrap_or_default(),
         "permissions.file_system.write",
     );
     let sandbox_policy = if !fs_write.is_empty() {
@@ -90,7 +56,7 @@ pub(crate) fn compile_permission_profile(
                     readable_roots: fs_read,
                 }
             },
-            network_access: permissions.network,
+            network_access: network.unwrap_or_default(),
             exclude_tmpdir_env_var: false,
             exclude_slash_tmp: false,
         }
@@ -105,13 +71,15 @@ pub(crate) fn compile_permission_profile(
         // Default sandbox policy
         SandboxPolicy::new_read_only_policy()
     };
+    let macos_permissions = macos.unwrap_or_default();
     let macos_seatbelt_profile_extensions =
-        build_macos_seatbelt_profile_extensions(&permissions.macos);
+        build_macos_seatbelt_profile_extensions(&macos_permissions);
 
     Some(Permissions {
         approval_policy: Constrained::allow_any(AskForApproval::Never),
         sandbox_policy: Constrained::allow_any(sandbox_policy),
         network: None,
+        allow_login_shell: true,
         shell_environment_policy: ShellEnvironmentPolicy::default(),
         windows_sandbox_mode: None,
         macos_seatbelt_profile_extensions,
@@ -120,7 +88,7 @@ pub(crate) fn compile_permission_profile(
 
 fn normalize_permission_paths(
     skill_dir: &Path,
-    values: &[String],
+    values: &[PathBuf],
     field: &str,
 ) -> Vec<AbsolutePathBuf> {
     let mut paths = Vec::new();
@@ -140,9 +108,10 @@ fn normalize_permission_paths(
 
 fn normalize_permission_path(
     skill_dir: &Path,
-    value: &str,
+    value: &Path,
     field: &str,
 ) -> Option<AbsolutePathBuf> {
+    let value = value.to_string_lossy();
     let trimmed = value.trim();
     if trimmed.is_empty() {
         warn!("ignoring {field}: value is empty");
@@ -150,11 +119,10 @@ fn normalize_permission_path(
     }
 
     let expanded = expand_home(trimmed);
-    let path = PathBuf::from(expanded);
-    let absolute = if path.is_absolute() {
-        path
+    let absolute = if expanded.is_absolute() {
+        expanded
     } else {
-        skill_dir.join(path)
+        skill_dir.join(expanded)
     };
     let normalized = normalize_lexically(&absolute);
     let canonicalized = canonicalize_path(&normalized).unwrap_or(normalized);
@@ -167,24 +135,24 @@ fn normalize_permission_path(
     }
 }
 
-fn expand_home(path: &str) -> String {
+fn expand_home(path: &str) -> PathBuf {
     if path == "~" {
         if let Some(home) = home_dir() {
-            return home.to_string_lossy().to_string();
+            return home;
         }
-        return path.to_string();
+        return PathBuf::from(path);
     }
     if let Some(rest) = path.strip_prefix("~/")
         && let Some(home) = home_dir()
     {
-        return home.join(rest).to_string_lossy().to_string();
+        return home.join(rest);
     }
-    path.to_string()
+    PathBuf::from(path)
 }
 
 #[cfg(target_os = "macos")]
 fn build_macos_seatbelt_profile_extensions(
-    permissions: &SkillManifestMacOsPermissions,
+    permissions: &MacOsPermissions,
 ) -> Option<MacOsSeatbeltProfileExtensions> {
     let defaults = MacOsSeatbeltProfileExtensions::default();
 
@@ -197,8 +165,10 @@ fn build_macos_seatbelt_profile_extensions(
             permissions.automations.as_ref(),
             defaults.macos_automation,
         ),
-        macos_accessibility: permissions.accessibility,
-        macos_calendar: permissions.calendar,
+        macos_accessibility: permissions
+            .accessibility
+            .unwrap_or(defaults.macos_accessibility),
+        macos_calendar: permissions.calendar.unwrap_or(defaults.macos_calendar),
     };
     Some(extensions)
 }
@@ -261,7 +231,7 @@ fn resolve_macos_automation_permission(
 
 #[cfg(not(target_os = "macos"))]
 fn build_macos_seatbelt_profile_extensions(
-    _: &SkillManifestMacOsPermissions,
+    _: &MacOsPermissions,
 ) -> Option<MacOsSeatbeltProfileExtensions> {
     None
 }
@@ -284,10 +254,6 @@ fn normalize_lexically(path: &Path) -> PathBuf {
 
 #[cfg(test)]
 mod tests {
-    use super::SkillManifestFileSystemPermissions;
-    #[cfg(target_os = "macos")]
-    use super::SkillManifestMacOsPermissions;
-    use super::SkillManifestPermissions;
     use super::compile_permission_profile;
     use crate::config::Constrained;
     use crate::config::Permissions;
@@ -295,9 +261,18 @@ mod tests {
     use crate::protocol::AskForApproval;
     use crate::protocol::ReadOnlyAccess;
     use crate::protocol::SandboxPolicy;
+    use codex_protocol::models::FileSystemPermissions;
+    #[cfg(target_os = "macos")]
+    use codex_protocol::models::MacOsAutomationValue;
+    #[cfg(target_os = "macos")]
+    use codex_protocol::models::MacOsPermissions;
+    #[cfg(target_os = "macos")]
+    use codex_protocol::models::MacOsPreferencesValue;
+    use codex_protocol::models::PermissionProfile;
     use codex_utils_absolute_path::AbsolutePathBuf;
     use pretty_assertions::assert_eq;
     use std::fs;
+    use std::path::PathBuf;
 
     #[test]
     fn compile_permission_profile_normalizes_paths() {
@@ -309,16 +284,16 @@ mod tests {
 
         let profile = compile_permission_profile(
             &skill_dir,
-            Some(SkillManifestPermissions {
-                network: true,
-                file_system: SkillManifestFileSystemPermissions {
-                    read: vec![
-                        "./data".to_string(),
-                        "./data".to_string(),
-                        "scripts/../data".to_string(),
-                    ],
-                    write: vec!["./output".to_string()],
-                },
+            Some(PermissionProfile {
+                network: Some(true),
+                file_system: Some(FileSystemPermissions {
+                    read: Some(vec![
+                        PathBuf::from("./data"),
+                        PathBuf::from("./data"),
+                        PathBuf::from("scripts/../data"),
+                    ]),
+                    write: Some(vec![PathBuf::from("./output")]),
+                }),
                 ..Default::default()
             }),
         )
@@ -347,6 +322,7 @@ mod tests {
                     exclude_slash_tmp: false,
                 }),
                 network: None,
+                allow_login_shell: true,
                 shell_environment_policy: ShellEnvironmentPolicy::default(),
                 windows_sandbox_mode: None,
                 #[cfg(target_os = "macos")]
@@ -378,8 +354,8 @@ mod tests {
 
         let profile = compile_permission_profile(
             &skill_dir,
-            Some(SkillManifestPermissions {
-                network: true,
+            Some(PermissionProfile {
+                network: Some(true),
                 ..Default::default()
             }),
         )
@@ -391,6 +367,7 @@ mod tests {
                 approval_policy: Constrained::allow_any(AskForApproval::Never),
                 sandbox_policy: Constrained::allow_any(SandboxPolicy::new_read_only_policy()),
                 network: None,
+                allow_login_shell: true,
                 shell_environment_policy: ShellEnvironmentPolicy::default(),
                 windows_sandbox_mode: None,
                 #[cfg(target_os = "macos")]
@@ -412,12 +389,12 @@ mod tests {
 
         let profile = compile_permission_profile(
             &skill_dir,
-            Some(SkillManifestPermissions {
-                network: true,
-                file_system: SkillManifestFileSystemPermissions {
-                    read: vec!["./data".to_string()],
-                    write: Vec::new(),
-                },
+            Some(PermissionProfile {
+                network: Some(true),
+                file_system: Some(FileSystemPermissions {
+                    read: Some(vec![PathBuf::from("./data")]),
+                    write: Some(Vec::new()),
+                }),
                 ..Default::default()
             }),
         )
@@ -439,6 +416,7 @@ mod tests {
                     },
                 }),
                 network: None,
+                allow_login_shell: true,
                 shell_environment_policy: ShellEnvironmentPolicy::default(),
                 windows_sandbox_mode: None,
                 #[cfg(target_os = "macos")]
@@ -460,15 +438,15 @@ mod tests {
 
         let profile = compile_permission_profile(
             &skill_dir,
-            Some(SkillManifestPermissions {
-                macos: SkillManifestMacOsPermissions {
-                    preferences: Some(super::MacOsPreferencesValue::Mode("readwrite".to_string())),
-                    automations: Some(super::MacOsAutomationValue::BundleIds(vec![
+            Some(PermissionProfile {
+                macos: Some(MacOsPermissions {
+                    preferences: Some(MacOsPreferencesValue::Mode("readwrite".to_string())),
+                    automations: Some(MacOsAutomationValue::BundleIds(vec![
                         "com.apple.Notes".to_string(),
                     ])),
-                    accessibility: true,
-                    calendar: true,
-                },
+                    accessibility: Some(true),
+                    calendar: Some(true),
+                }),
                 ..Default::default()
             }),
         )
@@ -498,9 +476,8 @@ mod tests {
         let skill_dir = tempdir.path().join("skill");
         fs::create_dir_all(&skill_dir).expect("skill dir");
 
-        let profile =
-            compile_permission_profile(&skill_dir, Some(SkillManifestPermissions::default()))
-                .expect("profile");
+        let profile = compile_permission_profile(&skill_dir, Some(PermissionProfile::default()))
+            .expect("profile");
 
         assert_eq!(
             profile.macos_seatbelt_profile_extensions,
