@@ -22,6 +22,8 @@ use std::collections::HashMap;
 use std::sync::Arc;
 use tracing::instrument;
 
+pub use crate::tools::context::ToolCallSource;
+
 #[derive(Clone, Debug)]
 pub struct ToolCall {
     pub tool_name: String,
@@ -29,12 +31,6 @@ pub struct ToolCall {
     pub payload: ToolPayload,
 }
 
-#[derive(Clone, Copy, Debug, Eq, PartialEq)]
-pub enum ToolCallSource {
-    Direct,
-    JsRepl,
-}
-
 pub struct ToolRouter {
     registry: ToolRegistry,
     specs: Vec<ConfiguredToolSpec>,
@@ -124,6 +120,7 @@ impl ToolRouter {
                             workdir: exec.working_directory,
                             timeout_ms: exec.timeout_ms,
                             sandbox_permissions: Some(SandboxPermissions::UseDefault),
+                            additional_permissions: None,
                             prefix_rule: None,
                             justification: None,
                         };
