@@ -9,7 +9,9 @@ use crate::path_utils;
 use crate::sandboxing::CommandSpec;
 use crate::sandboxing::SandboxPermissions;
 use crate::shell::Shell;
+use crate::skills::SkillMetadata;
 use crate::tools::sandboxing::ToolError;
+use codex_protocol::models::PermissionProfile;
 use std::collections::HashMap;
 use std::path::Path;
 
@@ -17,6 +19,14 @@ pub mod apply_patch;
 pub mod shell;
 pub mod unified_exec;
 
+#[derive(Debug, Clone)]
+pub(crate) struct ExecveSessionApproval {
+    /// If this execve session approval is associated with a skill script, this
+    /// field contains metadata about the skill.
+    #[cfg_attr(not(unix), allow(dead_code))]
+    pub skill: Option<SkillMetadata>,
+}
+
 /// Shared helper to construct a CommandSpec from a tokenized command line.
 /// Validates that at least a program is present.
 pub(crate) fn build_command_spec(
@@ -25,6 +35,7 @@ pub(crate) fn build_command_spec(
     env: &HashMap<String, String>,
     expiration: ExecExpiration,
     sandbox_permissions: SandboxPermissions,
+    additional_permissions: Option<PermissionProfile>,
     justification: Option<String>,
 ) -> Result<CommandSpec, ToolError> {
     let (program, args) = command
@@ -37,6 +48,7 @@ pub(crate) fn build_command_spec(
         env: env.clone(),
         expiration,
         sandbox_permissions,
+        additional_permissions,
         justification,
     })
 }
