@@ -4,6 +4,7 @@ use core_test_support::responses::WebSocketConnectionConfig;
 use core_test_support::responses::ev_assistant_message;
 use core_test_support::responses::ev_completed;
 use core_test_support::responses::ev_done;
+use core_test_support::responses::ev_done_with_id;
 use core_test_support::responses::ev_response_created;
 use core_test_support::responses::ev_shell_command_call;
 use core_test_support::responses::start_websocket_server;
@@ -38,24 +39,28 @@ async fn websocket_test_codex_shell_chain() -> Result<()> {
     let mut builder = test_codex();
 
     let test = builder.build_with_websocket_server(&server).await?;
-    test.submit_turn("run the echo command").await?;
+    test.submit_turn_with_policy(
+        "run the echo command",
+        test.config.permissions.sandbox_policy.get().clone(),
+    )
+    .await?;
 
     let connection = server.single_connection();
     assert_eq!(connection.len(), 2);
 
-    let first = connection
+    let first_turn = connection
         .first()
-        .expect("missing first request")
+        .expect("missing first turn request")
         .body_json();
-    let second = connection
+    let second_turn = connection
         .get(1)
-        .expect("missing second request")
+        .expect("missing second turn request")
         .body_json();
 
-    assert_eq!(first["type"].as_str(), Some("response.create"));
-    assert_eq!(second["type"].as_str(), Some("response.append"));
+    assert_eq!(first_turn["type"].as_str(), Some("response.create"));
+    assert_eq!(second_turn["type"].as_str(), Some("response.append"));
 
-    let append_items = second
+    let append_items = second_turn
         .get("input")
         .and_then(Value::as_array)
         .expect("response.append input array");
@@ -75,50 +80,81 @@ async fn websocket_test_codex_shell_chain() -> Result<()> {
 }
 
 #[tokio::test(flavor = "multi_thread", worker_threads = 2)]
-async fn websocket_preconnect_happens_on_session_start() -> Result<()> {
+async fn websocket_first_turn_uses_preconnect_and_create() -> Result<()> {
     skip_if_no_network!(Ok(()));
 
     let server = start_websocket_server(vec![vec![vec![
         ev_response_created("resp-1"),
+        ev_assistant_message("msg-1", "hello"),
         ev_completed("resp-1"),
     ]]])
     .await;
 
     let mut builder = test_codex();
     let test = builder.build_with_websocket_server(&server).await?;
-
-    assert!(
-        server.wait_for_handshakes(1, Duration::from_secs(2)).await,
-        "expected websocket preconnect handshake during session startup"
-    );
-
-    test.submit_turn("hello").await?;
+    test.submit_turn_with_policy(
+        "hello",
+        test.config.permissions.sandbox_policy.get().clone(),
+    )
+    .await?;
 
     assert_eq!(server.handshakes().len(), 1);
-    assert_eq!(server.single_connection().len(), 1);
+    let connection = server.single_connection();
+    assert_eq!(connection.len(), 1);
+    let turn = connection
+        .first()
+        .expect("missing turn request")
+        .body_json();
+    assert!(
+        turn["tools"]
+            .as_array()
+            .is_some_and(|tools| !tools.is_empty()),
+        "expected request tools to be populated"
+    );
+    assert_eq!(turn["type"].as_str(), Some("response.create"));
 
     server.shutdown().await;
     Ok(())
 }
 
 #[tokio::test(flavor = "multi_thread", worker_threads = 2)]
-async fn websocket_first_turn_waits_for_inflight_preconnect() -> Result<()> {
+async fn websocket_first_turn_handles_handshake_delay_with_preconnect() -> Result<()> {
     skip_if_no_network!(Ok(()));
 
     let server = start_websocket_server_with_headers(vec![WebSocketConnectionConfig {
-        requests: vec![vec![ev_response_created("resp-1"), ev_completed("resp-1")]],
+        requests: vec![vec![
+            ev_response_created("resp-1"),
+            ev_assistant_message("msg-1", "hello"),
+            ev_completed("resp-1"),
+        ]],
         response_headers: Vec::new(),
-        // Delay handshake so submit_turn() observes startup preconnect as in-flight.
+        // Delay handshake so turn processing must tolerate websocket startup latency.
         accept_delay: Some(Duration::from_millis(150)),
     }])
     .await;
 
     let mut builder = test_codex();
     let test = builder.build_with_websocket_server(&server).await?;
-    test.submit_turn("hello").await?;
+    test.submit_turn_with_policy(
+        "hello",
+        test.config.permissions.sandbox_policy.get().clone(),
+    )
+    .await?;
 
     assert_eq!(server.handshakes().len(), 1);
-    assert_eq!(server.single_connection().len(), 1);
+    let connection = server.single_connection();
+    assert_eq!(connection.len(), 1);
+    let turn = connection
+        .first()
+        .expect("missing turn request")
+        .body_json();
+    assert!(
+        turn["tools"]
+            .as_array()
+            .is_some_and(|tools| !tools.is_empty()),
+        "expected request tools to be populated"
+    );
+    assert_eq!(turn["type"].as_str(), Some("response.create"));
 
     server.shutdown().await;
     Ok(())
@@ -130,6 +166,7 @@ async fn websocket_v2_test_codex_shell_chain() -> Result<()> {
 
     let call_id = "shell-command-call";
     let server = start_websocket_server(vec![vec![
+        vec![ev_response_created("warm-1"), ev_done_with_id("warm-1")],
         vec![
             ev_response_created("resp-1"),
             ev_shell_command_call(call_id, "echo websocket"),
@@ -148,25 +185,42 @@ async fn websocket_v2_test_codex_shell_chain() -> Result<()> {
     });
 
     let test = builder.build_with_websocket_server(&server).await?;
-    test.submit_turn("run the echo command").await?;
+    test.submit_turn_with_policy(
+        "run the echo command",
+        test.config.permissions.sandbox_policy.get().clone(),
+    )
+    .await?;
 
     let connection = server.single_connection();
-    assert_eq!(connection.len(), 2);
+    assert_eq!(connection.len(), 3);
 
-    let first = connection
+    let warmup = connection
         .first()
-        .expect("missing first request")
+        .expect("missing warmup request")
         .body_json();
-    let second = connection
+    let first_turn = connection
         .get(1)
-        .expect("missing second request")
+        .expect("missing first turn request")
+        .body_json();
+    let second_turn = connection
+        .get(2)
+        .expect("missing second turn request")
         .body_json();
 
-    assert_eq!(first["type"].as_str(), Some("response.create"));
-    assert_eq!(second["type"].as_str(), Some("response.create"));
-    assert_eq!(second["previous_response_id"].as_str(), Some("resp-1"));
+    assert_eq!(warmup["type"].as_str(), Some("response.create"));
+    assert_eq!(warmup["generate"].as_bool(), Some(false));
+    assert_eq!(first_turn["type"].as_str(), Some("response.create"));
+    assert_eq!(first_turn["previous_response_id"].as_str(), Some("warm-1"));
+    assert!(
+        first_turn
+            .get("input")
+            .and_then(Value::as_array)
+            .is_some_and(|items| !items.is_empty())
+    );
+    assert_eq!(second_turn["type"].as_str(), Some("response.create"));
+    assert_eq!(second_turn["previous_response_id"].as_str(), Some("resp-1"));
 
-    let create_items = second
+    let create_items = second_turn
         .get("input")
         .and_then(Value::as_array)
         .expect("response.create input array");
