@@ -2,19 +2,19 @@
 
 use codex_apply_patch::APPLY_PATCH_TOOL_INSTRUCTIONS;
 use codex_core::features::Feature;
-use codex_core::protocol::AskForApproval;
-use codex_core::protocol::ENVIRONMENT_CONTEXT_OPEN_TAG;
-use codex_core::protocol::EventMsg;
-use codex_core::protocol::Op;
-use codex_core::protocol::SandboxPolicy;
-use codex_core::protocol_config_types::ReasoningSummary;
 use codex_core::shell::Shell;
 use codex_core::shell::default_user_shell;
 use codex_protocol::config_types::CollaborationMode;
 use codex_protocol::config_types::ModeKind;
+use codex_protocol::config_types::ReasoningSummary;
 use codex_protocol::config_types::Settings;
 use codex_protocol::config_types::WebSearchMode;
 use codex_protocol::openai_models::ReasoningEffort;
+use codex_protocol::protocol::AskForApproval;
+use codex_protocol::protocol::ENVIRONMENT_CONTEXT_OPEN_TAG;
+use codex_protocol::protocol::EventMsg;
+use codex_protocol::protocol::Op;
+use codex_protocol::protocol::SandboxPolicy;
 use codex_protocol::user_input::UserInput;
 use codex_utils_absolute_path::AbsolutePathBuf;
 use core_test_support::responses::ev_completed;
@@ -30,10 +30,17 @@ use pretty_assertions::assert_eq;
 use tempfile::TempDir;
 
 fn text_user_input(text: String) -> serde_json::Value {
+    text_user_input_parts(vec![text])
+}
+
+fn text_user_input_parts(texts: Vec<String>) -> serde_json::Value {
     serde_json::json!({
         "type": "message",
         "role": "user",
-        "content": [ { "type": "input_text", "text": text } ]
+        "content": texts
+            .into_iter()
+            .map(|text| serde_json::json!({ "type": "input_text", "text": text }))
+            .collect::<Vec<_>>()
     })
 }
 
@@ -297,8 +304,8 @@ async fn prefixes_context_and_instructions_once_and_consistently_across_requests
     let input1 = body1["input"].as_array().expect("input array");
     assert_eq!(
         input1.len(),
-        4,
-        "expected permissions + cached prefix + env + user msg"
+        3,
+        "expected permissions + cached contextual user prefix + user msg"
     );
 
     let ui_text = input1[1]["content"][0]["text"]
@@ -313,11 +320,11 @@ async fn prefixes_context_and_instructions_once_and_consistently_across_requests
     let cwd_str = config.cwd.to_string_lossy();
     let expected_env_text = default_env_context_str(&cwd_str, &shell);
     assert_eq!(
-        input1[2],
-        text_user_input(expected_env_text),
-        "expected environment context after UI message"
+        input1[1]["content"][1]["text"].as_str(),
+        Some(expected_env_text.as_str()),
+        "expected environment context bundled after UI message in cached contextual message"
     );
-    assert_eq!(input1[3], text_user_input("hello 1".to_string()));
+    assert_eq!(input1[2], text_user_input("hello 1".to_string()));
 
     let body2 = req2.single_request().body_json();
     let input2 = body2["input"].as_array().expect("input array");
@@ -402,8 +409,10 @@ async fn overrides_turn_context_but_keeps_cached_prefix_and_key_constant() -> an
         .await?;
     wait_for_event(&codex, |ev| matches!(ev, EventMsg::TurnComplete(_))).await;
 
-    let body1 = req1.single_request().body_json();
-    let body2 = req2.single_request().body_json();
+    let request1 = req1.single_request();
+    let request2 = req2.single_request();
+    let body1 = request1.body_json();
+    let body2 = request2.body_json();
     // prompt_cache_key should remain constant across overrides
     assert_eq!(
         body1["prompt_cache_key"], body2["prompt_cache_key"],
@@ -500,11 +509,14 @@ async fn override_before_first_turn_emits_environment_context() -> anyhow::Resul
     let env_texts: Vec<&str> = input
         .iter()
         .filter_map(|msg| {
-            msg["content"]
-                .as_array()
-                .and_then(|content| content.first())
-                .and_then(|item| item["text"].as_str())
+            msg["content"].as_array().map(|content| {
+                content
+                    .iter()
+                    .filter_map(|item| item["text"].as_str())
+                    .collect::<Vec<_>>()
+            })
         })
+        .flatten()
         .filter(|text| text.starts_with(ENVIRONMENT_CONTEXT_OPEN_TAG))
         .collect();
     assert!(
@@ -559,11 +571,14 @@ async fn override_before_first_turn_emits_environment_context() -> anyhow::Resul
     let user_texts: Vec<&str> = input
         .iter()
         .filter_map(|msg| {
-            msg["content"]
-                .as_array()
-                .and_then(|content| content.first())
-                .and_then(|item| item["text"].as_str())
+            msg["content"].as_array().map(|content| {
+                content
+                    .iter()
+                    .filter_map(|item| item["text"].as_str())
+                    .collect::<Vec<_>>()
+            })
         })
+        .flatten()
         .collect();
     assert!(
         user_texts.contains(&"first message"),
@@ -639,8 +654,10 @@ async fn per_turn_overrides_keep_cached_prefix_and_key_constant() -> anyhow::Res
         .await?;
     wait_for_event(&codex, |ev| matches!(ev, EventMsg::TurnComplete(_))).await;
 
-    let body1 = req1.single_request().body_json();
-    let body2 = req2.single_request().body_json();
+    let request1 = req1.single_request();
+    let request2 = req2.single_request();
+    let body1 = request1.body_json();
+    let body2 = request2.body_json();
 
     // prompt_cache_key should remain constant across per-turn overrides
     assert_eq!(
@@ -672,26 +689,24 @@ async fn per_turn_overrides_keep_cached_prefix_and_key_constant() -> anyhow::Res
     });
     let expected_permissions_msg = body1["input"][0].clone();
     let body1_input = body1["input"].as_array().expect("input array");
-    let expected_permissions_msg_2 = body2["input"][body1_input.len() + 1].clone();
+    let expected_settings_update_msg = body2["input"][body1_input.len()].clone();
     assert_ne!(
-        expected_permissions_msg_2, expected_permissions_msg,
+        expected_settings_update_msg, expected_permissions_msg,
         "expected updated permissions message after per-turn override"
     );
-    let expected_model_switch_msg = body2["input"][body1_input.len() + 2].clone();
     assert_eq!(
-        expected_model_switch_msg["role"].as_str(),
+        expected_settings_update_msg["role"].as_str(),
         Some("developer")
     );
     assert!(
-        expected_model_switch_msg["content"][0]["text"]
-            .as_str()
-            .is_some_and(|text| text.contains("<model_switch>")),
-        "expected model switch message after model override: {expected_model_switch_msg:?}"
+        request2.has_message_with_input_texts("developer", |texts| {
+            texts.iter().any(|text| text.contains("<model_switch>"))
+        }),
+        "expected model switch section after model override: {expected_settings_update_msg:?}"
     );
     let mut expected_body2 = body1_input.to_vec();
+    expected_body2.push(expected_settings_update_msg);
     expected_body2.push(expected_env_msg_2);
-    expected_body2.push(expected_permissions_msg_2);
-    expected_body2.push(expected_model_switch_msg);
     expected_body2.push(expected_user_message_2);
     assert_eq!(body2["input"], serde_json::Value::Array(expected_body2));
 
@@ -746,7 +761,7 @@ async fn send_user_turn_with_no_changes_does_not_send_environment_context() -> a
             sandbox_policy: default_sandbox_policy.clone(),
             model: default_model.clone(),
             effort: default_effort,
-            summary: default_summary,
+            summary: default_summary.unwrap_or(ReasoningSummary::Auto),
             collaboration_mode: None,
             final_output_json_schema: None,
             personality: None,
@@ -765,7 +780,7 @@ async fn send_user_turn_with_no_changes_does_not_send_environment_context() -> a
             sandbox_policy: default_sandbox_policy.clone(),
             model: default_model.clone(),
             effort: default_effort,
-            summary: default_summary,
+            summary: default_summary.unwrap_or(ReasoningSummary::Auto),
             collaboration_mode: None,
             final_output_json_schema: None,
             personality: None,
@@ -773,8 +788,10 @@ async fn send_user_turn_with_no_changes_does_not_send_environment_context() -> a
         .await?;
     wait_for_event(&codex, |ev| matches!(ev, EventMsg::TurnComplete(_))).await;
 
-    let body1 = req1.single_request().body_json();
-    let body2 = req2.single_request().body_json();
+    let request1 = req1.single_request();
+    let request2 = req2.single_request();
+    let body1 = request1.body_json();
+    let body2 = request2.body_json();
 
     let expected_permissions_msg = body1["input"][0].clone();
     let expected_ui_msg = body1["input"][1].clone();
@@ -782,13 +799,18 @@ async fn send_user_turn_with_no_changes_does_not_send_environment_context() -> a
     let shell = default_user_shell();
     let default_cwd_lossy = default_cwd.to_string_lossy();
 
-    let expected_env_msg_1 = text_user_input(default_env_context_str(&default_cwd_lossy, &shell));
+    let expected_contextual_user_msg_1 = text_user_input_parts(vec![
+        expected_ui_msg["content"][0]["text"]
+            .as_str()
+            .expect("cached user instructions text")
+            .to_string(),
+        default_env_context_str(&default_cwd_lossy, &shell),
+    ]);
     let expected_user_message_1 = text_user_input("hello 1".to_string());
 
     let expected_input_1 = serde_json::Value::Array(vec![
         expected_permissions_msg.clone(),
-        expected_ui_msg.clone(),
-        expected_env_msg_1.clone(),
+        expected_contextual_user_msg_1.clone(),
         expected_user_message_1.clone(),
     ]);
     assert_eq!(body1["input"], expected_input_1);
@@ -796,8 +818,7 @@ async fn send_user_turn_with_no_changes_does_not_send_environment_context() -> a
     let expected_user_message_2 = text_user_input("hello 2".to_string());
     let expected_input_2 = serde_json::Value::Array(vec![
         expected_permissions_msg,
-        expected_ui_msg,
-        expected_env_msg_1,
+        expected_contextual_user_msg_1,
         expected_user_message_1,
         expected_user_message_2,
     ]);
@@ -854,7 +875,7 @@ async fn send_user_turn_with_changes_sends_environment_context() -> anyhow::Resu
             sandbox_policy: default_sandbox_policy.clone(),
             model: default_model,
             effort: default_effort,
-            summary: default_summary,
+            summary: default_summary.unwrap_or(ReasoningSummary::Auto),
             collaboration_mode: None,
             final_output_json_schema: None,
             personality: None,
@@ -881,49 +902,53 @@ async fn send_user_turn_with_changes_sends_environment_context() -> anyhow::Resu
         .await?;
     wait_for_event(&codex, |ev| matches!(ev, EventMsg::TurnComplete(_))).await;
 
-    let body1 = req1.single_request().body_json();
-    let body2 = req2.single_request().body_json();
+    let request1 = req1.single_request();
+    let request2 = req2.single_request();
+    let body1 = request1.body_json();
+    let body2 = request2.body_json();
 
     let expected_permissions_msg = body1["input"][0].clone();
     let expected_ui_msg = body1["input"][1].clone();
 
     let shell = default_user_shell();
     let expected_env_text_1 = default_env_context_str(&default_cwd.to_string_lossy(), &shell);
-    let expected_env_msg_1 = text_user_input(expected_env_text_1);
+    let expected_contextual_user_msg_1 = text_user_input_parts(vec![
+        expected_ui_msg["content"][0]["text"]
+            .as_str()
+            .expect("cached user instructions text")
+            .to_string(),
+        expected_env_text_1,
+    ]);
     let expected_user_message_1 = text_user_input("hello 1".to_string());
     let expected_input_1 = serde_json::Value::Array(vec![
         expected_permissions_msg.clone(),
-        expected_ui_msg.clone(),
-        expected_env_msg_1.clone(),
+        expected_contextual_user_msg_1.clone(),
         expected_user_message_1.clone(),
     ]);
     assert_eq!(body1["input"], expected_input_1);
 
     let body1_input = body1["input"].as_array().expect("input array");
-    let expected_permissions_msg_2 = body2["input"][body1_input.len()].clone();
+    let expected_settings_update_msg = body2["input"][body1_input.len()].clone();
     assert_ne!(
-        expected_permissions_msg_2, expected_permissions_msg,
+        expected_settings_update_msg, expected_permissions_msg,
         "expected updated permissions message after policy change"
     );
-    let expected_model_switch_msg = body2["input"][body1_input.len() + 1].clone();
     assert_eq!(
-        expected_model_switch_msg["role"].as_str(),
+        expected_settings_update_msg["role"].as_str(),
         Some("developer")
     );
     assert!(
-        expected_model_switch_msg["content"][0]["text"]
-            .as_str()
-            .is_some_and(|text| text.contains("<model_switch>")),
-        "expected model switch message after model override: {expected_model_switch_msg:?}"
+        request2.has_message_with_input_texts("developer", |texts| {
+            texts.iter().any(|text| text.contains("<model_switch>"))
+        }),
+        "expected model switch section after model override: {expected_settings_update_msg:?}"
     );
     let expected_user_message_2 = text_user_input("hello 2".to_string());
     let expected_input_2 = serde_json::Value::Array(vec![
         expected_permissions_msg,
-        expected_ui_msg,
-        expected_env_msg_1,
+        expected_contextual_user_msg_1,
         expected_user_message_1,
-        expected_permissions_msg_2,
-        expected_model_switch_msg,
+        expected_settings_update_msg,
         expected_user_message_2,
     ]);
     assert_eq!(body2["input"], expected_input_2);
