@@ -9,11 +9,6 @@ use codex_core::ModelProviderInfo;
 use codex_core::built_in_model_providers;
 use codex_core::models_manager::manager::ModelsManager;
 use codex_core::models_manager::manager::RefreshStrategy;
-use codex_core::protocol::AskForApproval;
-use codex_core::protocol::EventMsg;
-use codex_core::protocol::ExecCommandSource;
-use codex_core::protocol::Op;
-use codex_core::protocol::SandboxPolicy;
 use codex_protocol::config_types::ReasoningSummary;
 use codex_protocol::openai_models::ConfigShellToolType;
 use codex_protocol::openai_models::ModelInfo;
@@ -24,6 +19,11 @@ use codex_protocol::openai_models::ReasoningEffort;
 use codex_protocol::openai_models::ReasoningEffortPreset;
 use codex_protocol::openai_models::TruncationPolicyConfig;
 use codex_protocol::openai_models::default_input_modalities;
+use codex_protocol::protocol::AskForApproval;
+use codex_protocol::protocol::EventMsg;
+use codex_protocol::protocol::ExecCommandSource;
+use codex_protocol::protocol::Op;
+use codex_protocol::protocol::SandboxPolicy;
 use codex_protocol::user_input::UserInput;
 use core_test_support::load_default_config_for_test;
 use core_test_support::responses::ev_assistant_message;
@@ -175,7 +175,9 @@ async fn remote_models_long_model_slug_is_sent_with_high_reasoning() -> Result<(
             sandbox_policy: config.permissions.sandbox_policy.get().clone(),
             model: requested_model.to_string(),
             effort: None,
-            summary: config.model_reasoning_summary,
+            summary: config
+                .model_reasoning_summary
+                .unwrap_or(ReasoningSummary::Auto),
             collaboration_mode: None,
             personality: None,
         })
@@ -195,6 +197,67 @@ async fn remote_models_long_model_slug_is_sent_with_high_reasoning() -> Result<(
     Ok(())
 }
 
+#[tokio::test(flavor = "multi_thread", worker_threads = 2)]
+async fn namespaced_model_slug_uses_catalog_metadata_without_fallback_warning() -> Result<()> {
+    skip_if_no_network!(Ok(()));
+    skip_if_sandbox!(Ok(()));
+
+    let server = MockServer::start().await;
+    let requested_model = "custom/gpt-5.2-codex";
+    let response_mock = mount_sse_once(
+        &server,
+        sse(vec![ev_response_created("resp-1"), ev_completed("resp-1")]),
+    )
+    .await;
+
+    let TestCodex {
+        codex, cwd, config, ..
+    } = test_codex()
+        .with_model(requested_model)
+        .build(&server)
+        .await?;
+
+    codex
+        .submit(Op::UserTurn {
+            items: vec![UserInput::Text {
+                text: "check namespaced model metadata".into(),
+                text_elements: Vec::new(),
+            }],
+            final_output_json_schema: None,
+            cwd: cwd.path().to_path_buf(),
+            approval_policy: config.permissions.approval_policy.value(),
+            sandbox_policy: config.permissions.sandbox_policy.get().clone(),
+            model: requested_model.to_string(),
+            effort: None,
+            summary: config
+                .model_reasoning_summary
+                .unwrap_or(ReasoningSummary::Auto),
+            collaboration_mode: None,
+            personality: None,
+        })
+        .await?;
+
+    let mut fallback_warning_count = 0;
+    loop {
+        let event = wait_for_event(&codex, |_| true).await;
+        match event {
+            EventMsg::Warning(warning)
+                if warning.message.contains("Defaulting to fallback metadata") =>
+            {
+                fallback_warning_count += 1;
+            }
+            EventMsg::TurnComplete(_) => break,
+            _ => {}
+        }
+    }
+
+    let body = response_mock.single_request().body_json();
+    assert_eq!(body["model"].as_str(), Some(requested_model));
+    assert_eq!(fallback_warning_count, 0);
+
+    Ok(())
+}
+
 #[tokio::test(flavor = "multi_thread", worker_threads = 2)]
 async fn remote_models_remote_model_uses_unified_exec() -> Result<()> {
     skip_if_no_network!(Ok(()));
@@ -225,6 +288,7 @@ async fn remote_models_remote_model_uses_unified_exec() -> Result<()> {
         base_instructions: "base instructions".to_string(),
         model_messages: None,
         supports_reasoning_summaries: false,
+        default_reasoning_summary: ReasoningSummary::Auto,
         support_verbosity: false,
         default_verbosity: None,
         apply_patch_tool_type: None,
@@ -461,6 +525,7 @@ async fn remote_models_apply_remote_base_instructions() -> Result<()> {
         base_instructions: remote_base.to_string(),
         model_messages: None,
         supports_reasoning_summaries: false,
+        default_reasoning_summary: ReasoningSummary::Auto,
         support_verbosity: false,
         default_verbosity: None,
         apply_patch_tool_type: None,
@@ -548,7 +613,7 @@ async fn remote_models_apply_remote_base_instructions() -> Result<()> {
 }
 
 #[tokio::test(flavor = "multi_thread", worker_threads = 2)]
-async fn remote_models_preserve_builtin_presets() -> Result<()> {
+async fn remote_models_do_not_append_removed_builtin_presets() -> Result<()> {
     skip_if_no_network!(Ok(()));
     skip_if_sandbox!(Ok(()));
 
@@ -593,12 +658,6 @@ async fn remote_models_preserve_builtin_presets() -> Result<()> {
         1,
         "expected a single default model"
     );
-    assert!(
-        available
-            .iter()
-            .any(|model| model.model == "gpt-5.1-codex-max"),
-        "builtin presets should remain available after refresh"
-    );
     assert_eq!(
         models_mock.requests().len(),
         1,
@@ -777,8 +836,9 @@ async fn remote_models_request_times_out_after_5s() -> Result<()> {
     let elapsed = start.elapsed();
     // get_model should return a default model even when refresh times out
     let default_model = model.expect("get_model should finish and return default model");
+    let expected_default = bundled_default_model_slug();
     assert!(
-        default_model == "gpt-5.2-codex",
+        default_model == expected_default,
         "get_model should return default model when refresh times out, got: {default_model}"
     );
     let _ = server
@@ -836,7 +896,7 @@ async fn remote_models_hide_picker_only_models() -> Result<()> {
     let selected = manager
         .get_default_model(&None, RefreshStrategy::OnlineIfUncached)
         .await;
-    assert_eq!(selected, "gpt-5.2-codex");
+    assert_eq!(selected, bundled_default_model_slug());
 
     let available = manager.list_models(RefreshStrategy::OnlineIfUncached).await;
     let hidden = available
@@ -882,6 +942,15 @@ fn bundled_model_slug() -> String {
         .clone()
 }
 
+fn bundled_default_model_slug() -> String {
+    codex_core::test_support::all_model_presets()
+        .iter()
+        .find(|preset| preset.is_default)
+        .expect("bundled models should include a default")
+        .model
+        .clone()
+}
+
 fn test_remote_model(slug: &str, visibility: ModelVisibility, priority: i32) -> ModelInfo {
     test_remote_model_with_policy(
         slug,
@@ -917,6 +986,7 @@ fn test_remote_model_with_policy(
         base_instructions: "base instructions".to_string(),
         model_messages: None,
         supports_reasoning_summaries: false,
+        default_reasoning_summary: ReasoningSummary::Auto,
         support_verbosity: false,
         default_verbosity: None,
         apply_patch_tool_type: None,
