@@ -3,8 +3,8 @@
 
 use codex_core::ModelProviderInfo;
 use codex_core::WireApi;
-use codex_core::protocol::EventMsg;
-use codex_core::protocol::Op;
+use codex_protocol::protocol::EventMsg;
+use codex_protocol::protocol::Op;
 use codex_protocol::user_input::UserInput;
 use core_test_support::load_sse_fixture;
 use core_test_support::responses::ev_completed;
