@@ -4,10 +4,6 @@ use base64::Engine;
 use base64::engine::general_purpose::STANDARD as BASE64_STANDARD;
 use codex_core::CodexAuth;
 use codex_core::features::Feature;
-use codex_core::protocol::AskForApproval;
-use codex_core::protocol::EventMsg;
-use codex_core::protocol::Op;
-use codex_core::protocol::SandboxPolicy;
 use codex_protocol::config_types::ReasoningSummary;
 use codex_protocol::openai_models::ConfigShellToolType;
 use codex_protocol::openai_models::InputModality;
@@ -17,6 +13,10 @@ use codex_protocol::openai_models::ModelsResponse;
 use codex_protocol::openai_models::ReasoningEffort;
 use codex_protocol::openai_models::ReasoningEffortPreset;
 use codex_protocol::openai_models::TruncationPolicyConfig;
+use codex_protocol::protocol::AskForApproval;
+use codex_protocol::protocol::EventMsg;
+use codex_protocol::protocol::Op;
+use codex_protocol::protocol::SandboxPolicy;
 use codex_protocol::user_input::UserInput;
 use core_test_support::responses;
 use core_test_support::responses::ev_assistant_message;
@@ -36,28 +36,37 @@ use image::GenericImageView;
 use image::ImageBuffer;
 use image::Rgba;
 use image::load_from_memory;
+use pretty_assertions::assert_eq;
 use serde_json::Value;
 use tokio::time::Duration;
 use wiremock::BodyPrintLimit;
 use wiremock::MockServer;
 
-fn find_image_message(body: &Value) -> Option<&Value> {
+fn image_messages(body: &Value) -> Vec<&Value> {
     body.get("input")
         .and_then(Value::as_array)
-        .and_then(|items| {
-            items.iter().find(|item| {
-                item.get("type").and_then(Value::as_str) == Some("message")
-                    && item
-                        .get("content")
-                        .and_then(Value::as_array)
-                        .map(|content| {
-                            content.iter().any(|span| {
-                                span.get("type").and_then(Value::as_str) == Some("input_image")
+        .map(|items| {
+            items
+                .iter()
+                .filter(|item| {
+                    item.get("type").and_then(Value::as_str) == Some("message")
+                        && item
+                            .get("content")
+                            .and_then(Value::as_array)
+                            .map(|content| {
+                                content.iter().any(|span| {
+                                    span.get("type").and_then(Value::as_str) == Some("input_image")
+                                })
                             })
-                        })
-                        .unwrap_or(false)
-            })
+                            .unwrap_or(false)
+                })
+                .collect()
         })
+        .unwrap_or_default()
+}
+
+fn find_image_message(body: &Value) -> Option<&Value> {
+    image_messages(body).into_iter().next()
 }
 
 #[tokio::test(flavor = "multi_thread", worker_threads = 2)]
@@ -237,40 +246,29 @@ async fn view_image_tool_attaches_local_image() -> anyhow::Result<()> {
 
     let req = mock.single_request();
     let body = req.body_json();
-    let output_text = req
-        .function_call_output_content_and_success(call_id)
-        .and_then(|(content, _)| content)
-        .expect("output text present");
-    assert_eq!(output_text, "attached local image path");
+    assert!(
+        find_image_message(&body).is_none(),
+        "view_image tool should not inject a separate image message"
+    );
 
-    let image_message =
-        find_image_message(&body).expect("pending input image message not included in request");
-    let content_items = image_message
-        .get("content")
+    let function_output = req.function_call_output(call_id);
+    let output_items = function_output
+        .get("output")
         .and_then(Value::as_array)
-        .expect("image message has content array");
+        .expect("function_call_output should be a content item array");
     assert_eq!(
-        content_items.len(),
+        output_items.len(),
         1,
-        "view_image should inject only the image content item (no tag/label text)"
+        "view_image should return only the image content item (no tag/label text)"
     );
     assert_eq!(
-        content_items[0].get("type").and_then(Value::as_str),
+        output_items[0].get("type").and_then(Value::as_str),
         Some("input_image"),
-        "view_image should inject only an input_image content item"
+        "view_image should return only an input_image content item"
     );
-    let image_url = image_message
-        .get("content")
-        .and_then(Value::as_array)
-        .and_then(|content| {
-            content.iter().find_map(|span| {
-                if span.get("type").and_then(Value::as_str) == Some("input_image") {
-                    span.get("image_url").and_then(Value::as_str)
-                } else {
-                    None
-                }
-            })
-        })
+    let image_url = output_items[0]
+        .get("image_url")
+        .and_then(Value::as_str)
         .expect("image_url present");
 
     let (prefix, encoded) = image_url
@@ -364,12 +362,6 @@ console.log(out.output?.body?.text ?? "");
         .custom_tool_call_output_content_and_success(call_id)
         .expect("custom tool output present");
     let js_repl_output = js_repl_output.expect("custom tool output text present");
-    if js_repl_output.contains("Node runtime not found")
-        || js_repl_output.contains("Node runtime too old for js_repl")
-    {
-        eprintln!("Skipping js_repl image test: {js_repl_output}");
-        return Ok(());
-    }
     assert_ne!(
         js_repl_success,
         Some(false),
@@ -377,8 +369,16 @@ console.log(out.output?.body?.text ?? "");
     );
 
     let body = req.body_json();
-    let image_message =
-        find_image_message(&body).expect("pending input image message not included in request");
+    let image_messages = image_messages(&body);
+    assert_eq!(
+        image_messages.len(),
+        1,
+        "js_repl view_image should inject exactly one pending input image message"
+    );
+    let image_message = image_messages
+        .into_iter()
+        .next()
+        .expect("pending input image message not included in request");
     let image_url = image_message
         .get("content")
         .and_then(Value::as_array)
@@ -535,38 +535,36 @@ async fn view_image_tool_placeholder_for_non_image_files() -> anyhow::Result<()>
         request.inputs_of_type("input_image").is_empty(),
         "non-image file should not produce an input_image message"
     );
+    let function_output = request.function_call_output(call_id);
+    let output_items = function_output
+        .get("output")
+        .and_then(Value::as_array)
+        .expect("function_call_output should be a content item array");
+    assert_eq!(
+        output_items.len(),
+        1,
+        "non-image placeholder should be returned as a single content item"
+    );
+    assert_eq!(
+        output_items[0].get("type").and_then(Value::as_str),
+        Some("input_text"),
+        "non-image placeholder should be returned as input_text"
+    );
+    let placeholder = output_items[0]
+        .get("text")
+        .and_then(Value::as_str)
+        .expect("placeholder text present");
 
-    let placeholder = request
-        .inputs_of_type("message")
-        .iter()
-        .find_map(|item| {
-            let content = item.get("content").and_then(Value::as_array)?;
-            content.iter().find_map(|span| {
-                if span.get("type").and_then(Value::as_str) == Some("input_text") {
-                    let text = span.get("text").and_then(Value::as_str)?;
-                    if text.contains("Codex could not read the local image at")
-                        && text.contains("unsupported MIME type `application/json`")
-                    {
-                        return Some(text.to_string());
-                    }
-                }
-                None
-            })
-        })
-        .expect("placeholder text found");
-
+    assert!(
+        placeholder.contains("Codex could not read the local image at")
+            && placeholder.contains("unsupported MIME type `application/json`"),
+        "placeholder should describe the unsupported file type: {placeholder}"
+    );
     assert!(
         placeholder.contains(&abs_path.display().to_string()),
         "placeholder should mention path: {placeholder}"
     );
 
-    let output_text = mock
-        .single_request()
-        .function_call_output_content_and_success(call_id)
-        .and_then(|(content, _)| content)
-        .expect("output text present");
-    assert_eq!(output_text, "attached local image path");
-
     Ok(())
 }
 
@@ -677,6 +675,7 @@ async fn view_image_tool_returns_unsupported_message_for_text_only_model() -> an
         base_instructions: "base instructions".to_string(),
         model_messages: None,
         supports_reasoning_summaries: false,
+        default_reasoning_summary: ReasoningSummary::Auto,
         support_verbosity: false,
         default_verbosity: None,
         apply_patch_tool_type: None,
