@@ -2,6 +2,8 @@
 mod add_dir;
 mod apply_patch;
 mod auth_env;
+mod ephemeral;
+mod mcp_required_exit;
 mod originator;
 mod output_schema;
 mod resume;
