@@ -8,6 +8,7 @@ pub mod rule;
 
 pub use amend::AmendError;
 pub use amend::blocking_append_allow_prefix_rule;
+pub use amend::blocking_append_network_rule;
 pub use decision::Decision;
 pub use error::Error;
 pub use error::ErrorLocation;
@@ -18,6 +19,7 @@ pub use execpolicycheck::ExecPolicyCheckCommand;
 pub use parser::PolicyParser;
 pub use policy::Evaluation;
 pub use policy::Policy;
+pub use rule::NetworkRuleProtocol;
 pub use rule::Rule;
 pub use rule::RuleMatch;
 pub use rule::RuleRef;
