@@ -1,7 +1,6 @@
 use crate::admin;
 use crate::config;
 use crate::http_proxy;
-use crate::metadata::proxy_username_for_attempt_id;
 use crate::network_policy::NetworkPolicyDecider;
 use crate::runtime::BlockedRequestObserver;
 use crate::runtime::unix_socket_permissions_supported;
@@ -207,6 +206,10 @@ impl NetworkProxyBuilder {
             socks_addr,
             socks_enabled: current_cfg.network.enable_socks5,
             allow_local_binding: current_cfg.network.allow_local_binding,
+            allow_unix_sockets: current_cfg.network.allow_unix_sockets.clone(),
+            dangerously_allow_all_unix_sockets: current_cfg
+                .network
+                .dangerously_allow_all_unix_sockets,
             admin_addr,
             reserved_listeners,
             policy_decider: self.policy_decider,
@@ -241,6 +244,8 @@ pub struct NetworkProxy {
     socks_addr: SocketAddr,
     socks_enabled: bool,
     allow_local_binding: bool,
+    allow_unix_sockets: Vec<String>,
+    dangerously_allow_all_unix_sockets: bool,
     admin_addr: SocketAddr,
     reserved_listeners: Option<Arc<ReservedListeners>>,
     policy_decider: Option<Arc<dyn NetworkPolicyDecider>>,
@@ -338,12 +343,8 @@ fn apply_proxy_env_overrides(
     socks_addr: SocketAddr,
     socks_enabled: bool,
     allow_local_binding: bool,
-    network_attempt_id: Option<&str>,
 ) {
-    let http_proxy_url = network_attempt_id
-        .map(proxy_username_for_attempt_id)
-        .map(|username| format!("http://{username}@{http_addr}"))
-        .unwrap_or_else(|| format!("http://{http_addr}"));
+    let http_proxy_url = format!("http://{http_addr}");
     let socks_proxy_url = format!("socks5h://{socks_addr}");
     env.insert(
         ALLOW_LOCAL_BINDING_ENV_KEY.to_string(),
@@ -390,9 +391,7 @@ fn apply_proxy_env_overrides(
     // Keep HTTP_PROXY/HTTPS_PROXY as HTTP endpoints. A lot of clients break if
     // those vars contain SOCKS URLs. We only switch ALL_PROXY here.
     //
-    // For attempt-scoped runs, point ALL_PROXY at the HTTP proxy URL so the
-    // attempt metadata survives in proxy credentials for correlation.
-    if socks_enabled && network_attempt_id.is_none() {
+    if socks_enabled {
         set_env_keys(env, ALL_PROXY_ENV_KEYS, &socks_proxy_url);
         set_env_keys(env, FTP_PROXY_ENV_KEYS, &socks_proxy_url);
     } else {
@@ -426,15 +425,27 @@ impl NetworkProxy {
         self.admin_addr
     }
 
-    pub fn apply_to_env(&self, env: &mut HashMap<String, String>) {
-        self.apply_to_env_for_attempt(env, None);
+    pub async fn add_allowed_domain(&self, host: &str) -> Result<()> {
+        self.state.add_allowed_domain(host).await
     }
 
-    pub fn apply_to_env_for_attempt(
-        &self,
-        env: &mut HashMap<String, String>,
-        network_attempt_id: Option<&str>,
-    ) {
+    pub async fn add_denied_domain(&self, host: &str) -> Result<()> {
+        self.state.add_denied_domain(host).await
+    }
+
+    pub fn allow_local_binding(&self) -> bool {
+        self.allow_local_binding
+    }
+
+    pub fn allow_unix_sockets(&self) -> &[String] {
+        &self.allow_unix_sockets
+    }
+
+    pub fn dangerously_allow_all_unix_sockets(&self) -> bool {
+        self.dangerously_allow_all_unix_sockets
+    }
+
+    pub fn apply_to_env(&self, env: &mut HashMap<String, String>) {
         // Enforce proxying for child processes. We intentionally override existing values so
         // command-level environment cannot bypass the managed proxy endpoint.
         apply_proxy_env_overrides(
@@ -443,7 +454,6 @@ impl NetworkProxy {
             self.socks_addr,
             self.socks_enabled,
             self.allow_local_binding,
-            network_attempt_id,
         );
     }
 
@@ -457,7 +467,9 @@ impl NetworkProxy {
         ensure_rustls_crypto_provider();
 
         if !unix_socket_permissions_supported() {
-            warn!("allowUnixSockets is macOS-only; requests will be rejected on this platform");
+            warn!(
+                "allowUnixSockets and dangerouslyAllowAllUnixSockets are macOS-only; requests will be rejected on this platform"
+            );
         }
 
         let reserved_listeners = self.reserved_listeners.as_ref();
@@ -751,7 +763,6 @@ mod tests {
             SocketAddr::new(IpAddr::V4(Ipv4Addr::LOCALHOST), 8081),
             true,
             false,
-            None,
         );
 
         assert_eq!(
@@ -802,7 +813,6 @@ mod tests {
             SocketAddr::new(IpAddr::V4(Ipv4Addr::LOCALHOST), 8081),
             false,
             true,
-            None,
         );
 
         assert_eq!(
@@ -813,7 +823,7 @@ mod tests {
     }
 
     #[test]
-    fn apply_proxy_env_overrides_embeds_attempt_id_in_http_proxy_url() {
+    fn apply_proxy_env_overrides_uses_plain_http_proxy_url() {
         let mut env = HashMap::new();
         apply_proxy_env_overrides(
             &mut env,
@@ -821,28 +831,27 @@ mod tests {
             SocketAddr::new(IpAddr::V4(Ipv4Addr::LOCALHOST), 8081),
             true,
             false,
-            Some("attempt-123"),
         );
 
         assert_eq!(
             env.get("HTTP_PROXY"),
-            Some(&"http://codex-net-attempt-attempt-123@127.0.0.1:3128".to_string())
+            Some(&"http://127.0.0.1:3128".to_string())
         );
         assert_eq!(
             env.get("HTTPS_PROXY"),
-            Some(&"http://codex-net-attempt-attempt-123@127.0.0.1:3128".to_string())
+            Some(&"http://127.0.0.1:3128".to_string())
         );
         assert_eq!(
             env.get("WS_PROXY"),
-            Some(&"http://codex-net-attempt-attempt-123@127.0.0.1:3128".to_string())
+            Some(&"http://127.0.0.1:3128".to_string())
         );
         assert_eq!(
             env.get("WSS_PROXY"),
-            Some(&"http://codex-net-attempt-attempt-123@127.0.0.1:3128".to_string())
+            Some(&"http://127.0.0.1:3128".to_string())
         );
         assert_eq!(
             env.get("ALL_PROXY"),
-            Some(&"http://codex-net-attempt-attempt-123@127.0.0.1:3128".to_string())
+            Some(&"socks5h://127.0.0.1:8081".to_string())
         );
         #[cfg(target_os = "macos")]
         assert_eq!(
@@ -867,7 +876,6 @@ mod tests {
             SocketAddr::new(IpAddr::V4(Ipv4Addr::LOCALHOST), 8081),
             true,
             false,
-            None,
         );
 
         assert_eq!(
