@@ -12,6 +12,7 @@ use super::ThreadMetadata;
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct Stage1Output {
     pub thread_id: ThreadId,
+    pub rollout_path: PathBuf,
     pub source_updated_at: DateTime<Utc>,
     pub raw_memory: String,
     pub rollout_summary: String,
@@ -20,9 +21,25 @@ pub struct Stage1Output {
     pub generated_at: DateTime<Utc>,
 }
 
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct Stage1OutputRef {
+    pub thread_id: ThreadId,
+    pub source_updated_at: DateTime<Utc>,
+    pub rollout_slug: Option<String>,
+}
+
+#[derive(Debug, Clone, PartialEq, Eq, Default)]
+pub struct Phase2InputSelection {
+    pub selected: Vec<Stage1Output>,
+    pub previous_selected: Vec<Stage1Output>,
+    pub retained_thread_ids: Vec<ThreadId>,
+    pub removed: Vec<Stage1OutputRef>,
+}
+
 #[derive(Debug)]
 pub(crate) struct Stage1OutputRow {
     thread_id: String,
+    rollout_path: String,
     source_updated_at: i64,
     raw_memory: String,
     rollout_summary: String,
@@ -35,6 +52,7 @@ impl Stage1OutputRow {
     pub(crate) fn try_from_row(row: &SqliteRow) -> Result<Self> {
         Ok(Self {
             thread_id: row.try_get("thread_id")?,
+            rollout_path: row.try_get("rollout_path")?,
             source_updated_at: row.try_get("source_updated_at")?,
             raw_memory: row.try_get("raw_memory")?,
             rollout_summary: row.try_get("rollout_summary")?,
@@ -51,6 +69,7 @@ impl TryFrom<Stage1OutputRow> for Stage1Output {
     fn try_from(row: Stage1OutputRow) -> std::result::Result<Self, Self::Error> {
         Ok(Self {
             thread_id: ThreadId::try_from(row.thread_id)?,
+            rollout_path: PathBuf::from(row.rollout_path),
             source_updated_at: epoch_seconds_to_datetime(row.source_updated_at)?,
             raw_memory: row.raw_memory,
             rollout_summary: row.rollout_summary,
@@ -66,6 +85,18 @@ fn epoch_seconds_to_datetime(secs: i64) -> Result<DateTime<Utc>> {
         .ok_or_else(|| anyhow::anyhow!("invalid unix timestamp: {secs}"))
 }
 
+pub(crate) fn stage1_output_ref_from_parts(
+    thread_id: String,
+    source_updated_at: i64,
+    rollout_slug: Option<String>,
+) -> Result<Stage1OutputRef> {
+    Ok(Stage1OutputRef {
+        thread_id: ThreadId::try_from(thread_id)?,
+        source_updated_at: epoch_seconds_to_datetime(source_updated_at)?,
+        rollout_slug,
+    })
+}
+
 /// Result of trying to claim a stage-1 memory extraction job.
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum Stage1JobClaimOutcome {
