@@ -5,7 +5,7 @@ use crate::app_event::AppEvent;
 use crate::app_event_sender::AppEventSender;
 use crate::bottom_pane::MentionBinding;
 use crate::mention_codec::decode_history_mentions;
-use codex_core::protocol::Op;
+use codex_protocol::protocol::Op;
 use codex_protocol::user_input::TextElement;
 
 /// A composer history entry that can rehydrate draft state.
@@ -17,6 +17,8 @@ pub(crate) struct HistoryEntry {
     pub(crate) text_elements: Vec<TextElement>,
     /// Local image paths captured alongside `text_elements`.
     pub(crate) local_image_paths: Vec<PathBuf>,
+    /// Remote image URLs restored with this draft.
+    pub(crate) remote_image_urls: Vec<String>,
     /// Mention bindings for tool/app/skill references inside `text`.
     pub(crate) mention_bindings: Vec<MentionBinding>,
     /// Placeholder-to-payload pairs used to restore large paste content.
@@ -30,6 +32,7 @@ impl HistoryEntry {
             text: decoded.text,
             text_elements: Vec::new(),
             local_image_paths: Vec::new(),
+            remote_image_urls: Vec::new(),
             mention_bindings: decoded
                 .mentions
                 .into_iter()
@@ -53,6 +56,25 @@ impl HistoryEntry {
             text,
             text_elements,
             local_image_paths,
+            remote_image_urls: Vec::new(),
+            mention_bindings: Vec::new(),
+            pending_pastes,
+        }
+    }
+
+    #[cfg(test)]
+    pub(crate) fn with_pending_and_remote(
+        text: String,
+        text_elements: Vec<TextElement>,
+        local_image_paths: Vec<PathBuf>,
+        pending_pastes: Vec<(String, String)>,
+        remote_image_urls: Vec<String>,
+    ) -> Self {
+        Self {
+            text,
+            text_elements,
+            local_image_paths,
+            remote_image_urls,
             mention_bindings: Vec::new(),
             pending_pastes,
         }
@@ -70,7 +92,7 @@ pub(crate) struct ChatComposerHistory {
     history_entry_count: usize,
 
     /// Messages submitted by the user *during this UI session* (newest at END).
-    /// Local entries retain full draft state (text elements, image paths, pending pastes).
+    /// Local entries retain full draft state (text elements, image paths, pending pastes, remote image URLs).
     local_history: Vec<HistoryEntry>,
 
     /// Cache of persistent history entries fetched on-demand (text-only).
@@ -115,6 +137,7 @@ impl ChatComposerHistory {
         if entry.text.is_empty()
             && entry.text_elements.is_empty()
             && entry.local_image_paths.is_empty()
+            && entry.remote_image_urls.is_empty()
             && entry.mention_bindings.is_empty()
             && entry.pending_pastes.is_empty()
         {
@@ -269,7 +292,7 @@ impl ChatComposerHistory {
 mod tests {
     use super::*;
     use crate::app_event::AppEvent;
-    use codex_core::protocol::Op;
+    use codex_protocol::protocol::Op;
     use pretty_assertions::assert_eq;
     use tokio::sync::mpsc::unbounded_channel;
 
