@@ -19,7 +19,7 @@ use crate::app_event::FeedbackCategory;
 use crate::app_event_sender::AppEventSender;
 use crate::history_cell;
 use crate::render::renderable::Renderable;
-use codex_core::protocol::SessionSource;
+use codex_protocol::protocol::SessionSource;
 
 use super::CancellationEvent;
 use super::bottom_pane_view::BottomPaneView;
@@ -87,7 +87,11 @@ impl FeedbackNoteView {
         } else {
             Some(note.as_str())
         };
-        let rollout_path_ref = self.rollout_path.as_deref();
+        let log_file_paths = if self.include_logs {
+            self.rollout_path.iter().cloned().collect::<Vec<_>>()
+        } else {
+            Vec::new()
+        };
         let classification = feedback_classification(self.category);
 
         let mut thread_id = self.snapshot.thread_id.clone();
@@ -96,11 +100,7 @@ impl FeedbackNoteView {
             classification,
             reason_opt,
             self.include_logs,
-            if self.include_logs {
-                rollout_path_ref
-            } else {
-                None
-            },
+            &log_file_paths,
             Some(SessionSource::Cli),
         );
 
@@ -129,7 +129,7 @@ impl FeedbackNoteView {
                             Line::from("  Share this and add some info about your problem:"),
                             Line::from(vec![
                                 "    ".into(),
-                                format!("go/codex-feedback/{thread_id}").bold(),
+                                format!("https://go/codex-feedback/{thread_id}").bold(),
                             ]),
                         ]);
                     }
@@ -353,6 +353,10 @@ fn feedback_title_and_placeholder(category: FeedbackCategory) -> (String, String
             "Tell us more (bug)".to_string(),
             "(optional) Write a short description to help us further".to_string(),
         ),
+        FeedbackCategory::SafetyCheck => (
+            "Tell us more (safety check)".to_string(),
+            "(optional) Share what was refused and why it should have been allowed".to_string(),
+        ),
         FeedbackCategory::Other => (
             "Tell us more (other)".to_string(),
             "(optional) Write a short description to help us further".to_string(),
@@ -365,6 +369,7 @@ fn feedback_classification(category: FeedbackCategory) -> &'static str {
         FeedbackCategory::BadResult => "bad_result",
         FeedbackCategory::GoodResult => "good_result",
         FeedbackCategory::Bug => "bug",
+        FeedbackCategory::SafetyCheck => "safety_check",
         FeedbackCategory::Other => "other",
     }
 }
@@ -378,14 +383,15 @@ fn issue_url_for_category(
     // the external GitHub behavior identical while routing internal users to
     // the internal go link.
     match category {
-        FeedbackCategory::Bug | FeedbackCategory::BadResult | FeedbackCategory::Other => {
-            Some(match feedback_audience {
-                FeedbackAudience::OpenAiEmployee => slack_feedback_url(thread_id),
-                FeedbackAudience::External => {
-                    format!("{BASE_BUG_ISSUE_URL}&steps=Uploaded%20thread:%20{thread_id}")
-                }
-            })
-        }
+        FeedbackCategory::Bug
+        | FeedbackCategory::BadResult
+        | FeedbackCategory::SafetyCheck
+        | FeedbackCategory::Other => Some(match feedback_audience {
+            FeedbackAudience::OpenAiEmployee => slack_feedback_url(thread_id),
+            FeedbackAudience::External => {
+                format!("{BASE_BUG_ISSUE_URL}&steps=Uploaded%20thread:%20{thread_id}")
+            }
+        }),
         FeedbackCategory::GoodResult => None,
     }
 }
@@ -423,6 +429,12 @@ pub(crate) fn feedback_selection_params(
                 "Helpful, correct, high‑quality, or delightful result worth celebrating.",
                 FeedbackCategory::GoodResult,
             ),
+            make_feedback_item(
+                app_event_tx.clone(),
+                "safety check",
+                "Benign usage blocked due to safety checks or refusals.",
+                FeedbackCategory::SafetyCheck,
+            ),
             make_feedback_item(
                 app_event_tx,
                 "other",
@@ -616,7 +628,14 @@ mod tests {
     }
 
     #[test]
-    fn issue_url_available_for_bug_bad_result_and_other() {
+    fn feedback_view_safety_check() {
+        let view = make_view(FeedbackCategory::SafetyCheck);
+        let rendered = render(&view, 60);
+        insta::assert_snapshot!("feedback_view_safety_check", rendered);
+    }
+
+    #[test]
+    fn issue_url_available_for_bug_bad_result_safety_check_and_other() {
         let bug_url = issue_url_for_category(
             FeedbackCategory::Bug,
             "thread-1",
@@ -639,6 +658,13 @@ mod tests {
         );
         assert!(other_url.is_some());
 
+        let safety_check_url = issue_url_for_category(
+            FeedbackCategory::SafetyCheck,
+            "thread-4",
+            FeedbackAudience::OpenAiEmployee,
+        );
+        assert!(safety_check_url.is_some());
+
         assert!(
             issue_url_for_category(
                 FeedbackCategory::GoodResult,
