@@ -2,9 +2,16 @@ use std::fmt;
 use std::io;
 use std::io::Write;
 
-use crate::wrapping::word_wrap_lines_borrowed;
+use crate::wrapping::RtOptions;
+use crate::wrapping::adaptive_wrap_line;
+use crate::wrapping::line_contains_url_like;
+use crate::wrapping::line_has_mixed_url_and_non_url_tokens;
 use crossterm::Command;
+use crossterm::cursor::MoveDown;
 use crossterm::cursor::MoveTo;
+use crossterm::cursor::MoveToColumn;
+use crossterm::cursor::RestorePosition;
+use crossterm::cursor::SavePosition;
 use crossterm::queue;
 use crossterm::style::Color as CColor;
 use crossterm::style::Colors;
@@ -38,10 +45,35 @@ where
     let last_cursor_pos = terminal.last_known_cursor_pos;
     let writer = terminal.backend_mut();
 
-    // Pre-wrap lines using word-aware wrapping so terminal scrollback sees the same
-    // formatting as the TUI. This avoids character-level hard wrapping by the terminal.
-    let wrapped = word_wrap_lines_borrowed(&lines, area.width.max(1) as usize);
-    let wrapped_lines = wrapped.len() as u16;
+    // Pre-wrap lines for terminal scrollback. Three paths:
+    //
+    // - URL-only-ish lines are kept intact (no hard newlines inserted) so that
+    //   terminal emulators can match them as clickable links. The
+    //   terminal will character-wrap these lines at the viewport
+    //   boundary.
+    // - Mixed lines (URL + non-URL prose) are adaptively wrapped so
+    //   non-URL text still wraps naturally while URL tokens remain
+    //   unsplit.
+    // - Non-URL lines also flow through adaptive wrapping; behavior is
+    //   equivalent to standard wrapping when no URL is present.
+    let wrap_width = area.width.max(1) as usize;
+    let mut wrapped = Vec::new();
+    let mut wrapped_rows = 0usize;
+
+    for line in &lines {
+        let line_wrapped =
+            if line_contains_url_like(line) && !line_has_mixed_url_and_non_url_tokens(line) {
+                vec![line.clone()]
+            } else {
+                adaptive_wrap_line(line, RtOptions::new(wrap_width))
+            };
+        wrapped_rows += line_wrapped
+            .iter()
+            .map(|wrapped_line| wrapped_line.width().max(1).div_ceil(wrap_width))
+            .sum::<usize>();
+        wrapped.extend(line_wrapped);
+    }
+    let wrapped_lines = wrapped_rows as u16;
     let cursor_top = if area.bottom() < screen_size.height {
         // If the viewport is not at the bottom of the screen, scroll it down to make room.
         // Don't scroll it past the bottom of the screen.
@@ -94,6 +126,18 @@ where
 
     for line in wrapped {
         queue!(writer, Print("\r\n"))?;
+        // URL lines can be wider than the terminal and will
+        // character-wrap onto continuation rows. Pre-clear those rows
+        // so stale content from a previously longer line is erased.
+        let physical_rows = line.width().max(1).div_ceil(wrap_width);
+        if physical_rows > 1 {
+            queue!(writer, SavePosition)?;
+            for _ in 1..physical_rows {
+                queue!(writer, MoveDown(1), MoveToColumn(0))?;
+                queue!(writer, Clear(ClearType::UntilNewLine))?;
+            }
+            queue!(writer, RestorePosition)?;
+        }
         queue!(
             writer,
             SetColors(Colors::new(
@@ -130,6 +174,9 @@ where
     if should_update_area {
         terminal.set_viewport_area(area);
     }
+    if wrapped_lines > 0 {
+        terminal.note_history_rows_inserted(wrapped_lines);
+    }
 
     Ok(())
 }
@@ -527,4 +574,163 @@ mod tests {
             );
         }
     }
+
+    #[test]
+    fn vt100_prefixed_url_keeps_prefix_and_url_on_same_row() {
+        let width: u16 = 48;
+        let height: u16 = 8;
+        let backend = VT100Backend::new(width, height);
+        let mut term = crate::custom_terminal::Terminal::with_options(backend).expect("terminal");
+        let viewport = Rect::new(0, height - 1, width, 1);
+        term.set_viewport_area(viewport);
+
+        let url = "http://a-long-url.com/this/that/blablablab/new.aspx/many_people_like_how";
+        let line: Line<'static> = Line::from(vec!["  │ ".into(), url.into()]);
+
+        insert_history_lines(&mut term, vec![line]).expect("insert history");
+
+        let rows: Vec<String> = term.backend().vt100().screen().rows(0, width).collect();
+
+        assert!(
+            rows.iter().any(|r| r.contains("│ http://a-long-url.com")),
+            "expected prefix and URL on same row, rows: {rows:?}"
+        );
+        assert!(
+            !rows.iter().any(|r| r.trim_end() == "│"),
+            "unexpected orphan prefix row, rows: {rows:?}"
+        );
+    }
+
+    #[test]
+    fn vt100_prefixed_url_like_without_scheme_keeps_prefix_and_token_on_same_row() {
+        let width: u16 = 48;
+        let height: u16 = 8;
+        let backend = VT100Backend::new(width, height);
+        let mut term = crate::custom_terminal::Terminal::with_options(backend).expect("terminal");
+        let viewport = Rect::new(0, height - 1, width, 1);
+        term.set_viewport_area(viewport);
+
+        let url_like =
+            "example.test/api/v1/projects/alpha-team/releases/2026-02-17/builds/1234567890";
+        let line: Line<'static> = Line::from(vec!["  │ ".into(), url_like.into()]);
+
+        insert_history_lines(&mut term, vec![line]).expect("insert history");
+
+        let rows: Vec<String> = term.backend().vt100().screen().rows(0, width).collect();
+
+        assert!(
+            rows.iter()
+                .any(|r| r.contains("│ example.test/api/v1/projects")),
+            "expected prefix and URL-like token on same row, rows: {rows:?}"
+        );
+        assert!(
+            !rows.iter().any(|r| r.trim_end() == "│"),
+            "unexpected orphan prefix row, rows: {rows:?}"
+        );
+    }
+
+    #[test]
+    fn vt100_prefixed_mixed_url_line_wraps_suffix_words_together() {
+        let width: u16 = 24;
+        let height: u16 = 10;
+        let backend = VT100Backend::new(width, height);
+        let mut term = crate::custom_terminal::Terminal::with_options(backend).expect("terminal");
+        let viewport = Rect::new(0, height - 1, width, 1);
+        term.set_viewport_area(viewport);
+
+        let url = "https://example.test/path/abcdef12345";
+        let line: Line<'static> = Line::from(vec![
+            "  │ ".into(),
+            "see ".into(),
+            url.into(),
+            " tail words".into(),
+        ]);
+
+        insert_history_lines(&mut term, vec![line]).expect("insert mixed history");
+
+        let rows: Vec<String> = term.backend().vt100().screen().rows(0, width).collect();
+        assert!(
+            rows.iter().any(|r| r.contains("│ see")),
+            "expected prefixed prose before URL, rows: {rows:?}"
+        );
+        assert!(
+            rows.iter().any(|r| r.contains("tail words")),
+            "expected suffix words to wrap as a phrase, rows: {rows:?}"
+        );
+    }
+
+    #[test]
+    fn vt100_unwrapped_url_like_clears_continuation_rows() {
+        let width: u16 = 20;
+        let height: u16 = 10;
+        let backend = VT100Backend::new(width, height);
+        let mut term = crate::custom_terminal::Terminal::with_options(backend).expect("terminal");
+        let viewport = Rect::new(0, height - 1, width, 1);
+        term.set_viewport_area(viewport);
+
+        let filler_line: Line<'static> = Line::from(vec![
+            "  │ ".into(),
+            "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX".into(),
+        ]);
+        insert_history_lines(&mut term, vec![filler_line]).expect("insert filler history");
+
+        let url_like = "example.test/api/v1/short";
+        let url_line: Line<'static> = Line::from(vec!["  │ ".into(), url_like.into()]);
+        insert_history_lines(&mut term, vec![url_line]).expect("insert url-like history");
+
+        let rows: Vec<String> = term.backend().vt100().screen().rows(0, width).collect();
+        let first_row = rows
+            .iter()
+            .position(|row| row.contains("│ example.test/api"))
+            .unwrap_or_else(|| panic!("expected url-like first row in screen rows: {rows:?}"));
+        assert!(
+            first_row + 1 < rows.len(),
+            "expected a continuation row for wrapped URL-like line, rows: {rows:?}"
+        );
+        let continuation_row = rows[first_row + 1].trim_end();
+
+        assert!(
+            continuation_row.contains("/v1/short") || continuation_row.contains("short"),
+            "expected continuation row to contain wrapped URL-like tail, got: {continuation_row:?}"
+        );
+        assert!(
+            !continuation_row.contains('X'),
+            "expected continuation row to be cleared before writing wrapped URL-like content, got: {continuation_row:?}"
+        );
+    }
+
+    #[test]
+    fn vt100_long_unwrapped_url_does_not_insert_extra_blank_gap_before_content() {
+        let width: u16 = 56;
+        let height: u16 = 24;
+        let backend = VT100Backend::new(width, height);
+        let mut term = crate::custom_terminal::Terminal::with_options(backend).expect("terminal");
+        let viewport = Rect::new(0, height - 1, width, 1);
+        term.set_viewport_area(viewport);
+
+        let prompt = "Write a long URL as output for testing";
+        insert_history_lines(&mut term, vec![Line::from(prompt)]).expect("insert prompt line");
+
+        let long_url = format!(
+            "https://example.test/api/v1/projects/alpha-team/releases/2026-02-17/builds/1234567890/{}",
+            "very-long-segment-".repeat(16),
+        );
+        let url_line: Line<'static> = Line::from(vec!["• ".into(), long_url.into()]);
+        insert_history_lines(&mut term, vec![url_line]).expect("insert long url line");
+
+        let rows: Vec<String> = term.backend().vt100().screen().rows(0, width).collect();
+        let prompt_row = rows
+            .iter()
+            .position(|row| row.contains("Write a long URL as output for testing"))
+            .unwrap_or_else(|| panic!("expected prompt row in screen rows: {rows:?}"));
+        let url_row = rows
+            .iter()
+            .position(|row| row.contains("• https://example.test/api"))
+            .unwrap_or_else(|| panic!("expected URL first row in screen rows: {rows:?}"));
+
+        assert!(
+            url_row <= prompt_row + 2,
+            "expected URL content to appear immediately after prompt (allowing at most one spacer row), got prompt_row={prompt_row}, url_row={url_row}, rows={rows:?}",
+        );
+    }
 }
