@@ -1,5 +1,5 @@
-use codex_common::fuzzy_match::fuzzy_match;
 use codex_core::skills::model::SkillMetadata;
+use codex_utils_fuzzy_match::fuzzy_match;
 
 use crate::text_formatting::truncate_text;
 
